#ifndef __IjQLBaseTest_hpp
#define __IjQLBaseTest_hpp
// \file IjQLBaseTest.hpp
// \par Description
// Interface class for the IjQLBaseTest class, for use by jEDI drivers. (Incomplete!)
//
// This interface forms the base class of the jQLBaseSel_Test and jQLBaseIID_Test heirarchies.
//
// *ALL* access to jQL objects directly, related to the jQLBaseSel_Test / jQLBaseIID_Test objects *MUST* occur
// within the context of these Interface classes.
//
// These classese *MUST ONLY* be changed on the authority of the jEDI Driver Team.
//

// Implementation in Phase II

/// \TODO: Perhaps each node should 'process' itself -- i.e. 'processTree / ProcessID / ProcessWITH 'replacements'??
/// \TODO: Probably means we require a new call-back interface into driver to process the WITHs (or anywhere else 'something' is really 
/// \TODO: done in the driver)??

// Comment in processTree(), thus:
// It would be nice to cater for all the different inherited classes in the same chunk of code
// i.e. theck the tokentype, get like patterns, etc. but the methods for used for getting the
// literals and like tests for IID and Sel aren't base class methods.
// We could at some point merge the switch(tokenType) logic and if test the specifics (i.e. IID/Sel/Lit/etc).
//
// So, so we do this here, now??? SRP: Not in Phase II -- maybe later ;-)

/// Forward Declarations (for constructors, temp & private access methods)
class jQLBaseIIDTest;
class jQLBaseSelTest;
class jQLAttrDefn;	// Should eventually be removed?
class IjQLAttrDefn;	// 
class CVar;

class IjQLTokenTypes
{
public:
	enum {
		// These are mostly getNodeType()*
		AND_TEST = 1,			// AND;					AND_
		OR_TEST,				// OR;					OR_
		WITH_TEST,				// WITH;				WITH_, Null_
		LIKE_TEST,				// LIKE;				LIKE_
		UNLIKE_TEST,			// UNLIKE;				UNLIKE_
		WHEN_TEST,				// WHEN;				WHEN_
		QUOTED_STRING,			// QuotedString;		Lit_, Num_
		ATTRIBUTE_VALUE,		// AttributeValue;		Attr_
		RIGHT_IGNORE_STRING,	// RightIgnoreString;	RI_
		LEFT_IGNORE_STRING,		// LeftIgnoreString;	LI_
		LR_IGNORE_STRING,		// LRIgnoreString;		LRI_
		COMPLEX_WILD_STRING,	// ComplexWildString;	CW_
		// These are mostly getCompareType()*
		// *Except IID classes, which seem be confused and use the latter for the former!!
		NO_NULLS_VALUE = 100,	// NO_NULLS;
		EQ_TEST,				// EQ;
		NE_TEST,				// NQ;
		LT_TEST,				// LTOP;
		LE_TEST,				// LE;		
		GT_TEST,				// GT;		
		GE_TEST,				// GE;
		IID_SELECTION_TEST		// IIDSelection ; 
	};

//	static const IjQLTokenTypes convert(jQLParserTokenTypes &in) const;  //C2272: modifiers not allowed on static member functions
	static const int convert(int in);
};

// Pointer to logging funtion in Driver (for dump routines etc.)
typedef void (logfn_t)(DPSTRUCT * dp, void * driverData_ptr, void * fileData_ptr, char *str);

/// Pure Virtual Base for IjQLBaseIIDTest and IjQLBaseSelTest derivatives
/// We've refactored the 'jQL' originals to have a common base type,
/// and may be able to do more here in the future to ease the 
/// dynamic casting / testing that takes place with the jQL ones.
class IjQLBaseTest
{
public:
	// No default construction -- must be instantiated with exsiting jQL object. (doesn't work private?)
	IjQLBaseTest();	// :_pParent(0), _pLeft(0), _pRight(0) {};
	IjQLBaseTest(IjQLBaseTest* parent, IjQLBaseTest* left, IjQLBaseTest* right);
	virtual ~IjQLBaseTest();

	IjQLBaseTest(void *intest);
//	IjQLBaseTest(jQLBaseIIDTest *intest);
//	IjQLBaseTest(jQLBaseSelTest *intest);
	virtual int getNodeType() const = 0; // *MUST* be overridden by concrete derived types!
// \todo:	virtual const int getCompareType() = 0; // *MUST* be overridden by approprate concrete derived types!
// \todo:	virtual std::string* getLiteral() = 0; // *MUST* be overridden by approprate concrete derived types!
	virtual void DisableTest() = 0;
	virtual void EnableTest() = 0;
	virtual IjQLBaseTest* getLeft() { return _pLeft; };
	virtual IjQLBaseTest* getRight() { return _pRight; };
	virtual void setParent(IjQLBaseTest* in) { _pParent = in; };
	virtual void dumpTree(DPSTRUCT * dp, void * driverData_ptr, void * fileData_ptr, int level, logfn_t *plogfn);
	virtual const int getCompareType() const = 0;

private:

private:
	IjQLBaseTest* _pParent;
	IjQLBaseTest* _pLeft;
	IjQLBaseTest* _pRight;
};

/////////////////////////////////////////////////////////////////////////////
class IjQLBaseIIDTest;

//
class IjQLBaseIIDTest : public IjQLBaseTest
{
public:
	//IjQLBaseIIDTest(jQLBaseIIDTest* in);
	IjQLBaseIIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent = 0, IjQLBaseIIDTest* left = 0, IjQLBaseIIDTest* right = 0);
	virtual ~IjQLBaseIIDTest(){};
	virtual std::string * getLiteral() const;
	const int getCompareType() const;
	virtual int getNodeType() const = 0;
	virtual void DisableTest();
	virtual void EnableTest();
	virtual IjQLBaseIIDTest* getLeft() { return static_cast<IjQLBaseIIDTest*>(IjQLBaseTest::getLeft()); };
	virtual IjQLBaseIIDTest* getRight() { return static_cast<IjQLBaseIIDTest*>(IjQLBaseTest::getRight()); };

	virtual jQLBaseIIDTest *TEMP_getBase(void);
	virtual void * getPatterns() const;


protected:
	jQLBaseIIDTest* _pjQLBaseIIDTest;
};

class IjQL_AND_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_AND_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);

	//AND_IIDTest(){};
	//AND_IIDTest(jQLBaseIIDTest* _left,jQLBaseIIDTest* _right):left(_left),right(_right){};
	//~AND_IIDTest()
	//{
	//	delete left;
	//	delete right;
	//};

	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID)
	//{
	//	return (left->Accept(ItemID) && right->Accept(ItemID));
	//}

	//jQLBaseIIDTest* getLeft() { return left; }
	//jQLBaseIIDTest* getRight() { return right; }

	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::AND_TEST; };

private:
};

class IjQL_OR_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_OR_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//OR_IIDTest(){};
	//OR_IIDTest(jQLBaseIIDTest* _left,jQLBaseIIDTest* _right):left(_left),right(_right){};
	//~OR_IIDTest()
	//{
	//	delete left;
	//	delete right;
	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID)
	//{
	//	return (left->Accept(ItemID) || right->Accept(ItemID));
	//}

	//jQLBaseIIDTest* getLeft() { return left; }
	//jQLBaseIIDTest* getRight() { return right; }

	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::OR_TEST; };

private:
};

class IjQL_WITH_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_WITH_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);

	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::WITH_TEST; };

private:
};

///////////
class IjQL_Lit_IIDTest : public IjQLBaseIIDTest
{
public:
//	IjQL_Lit_IIDTest(){};
	IjQL_Lit_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
//	~IjQL_Lit_IIDTest(){};

	//const int getCompareType() const;
	/// TODO: Implement this (not implemented in jQL!)??
	int getNodeType() const { return IjQLTokenTypes::QUOTED_STRING; };

//	Lit_IIDTest(ANTLR_USE_NAMESPACE(std)string _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype){};

//	bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID)
//	{
//		return checkresult(ItemID.compare(literalstring),comparetype);
//	}

	//virtual std::string * getLiteral() const;

//	virtual void setLiteral(std::string & literal) { literal = literalstring.c_str(); }
//	int getCompareType() { return comparetype; }

private:
//	int comparetype;
};

//
// right justified compare
//
class IjQL_Num_IIDTest : public IjQLBaseIIDTest
{
public:
//	IjQL_Num_IIDTest(){};	/// TODO: error C2512: 'IjQLBaseIIDTest' : no appropriate default constructor available
	IjQL_Num_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//~IjQL_Num_IIDTest(){};
	//Num_IIDTest(ANTLR_USE_NAMESPACE(std)string _literalstring,int _comparetype ):comparetype(_comparetype)
	//{
	//	ConstVar = _literalstring.c_str();
	//	if (JRunDGetNumeric(dp, &ConstVar , GET_NUMERIC_ALLOW_BCD , &constvalue) == 1 )
	//	{
	//		usedouble = true;
	//	}
	//	else
	//	{
	//		usedouble = false;
	//	}

	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID);

	//virtual std::string * getLiteral() { return &literalstring; }
	//virtual void setLiteral(std::string & literal) { literal = literalstring.c_str(); }
	//int getCompareType() { return comparetype; }
	int getNodeType() const { return IjQLTokenTypes::QUOTED_STRING; };

private:
	//ANTLR_USE_NAMESPACE(std)string literalstring; 
	//double constvalue;
	//int comparetype;
	//CVar ConstVar;
	//bool usedouble;
};

// right ignore - "aaa]"
class IjQL_RI_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_RI_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//RI_IIDTest(){};
	//RI_IIDTest(ANTLR_USE_NAMESPACE(std)string _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	literallength = literalstring.length();
	//};

	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID);

	//virtual std::string * getLiteral() { return &literalstring; }
	//virtual void setLiteral(std::string & literal) 
	//{
	//	literal = literalstring.c_str();
	//	literal += "]";
	//}
	//int getCompareType() { return comparetype; }
	int getNodeType() const { return IjQLTokenTypes::RIGHT_IGNORE_STRING; };

private:
	//ANTLR_USE_NAMESPACE(std)string literalstring; 
	//int literallength;
	//int comparetype;
};

// Left ignore  - "[aaa"
class IjQL_LI_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_LI_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//LI_IIDTest(){};
	//~LI_IIDTest(){};
	//LI_IIDTest(ANTLR_USE_NAMESPACE(std)string _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	literallength = literalstring.length();
	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID);
	//virtual std::string * getLiteral() { return &literalstring; }
	//virtual void setLiteral(std::string & literal) 
	//{
	//	literal = "[";
	//	literal += literalstring.c_str();
	//}
	//int getCompareType() { return comparetype; }
	int getNodeType() const { return IjQLTokenTypes::LEFT_IGNORE_STRING; };

private:
	//ANTLR_USE_NAMESPACE(std)string literalstring; 
	//int literallength;
	//int comparetype;
};

// Left and right ignore  - "[aaa]"  aka contains substring 
class IjQL_LRI_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_LRI_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//LRI_IIDTest(){};
	//~LRI_IIDTest(){};

	//LRI_IIDTest(ANTLR_USE_NAMESPACE(std)string _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	literallength = literalstring.length();
	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID);

	//virtual std::string * getLiteral() { return &literalstring; }
	//virtual void setLiteral(std::string & literal) 
	//{
	//	literal = literalstring.c_str();
	//}
	//int getCompareType() { return comparetype; }
	int getNodeType() const { return IjQLTokenTypes::LR_IGNORE_STRING; };

private:
	//ANTLR_USE_NAMESPACE(std)string literalstring; 
	//int literallength;
	//int comparetype;
};

// A complex pattern containing embedded ']' and/or '^'
class IjQL_CW_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_CW_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//CW_IIDTest(){};
	//~CW_IIDTest(){JLibEReleaseMATCHES(dp, Patterns);}
	//CW_IIDTest(ANTLR_USE_NAMESPACE(std)string _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	if ( JLibEParsePATTERN(dp, &Patterns,(unsigned char*) literalstring.c_str(), literalstring.size() ) == 0)
	//		throw jError(JBASE_E_BAD_LIKE_PATTERN);  // how did it get here ???
	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID)
	//{
	//	CVar IIDVar(ItemID);
	//	if (comparetype == jQLParserTokenTypes::EQ)
	//	{
	//		return JLibETestMATCHES(dp, Patterns, IIDVar, IIDVar.Length() );
	//	}
	//	else
	//	{
	//		return !JLibETestMATCHES(dp, Patterns, IIDVar, IIDVar.Length() );
	//	}
	//};

	//virtual std::string * getLiteral() { return &literalstring; }
	//virtual void setLiteral(std::string & literal) { literal = literalstring.c_str(); }
	//int getCompareType() { return comparetype; }
	//void * getPatterns() const;
	int getNodeType() const { return IjQLTokenTypes::COMPLEX_WILD_STRING; };

private:
	//ANTLR_USE_NAMESPACE(std)string literalstring; 
	//int comparetype;
};

class IjQL_LIKE_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_LIKE_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//LIKE_IIDTest();
	//~LIKE_IIDTest(){JLibEReleaseMATCHES(dp, Patterns);}
	//LIKE_IIDTest(CVar& literalstring )
	//{
	//	if (JLibEParseMATCHES(dp, &Patterns, literalstring, literalstring.Length(), 1 ) == 0)
	//		throw jError(JBASE_E_BAD_LIKE_PATTERN);  // how did it get here ???
	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID)
	//{
	//	CVar IIDVar(ItemID);
	//	return JLibETestMATCHES(dp, Patterns, IIDVar, IIDVar.Length() );
	//};

	//void * getPatterns() const;
	//int getCompareType() { return jQLParserTokenTypes::LIKE; }
	int getNodeType() const { return IjQLTokenTypes::LIKE_TEST; };

private:
};

class IjQL_UNLIKE_IIDTest : public IjQLBaseIIDTest
{
public:
	IjQL_UNLIKE_IIDTest(jQLBaseIIDTest* in, IjQLBaseIIDTest* parent, IjQLBaseIIDTest *left, IjQLBaseIIDTest *right);
	//UNLIKE_IIDTest();
	//~UNLIKE_IIDTest(){JLibEReleaseMATCHES(dp, Patterns);}
	//UNLIKE_IIDTest(CVar& literalstring )
	//{
	//	if (JLibEParseMATCHES(dp, &Patterns, literalstring, literalstring.Length(), 1 ) == 0)
	//		throw jError(JBASE_E_BAD_LIKE_PATTERN);  // how did it get here ???
	//};
	//bool Accept(const ANTLR_USE_NAMESPACE(std)string& ItemID)
	//{
	//	CVar IIDVar(ItemID);
	//	return !(JLibETestMATCHES(dp, Patterns, IIDVar, IIDVar.Length()) );
	//};

	//void * getPatterns() const;
	//int getCompareType() { return jQLParserTokenTypes::UNLIKE; }
	int getNodeType() const { return IjQLTokenTypes::UNLIKE_TEST; }

private:
};


/////////////////////////////////////////////////////////////////////////////
class IjQLBaseSelTest;

//
class IjQLBaseSelTest : public IjQLBaseTest
{
public:
//	IjQLBaseSelTest();
	IjQLBaseSelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent = 0, IjQLBaseSelTest *left = 0, IjQLBaseSelTest *right = 0);
//	virtual ~IjQLBaseSelTest(){};
	virtual const int getCompareType() const;
	virtual int getNodeType() const = 0;
	virtual void DisableTest();
	virtual void EnableTest();
	virtual IjQLBaseSelTest* getLeft() { return static_cast<IjQLBaseSelTest*>(IjQLBaseTest::getLeft()); };
	virtual IjQLBaseSelTest* getRight() { return static_cast<IjQLBaseSelTest*>(IjQLBaseTest::getRight()); };

	/// \todo: Ultimately should be factored out!
	JBUILD_EXPORT virtual jQLBaseSelTest *TEMP_getBase(void);

protected:
	jQLBaseSelTest* _pjQLBaseSelTest;
};


class IjQL_AND_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_AND_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	int getNodeType() const { return IjQLTokenTypes::AND_TEST; };
private:
};

class IjQL_OR_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_OR_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	int getNodeType() const { return IjQLTokenTypes::OR_TEST; };

private:
};

class IjQL_WITH_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_WITH_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	virtual ~IjQL_WITH_SelTest();

	int getNodeType() const { return IjQLTokenTypes::WITH_TEST; };

	/// \todo: Ultimately should return a IjQLAttrDefn type.
	JBUILD_EXPORT jQLAttrDefn  *TEMP_getAttributeDefn();
	// Like this ;-) :
	JBUILD_EXPORT IjQLAttrDefn *getAttributeDefn();

	JBUILD_EXPORT bool getNotFlag(void);
	JBUILD_EXPORT bool getEachFlag(void);
private:
	IjQLAttrDefn *attrDefn;
};

class IjQL_WHEN_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_WHEN_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//IjQL_WHEN_SelTest(){};
	virtual ~IjQL_WHEN_SelTest();
	//{
	//	delete testingTree;
	//};
	/// \todo: Ultimately should return a IjQLAttrDefn type. (ORA only?)
	JBUILD_EXPORT jQLAttrDefn		*TEMP_getAttributeDefn();
	// Like this ;-) :
	JBUILD_EXPORT IjQLAttrDefn *getAttributeDefn();

	//WHEN_SelTest(ANTLR_USE_NAMESPACE(antlr)RefAST WhenAttrAST,jQLItemAccess* _DataSource, jQLBaseSelTest* _parent);
	//bool Accept();
	//bool Accept(CVar& Source,CVar* Result, CVar *NotResults); // This version of Accept is for display limiting only.
	//bool AcceptWhenAttrNo(int Attr, CVar& Source, CVar* Result, CVar *NotResult, bool limitFlag );
	//		
	//bool Not;
	//bool Each;
	//int GetHighestAttributeUsed(int highestSoFar);
	//int attributeHasEmbededConversion();
	//void lookForIndex();
	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::WHEN_TEST;}
	//jQLBaseSelTest* getLeft() { return testingTree; }
	//jQLBaseSelTest* getParent() { return parent; }
	private:
	//jQLAttrDefn* attrDefn;
	IjQLAttrDefn *attrDefn;
	//jQLBaseSelTest* testingTree;
	//jQLBaseSelTest* parent;
	//jQLItemAccess* DataSource;
};

// Compare to another attribute
class IjQL_Attr_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_Attr_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//Attr_SelTest(){};
	virtual ~IjQL_Attr_SelTest();

	//Attr_SelTest(jQLAttrDefn* _attrDefn,int _comparetype,int _LHSJust ):attrDefn( _attrDefn),comparetype(_comparetype),LHSJust(_LHSJust)
	//{
	//};
	//virtual bool PreAccept(CVar& CurrentValue);
	//bool Accept(CVar& CurrentValue);
	//int GetHighestAttributeUsed(int highestSoFar)
	//{ return attrDefn->getAttrNo() > highestSoFar ? attrDefn->getAttrNo():highestSoFar; } 

	//int attributeHasEmbededConversion();
	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::ATTRIBUTE_VALUE;}

	/// \todo: Ultimately should return a IjQLAttrDefn type.
	JBUILD_EXPORT jQLAttrDefn *TEMP_getAttributeDefn();
	// Like this ;-) :
	JBUILD_EXPORT IjQLAttrDefn *getAttributeDefn();

protected:
	IjQLAttrDefn *attrDefn;
	//int LHSJust;
};

// Compare to a plain literal string - no wildcards
class IjQL_Lit_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_Lit_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//Lit_SelTest(){};
	//~Lit_SelTest(){};

	//Lit_SelTest(CVar& _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype){};

	//bool Accept(CVar& CurrentValue);

	//void lookForIndex();
	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::QUOTED_STRING;}
	JBUILD_EXPORT CVar & getLiteralString() const;

private:
};

class IjQL_Null_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_Null_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//Null_SelTest(){};
	//~Null_SelTest(){};
	//bool Accept(CVar& CurrentValue)
	//{
	//	CVar nullStr("");
	//	return checkresult(CurrentValue.Compare(nullStr),jQLParserTokenTypes::NE);
	//}
	// Set this to WITH since it's derived from WITH as there is no enum for it.
	int getNodeType() const { return IjQLTokenTypes::WITH_TEST;}
private:
};

#if 0
///////////////////////////////////////////////////////////////////////////////////////////////////
//
// Dummy class so we have the object,  accept always passes....
//
class Dummy_SelTest : public jQLBaseSelTest
{
public:
	Dummy_SelTest(){};
	~Dummy_SelTest(){};
	bool Accept()
	{
		return true;
	}
	// Set this to WITH since it's derived from WITH as there is no enum for it.
	int getNodeType() const { return jQLParserTokenTypes::WITH;}
private:
};
#endif


// right justified compare
class IjQL_Num_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_Num_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//Num_SelTest(){};
	//~Num_SelTest(){};
	//Num_SelTest(CVar& _literalstring,int _comparetype );
	//bool Accept(CVar& CurrentValue);
	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::QUOTED_STRING;}
private:
	//CVar literalstring; 
	//double constvalue;
	//bool usedouble;
	//int nullLitteral;
};

// right ignore - "aaa]"
class IjQL_RI_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_RI_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//RI_SelTest(){};
	//~RI_SelTest(){};
	//RI_SelTest(CVar& _literalstring, int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	literallength = literalstring.Length();
	//};
	//bool Accept(CVar& CurrentValue);

	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::RIGHT_IGNORE_STRING;}
private:
	//CVar literalstring; 
	//CVar substring;
	//int literallength;
};

// Left ignore  - "[aaa"
class IjQL_LI_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_LI_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//LI_SelTest(){};
	//~LI_SelTest(){};
	//LI_SelTest(CVar& _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	literallength = literalstring.Length();
	//};
	//bool Accept(CVar& CurrentValue);

	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::LEFT_IGNORE_STRING;}
private:
	//CVar literalstring; 
	//CVar substring;
	//int literallength;
};

// Left and right ignore  - "[aaa]"  aka contains substring 
class IjQL_LRI_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_LRI_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//LRI_SelTest(){};
	//~LRI_SelTest(){};

	//LRI_SelTest(CVar& _literalstring,int _comparetype ):literalstring( _literalstring),comparetype(_comparetype)
	//{
	//	literallength = literalstring.Length();
	//};
	//bool Accept(CVar& CurrentValue);
	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::LR_IGNORE_STRING;}
private:
	//CVar literalstring; 
	//int literallength;
};

// A complex pattern containing embedded ']' and/or '^'
class IjQL_CW_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_CW_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//CW_SelTest();
	//~CW_SelTest();
	//CW_SelTest(CVar& _literalstring,int _comparetype );
	//bool Accept(CVar& CurrentValue);
	JBUILD_EXPORT void * GetPatterns() const;
	//const int getCompareType() const;
	int getNodeType() const { return IjQLTokenTypes::COMPLEX_WILD_STRING; };

private:
	//CVar literalstring; 
	//void* Patterns;
	//int useMatches;
	//CVar matchesPatern;
};


class IjQL_LIKE_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_LIKE_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//~LIKE_SelTest()
	//{
	//	CHECK_DP;
	//	JLibEReleaseMATCHES(dp, Patterns);
	//}

	//LIKE_SelTest(CVar& literalstring)
	//{
	//	LIKE_SelTest(literalstring,false);
	//}

	//LIKE_SelTest(CVar& literalstring,bool _usingSQL )
	//{
	//	usingSQL = _usingSQL ;
	//	Patterns = NULL;
	//	preParsePattern(literalstring);
	//	patternString = literalstring;
	//	if (JLibEParseMATCHES(dp, &Patterns, literalstring, literalstring.Length(), 1 ) == 0)
	//		throw jError(JBASE_E_BAD_LIKE_PATTERN);  // how did it get here ???
	//};


	//void preParsePattern(CVar& literalstring );

	//bool Accept(CVar& CurrentValue)
	//{
	//	CHECK_DP;
	//	return JLibETestMATCHES(dp, Patterns, CurrentValue, CurrentValue.Length() );
	//};
	//jQLIndexLookup* buildLookup(jQLIndexSubSegment* idxItr);
	int getNodeType() const { return IjQLTokenTypes::LIKE_TEST; };
	//CVar patternString;
	JBUILD_EXPORT void * GetPatterns() const;

private:
	//bool usingSQL;
};

class IjQL_UNLIKE_SelTest : public IjQLBaseSelTest
{
public:
	IjQL_UNLIKE_SelTest(jQLBaseSelTest* in, IjQLBaseSelTest* parent, IjQLBaseSelTest *left, IjQLBaseSelTest *right);
	//UNLIKE_SelTest();
	//~UNLIKE_SelTest()
	//{
	//	CHECK_DP;
	//	JLibEReleaseMATCHES(dp, Patterns);
	//}
	//UNLIKE_SelTest(CVar& literalstring ):Patterns(0)
	//{
	//	if (JLibEParseMATCHES(dp, &Patterns, literalstring, literalstring.Length(), 1 ) == 0)
	//		throw jError(JBASE_E_BAD_LIKE_PATTERN);  // how did it get here ???
	//};
	//bool Accept(CVar& CurrentValue)
	//{
	//	CHECK_DP;
	//	return !(JLibETestMATCHES(dp, Patterns, CurrentValue, CurrentValue.Length()) );
	//};
	int getNodeType() const { return IjQLTokenTypes::UNLIKE_TEST;}
	JBUILD_EXPORT void * GetPatterns() const;
private:
};

#endif // __IjQLBaseTest_hpp
