// \file IjQLHandler.hpp
// \par Description
// Interface class for the jQLHandler class, for use by jEDI drivers.
// Interface into the jQLHandler class.
// Interfaces are /containers/ for original class...
// If we derive from, then we end up making copies etc. of the original (thoug we still have to instantiate an Interface per object).
// Probably need to prevent copying, and default construction (private copy/assignment operators etc.)
// Implement is_, has_, get_, set_ for properties/attributes, rather than passing back ptrs etc. to other objects.
//
// *ALL* access to jQL objects directly, related to the jQLHandler object *MUST* occur
// within the context of this interface class.
//
// This class *MUST ONLY* be changed on the authority of the jEDI Driver Team.
//
#ifndef __IjQLHandler_hpp
#define __IjQLHandler_hpp

#ifdef _WIN32
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // _WIN32
#include <jsystem.h>			// For JBUILD_EXPORT, JediFileDescriptor etc.
#include <IjQLStatement.hpp>	// Because we declare _IjQLStatement object in here.

// Forward declarations

class IjQLHandler {
public:
	JBUILD_EXPORT IjQLHandler(JediFileDescriptor *fd);
	JBUILD_EXPORT ~IjQLHandler();
    // Accessors for the other Interfaces contained within this class
	JBUILD_EXPORT IjQLStatement& get_IjQLStatement();
    ///IjQLOutputSelectList& getIjQLOutputProcessor();

	// private methods
private:
	// Prevent copying...
	IjQLHandler(IjQLHandler& rh);
	IjQLHandler& operator=(IjQLHandler& rh);

	// private members
private:
	IjQLStatement _IjQLStatement;
	///	IjQLOutputSelectList _IjQLOutputProcessor;
	///	IjQLOutputCount _IjQLCountProcessor;
};

#endif // __IjQLHandler_hpp
