/*
	Defintions for the jBASE External Device Interface , and general
	file and record handling.
*/
#ifndef	JEDI_H
#define	JEDI_H

/* #include <jni.h> 
 * Since we don't want to include jni.h here ( as it breaks jBC programs ),
 * we need to define the equivalent jlong is for each platform...
 */

#ifdef WIN32
typedef __int64	jjlong;
#elif defined(LINUX_ELF) || defined(LINUX_OS390) || defined(HPUX_IA64) || defined(HPUX) || defined(SUN) || defined(OSF1) || defined(__MVS__)
typedef long long jjlong;
#elif defined(AIX_4) || defined(AIX_5)
typedef int64_t jjlong;
#elif defined(__OS400__)
/*
typedef struct JavaLong {
    signed   int high;
    unsigned int low;
} JavaLong;
*/
typedef JavaLong jjlong;

#endif

#ifdef	__MINGW32__
#undef	WIN32
typedef long    uid_t;
#endif

#ifndef DPSTRUCT_DEF
#define	DPSTRUCT_DEF
/*
 * typedef the jBASEDataAraeas structure so we can use it before it
 * is declared.
 */
typedef struct jBASEDataAreas DPSTRUCT;
#endif

#ifdef WIN32
#include <stdlib.h>
#include "jwinnt.h"
#endif
#if defined __OS400__
#include <unistd.h>
typedef	long						jlong64 ;
typedef	unsigned	long			julong64 ;
#endif
/*
	Define error message types sent to JediError()
*/
#define	JEDI_ERROR_CORRUPT	0x00000001	/* Corruption in a file */
#define	JEDI_ERROR_FATAL	0x00000002	/* fatal error */
#define	JEDI_ERROR_WARNING	0x00000004	/* Non-fatal warning */

/*
	The following are special case values of 'errno' to define an error that can not
	be easily identified by normal errno values.
*/
#define	JEDI_ERRNO_MIN						2000
#define	JEDI_ERRNO_DEVICE_QUIT				2000		/* The user has entered 'Q' to quit on a tape operation */
#define	JEDI_ERRNO_LOCK_TAKEN				2001		/* The function call has failed as the resource is locked - use
															instead of EACCES to avoid ambiguity over file open() 
															permissions. */
#define	JEDI_ERRNO_INV_FILE_TYPE			2002		/* Invalid or unknown file type specified */
#define	JEDI_ERRNO_RETRY_OPERATION			2004		/* User has entered 'R' to retry operation */
#define	JEDI_ERRNO_NOT_APPROPRIATE			2005		/* Command is not appropriate for device type */
#define	JEDI_ERRNO_IGNORE_REELNO			2006		/* Ignore expected reel number of tape and use that obtained from label. */
#define	JEDI_ERRNO_FILE_CORRUPT				2007		/* The file is corrupt in some way */
#define	JEDI_ERRNO_LOCKRDONLY				2008		/* Attempt to take an update lock on a read-only file */
#define JEDI_ERRNO_ALREADY_LOCKED			2009		/* Sequential file locked */
#define JEDI_ERRNO_BINLOCK_PERMISSION		2010		/* Cannot take a binary lock on a read-only file */
#define JEDI_ERRNO_LOGGER_EOS				2011		/* TJ error - End of current set or tape */
#define JEDI_ERRNO_LOGGER_EOF				2012		/* TJ error - End of file , i.e. end , no more ! */
#define JEDI_ERRNO_LOGGER_FATAL				2013		/* TJ error - Fatal error, you will have to skip this one */
#define	JEDI_ERRNO_LOGGER_RETRY				2014		/* TJ error - may be at end of current file, worth retrying */
#define	JEDI_ERRNO_NOT_JRLA					2015		/* You can't use this operation if jRLA is active */
#define	JEDI_ERRNO_JTI_TIMEOUT				2016		/* JTransactionIsolation timeout, resource locked by another transaction */
#define	JEDI_ERRNO_FILE_CORRUPT_UPDATE_NO	2017		/* The file is corrupt because the update number wrong */
#define	JEDI_ERRNO_DOCUMENT_RETURNED		2018		/* We read the record but it is a document */
#define	JEDI_ERRNO_DOCUMENT_PARSE_ERROR		2019		/* The document we read couldn't be parsed */

#define	MAX_BUCKET_SIZE						16384		/* Maximin bucket size of a HASH 1 filing system type */
#define	JEDI_LOGNAME_MAX					64			/* Maximum size of the name of a user */
#define	JEDI_HOMEDIR_MAX					256			/* Maximum size of the home directory of a user */
	/* Determine what Unix means when it says "Block Size". This is a fairly generic term, indenpendent
		of file system type */
#define	UNIX_BLOCK_SIZE						512
#define	JEDI_BINARY_FILELOCK_BEGIN			511			/* The offset in a file where the FILELOCK lock begins */

	/* Maximum size of any restore resizing specifications */
#define	JEDI_MAX_RESTORE_SPEC				64
/*
 * Size of page size.
 */
#if defined __OS400__
#define PAGESIZE 4096
#define PAGE_SIZE 4096
#define JEDI_PAGESIZE PAGESIZE
#elif defined(LINUX_PPC)
#define PAGESIZE 8192
#define PAGE_SIZE 8192
#define JEDI_PAGESIZE PAGESIZE
#else
#ifdef JBUILD_OPTION_NO_SYSCONF
#define JEDI_PAGESIZE	PAGESIZE
#else
#ifdef JBUILD_OPTION_USE_SC_PAGESIZE 
#define 	JEDI_PAGESIZE	sysconf(_SC_PAGESIZE)
#else
#define 	JEDI_PAGESIZE	sysconf(_SC_PAGE_SIZE)
#endif
#endif
#endif
/*
 * Structure for encryption and security.
 */
typedef struct {
	char	* security_key ;							/* Key for secure mode */
	int		security_key_len ;							/* Length of the key */
	int		security_mode ;								/* The alogorithm for encryption */
	long	security_seed[2] ;							/* The seed values for the encrypted record */
} jedi_security ;

/*
	The following structure is used when a SELECT operation is performed
	on an opened file descriptor.
*/
struct	JediSelectPtr {
	struct JediSelectPtr	*NextPtr;					// Next select pointer in the chain
	void	*TypePtr;									// Pointer to control block unique for each file type
	} ;
/*
	This structure below is that for all opened files.
*/

typedef struct 
{
	char	* key ;						// Returned record key
	int		keylen ;					// Length of the record key
	char	* rec ;						// Returned record
	int		reclen ;					// Length of the record
	void	* jediptr ;					// Used internally by jEDI
}
JediScanfile;


typedef struct _JQLHandlerStruct JQLHandlerStruct;


typedef	struct {
	int		DriverNumber;								// Defines the drive number to use for subsequent calls 
	void	*TypePtr;									// Points to a structure unique to each file type 
	struct	JediSelectPtr	*SelectPtr;					// Points to an allocated structure used during the SELECT process 
	int		FileType;									// The type of file we have opened -- see below 
	int		Status;										// Current file status - is it open, can it do field read/writes etc 
	struct	jfd * fdinfo ;								// General file status information 
	int		ConvertFlags ;								// Decides type of record conversion to do (see JEDI_CONVERT_xxx bits) 
	void	*TransactionLinkPtr;						// Linked list of file descriptors needed for a transaction 
	int		FileFlags ;									// Options with the 'JEDI_FILE_xxx' bit flags 
	char	*CurrentDir ;								// Directory user was in when file was opened 
	char	*OptionalArgs;								// Optional arguments used to open the file 
	char	*DeferredFilePath ;							// For deferred opens, save the file path area. 
	char	*DeferredPathName ;							// For deferred opens, save the path name (i.e. file name) 
	struct	JediControlBlock	* GeneralControl ;		// General control block for jBASE control file handling 
	void	* TriggerControl ;							// Control block for database triggers 
	void	* SchemaControl  ;							// Control block for schemas 
	void	* IndexControl ;							// Control block for indexes 
	char	** ControlFileList ;						// A list of control files associated with the file 
	int		ControlFileQty ;							// Number of elements in the control file list 
	char	* LoggerFileName ;							// If the logger is active, then this is the full name of the file 
	int		LoggerFileNameLen ;							// The length of the LoggerFileName 
	int		ReadFlags ;									// Set during a READ to show type of record 
	void	*jQLStatement;								// Holds a reference to a jQLStatement 
	char	*CodePage;									// Pointer to code page name 
	void	*CodePageConv;								// Pointer to code page converter 
	JQLHandlerStruct	*jQLHandler;
	JediScanfile		**scanFileHandle;				// Pointer to a JediScanfile--needed here because such that jRFS can access it
	char 	*OpenFileName;								/* File name use to open file */
	char	*ReadXSL;									/* String for XML converion on read */
	char	*WriteXSL;									/* String for XML conversion on write */
	int	RefCount;										/* Count of times opened i.e. if more than one then multiple file variables */
	int jQLStatusFlag;									// Status bit from last call to jQL Compile
	void * record_id_lookup_fd;							/* Pointer to VAR that holds the opened index var */
	struct jedi_audit * Audit ;
	jedi_security	security ;							/* Encryption security */
} JediFileDescriptor;

/*
 * Structure to control audit logging.
 */
#define	AUDIT_DICT_ATTRS			3		// Number of attributes in DICT @AUDIT we will parse
#define AUDIT_DICT_DIR				0		// Relative attribute for the directory name
#define AUDIT_DICT_OPTIONS	 		1		// Options.
#define AUDIT_OUTPUT_MV				0		// Output is MV delimited
#define AUDIT_OUTPUT_CSV			1		// Output is CSV format
#define AUDIT_OUTPUT_TAB			2		// Output is TAB delimited
#define AUDIT_OUTPUT_JSON			3		// Output is JSON format
#define	AUDIT_DISPLAY_TYPE_READ		0		// Internal structure -- tis a READ record
#define	AUDIT_DISPLAY_TYPE_WRITE	1		// Internal structure -- tis a WRITE record
#define	AUDIT_DISPLAY_TYPE_CHANGE	2		// Internal structure -- tis a diff/change record
#define	AUDIT_DISPLAY_TYPE_DELETE	3		// Internal structure -- tis a DELETE record
#define	AUDIT_DISPLAY_TYPE_ENV		4		// Internal structure -- tis a ENV record
#define	AUDIT_DISPLAY_TYPE_MAX		5		// Internal structure -- tis a READ record
#ifdef WIN32
#define DIR_DELIM_STR           "\\"
#define DIR_DELIM_CH            '\\'
#else
#define DIR_DELIM_STR           "/"
#define DIR_DELIM_CH            '/'
#endif
typedef struct jedi_audit {
    int			status ;					// 0 == never opened, 1 == open failed, 2 == open ok
    int			setup ;						// We've called the SETUP routine.
    int			options ;					// Whatever options, see AUDIT_OPTION_XX
    int			filetypeisaudit ;			// Set if the file type is AUDIT
    JediFileDescriptor	* fd_audit ;		// The @AUDIT file.
    JediFileDescriptor	* fd_user ;			// The file descriptor of user's file.
    char		unique_id[32]; 				// Unique identifier for this process.
    unsigned char	* config ;				// The @AUDIT_CONFIG record taken from DICT @AUDIT
    unsigned char	* attr[AUDIT_DICT_ATTRS];// The attributes in the DICT
    int			attr_len[AUDIT_DICT_ATTRS];// The lengths of the attributes
    char		outputdir[1024] ;			// Directory (or pipe) to write to
    int			config_len ;				// Length of the record.
    int			output_type ;				// See AUDIT_OUTPUT_xxx values
    int			write_cmd ;					// An area to pass the command to the jEDI
    char		current_file_name[1024+64] ;	// Name of file we are outputting to
	char		last_key[1024+128] ;		// Optionally to store the last key we write with
    struct jfd		* current_file ;		// File descriptor we are writing to.
    int			current_filetype ;			// See 'AUDIT_FILETYPE_xxx' values.
    time_t		current_time_utc ;			// The UTC of the current file
	int			interval_seconds ;			// Interval between file switches
	struct		log_additions_struct {
		char	cmd ;
		int		from , to ;
		char	* operand ;
	} * log_additions ;
	int		log_additions_allocated , log_additions_used ;
	/* log_additions_counters -- see AUDIT_DISPLAY_TYPE_xx for assignment to array */
	int		log_additions_counters[AUDIT_DISPLAY_TYPE_MAX] ;
} jedi_audit ;
struct jedi_audit    	* JediAuditSetup(struct jBASEDataAreas * , char * FilePath , JediFileDescriptor * fd);
void    		JediAuditBuildRecord(struct jBASEDataAreas * dp , int cmd , 
				char * passed_item_id , int passed_item_id_len , 
				char * passed_record , int passed_record_len , 
				JediFileDescriptor * FileDescriptor , 
				char * return_item_id , int * return_item_id_len , 
				char ** record , int * record_len , double * time_utc, int flags) ;
void    JediAuditWrite(struct jBASEDataAreas *dp, int cmd, char *item_id, int item_id_len ,char * record , int record_len , JediFileDescriptor *FileDescriptor);
void     JediAuditOpen(struct jBASEDataAreas * dp , JediFileDescriptor * fd , char * FilePath );
void	JediAuditOptionsSet(struct jBASEDataAreas * dp , struct jedi_audit * audit_ptr , unsigned char * str , int str_len);
#define	AUDIT_UPDATE_READRECORD		1	// READ a record
#define	AUDIT_UPDATE_WRITERECORD	2	// WRITE a record
#define	AUDIT_UPDATE_DELETERECORD	3	// DELETE a record
#define	AUDIT_UPDATE_CLEARFILE		4	// File has been cleared
#define	AUDIT_UPDATE_DELETEFILE		5	// File has been deleted
#define	AUDIT_UPDATE_QUERY		6	// Some jQL query command
#define AUDIT_FILETYPE_NOTOPEN		0	// File hasn't been opened yet
#define	AUDIT_FILETYPE_FAILED		1	// File cannot be opened
#define	AUDIT_FILETYPE_DIRECTORY	2	// The file type to output is a directory
#define	AUDIT_FILETYPE_FIFO		3	// The file type to output is a FIFO
#define	AUDIT_FILETYPE_REGULAR		4	// The file type to output is a single regular file
#define	AUDIT_OPTION_VERBOSE		0x01	// Verbose output of READs
#define	AUDIT_OPTION_NO_LOG_READ	0x02	// We do not want to log any READ
#define	AUDIT_OPTION_NO_LOG_WRITE	0x04	// We do not want to log any WRITE
#define	AUDIT_OPTION_NO_LOG_DELETE	0x08	// We do not want to log any DELETE
#define	AUDIT_OPTION_NO_READ_TRANSLATE	0x10	// We do not want to translate during READ
#define	AUDIT_OPTION_SAVE_KEY		0x20	// When creating an audit log, save the key

 struct  _JQLHandlerStruct {						/* Handler functions defined. */
	int     (*CompileHandler)(JediFileDescriptor *fd, int (*callback) (DPSTRUCT *dp, int code, void* dataVar,void* dataVar2), char* statement,int _useSelectList);
	int		(*GetPropertyHandler)(JediFileDescriptor *fd, void* propertyValue, void *columnNo,void *property);
	int		(*PutPropertyHandler)(JediFileDescriptor *fd, void* propertyValue, void *columnNo,void *property);
	int		(*PreExecuteHandler ) ( JediFileDescriptor *fd );
	int		(*ExecuteHandler)(JediFileDescriptor *fd, void *SelectList);
	void*	(*GetLastErrorHandler)(JediFileDescriptor *fd);
	int		(*GetSelectListHandler)(JediFileDescriptor *fd, void *pSelectVar, int* pDestination);
	void	(*DeletejQLStatement)(JediFileDescriptor *fd);
	int		(*GetExitMessageHandler)(JediFileDescriptor *fd, void *messageNo, int* count);
	void	(*SetItemHandler)(JediFileDescriptor *fd, JediScanfile *handle, int valueNo, int subValueNo, void **pDataSource );
	int		(*FetchNextHandler)(JediFileDescriptor *fd, void* pControlVar, void* pDataVar);
};

 struct JQLDriverInterfaceTable {
	 int	(*DriverSortTest)(DPSTRUCT * dp, JediFileDescriptor *fd,void* sortAST);
};

/*
	We can now define each file type that has been opened. The following bits can
	be or'ed together to fully describe the file. The value will be stored in the 'FileType'
	in the above 'JediFileDescriptor' variable type.
*/
#define		JEDI_TYPE_FILEHASH1			1				/* Eld HASH1 file type - no longer supported */
#define		JEDI_TYPE_FILEHASH2			2				/* Eld HASH2 file type - no longer supported */
#define		JEDI_TYPE_EXTERNAL			3				/* What ever file it is, it goes through an externally executable file */
#define		JEDI_TYPE_UNIX				4				/* The file is a Unix directory -- records become unix files */
#define		JEDI_TYPE_HALF				5				/* The media for seeing the records is half inch magnetic tape */
#define		JEDI_TYPE_SCT				6				/* The media for seeing the records is Streaming Cartridge Tape */
#define		JEDI_TYPE_FLOPPY			7				/* The media for seeing the records is floppy diskette */
#define		JEDI_TYPE_FILE				8				/* The data is coming from a file rather than a device */
#define		JEDI_TYPE_USER				9				/* User-defined shared-object database driver */
#define		JEDI_TYPE_REMOTE			10				/* Remote File */
#define		JEDI_TYPE_FILEHASH3			11				/* HASH3 file type */
#define		JEDI_TYPE_SEQ				12				/* SEQuential files */
#define		JEDI_TYPE_FILEHASH4			13				/* HASH4 file type */
#define		JEDI_TYPE_DISTRIB			14				/* Distributed file type */
#define		JEDI_TYPE_FILEHASH5			15				/* HASH5 file type */
#define		JEDI_TYPE_FILEHASH6			16				/* HASH6 file type */
#define		JEDI_TYPE_FILEHASHP			17				/* HASHP or JP file type */
#define		JEDI_TYPE_REMOTE3			18				/* Eld Remote Files */
#define		JEDI_TYPE_DELLA				19				/* Remote files controlled by jDELLA */
#define		JEDI_TYPE_FILEHASHR			20				/* Resilient files */
#define		JEDI_TYPE_FILEHASHD			21				/* HASHD file type , older original version */
#define		JEDI_TYPE_DYNAMIC_OBJECT	22				/* Based on a jBASE dynamic object */

/*
	The following bits are placed in the 'Status' field of the descriptor.
*/
#define		JEDI_STATUS_FIELDREAD		0x00000001		/* The file driver can perform field read operations */
#define		JEDI_STATUS_FIELDWRITE		0x00000002		/* The file driver can perform field write operations */
#define		JEDI_STATUS_INTERNAL_LOCKS	0x00000004		/* Show we are using our own internal locking system */
#define		JEDI_STATUS_QPOINTER		0x00000040		/* File is a Q pointer */
#define		JEDI_STATUS_MD				0x00000080		/* File is an MD definition */
#define		JEDI_STATUS_SYSTEM			0x00000100		/* File is a SYSTEM file */
#define		JEDI_STATUS_READ			0x00000200		/* User is allowed to READ from the file */
#define		JEDI_STATUS_WRITE			0x00000400		/* User is allowed to WRITE to the file */
#define		JEDI_STATUS_DICT			0x00000800		/* File is a DICT level */
#define		JEDI_STATUS_DATA			0x00001000		/* File is a DATA level */
#define		JEDI_STATUS_RPOINTER		0x00004000		/* The file OPEN has found a REMOTEFILE Pointer */
#define		JEDI_STATUS_NOBASE_LOCKS	0x00010000		/* If set it stops the jediBase code from controlling record locks */
#define		JEDI_STATUS_MUST_CLOSE		0x00020000		/* If set then upon program exit we MUST call the close function */
#define		JEDI_STATUS_REMOTE_LOG		0x00040000		/* Set if remote Q pointer or remote stub has been set to be logged */
#define		JEDI_STATUS_STUB_POINTER	0x00080000		/* File is an MD STUB file pointer */
#define		JEDI_STATUS_TRANS_CLRFILE	0x00100000		/* File supports it's own CLEARFILE in a transaction */
#define		JEDI_STATUS_FPOINTER		0x00200000		/* File is a F pointer,  need some way to spot this */
/*
				 * Note: Both the JEDI_STATUS_DICT and JEDI_STATUS_DATA flags are
				 *		 set on a single level file.
				 */

/*
	The following bits are set in the 'ConvertFlags' field of the file descriptor.
*/
#define	JEDI_CONVERT_READ_BINARY		0x00000001		/* All future reads in binary - no conversions at all ever */
#define	JEDI_CONVERT_READ_TEXT			0x00000002		/* All future reads in text format - conversion always applied */
#define	JEDI_CONVERT_READ_INTELLIGENT	0x00000004		/* All future reads will decide if to convert or not -- default value */
#define	JEDI_CONVERT_WRITE_BINARY		0x00000010		/* All future writes in binary - no conversions at all ever */
#define	JEDI_CONVERT_WRITE_TEXT			0x00000020		/* All future writes in text format - conversion always applied */
#define	JEDI_CONVERT_WRITE_INTELLIGENT	0x00000040		/* All future writes will decide if to convert or not -- default value */
#define	JEDI_CONVERT_KEYS_BINARY		0x00000100		/* All future record keys in binary - no conversions at all ever */
#define	JEDI_CONVERT_KEYS_TEXT			0x00000200		/* All future record keys in text format - conversion always applied */
#define	JEDI_CONVERT_KEYS_INTELLIGENT	0x00000400		/* All future record keys decide if to convert or not -- default value */
#define	JEDI_CONVERT_LAST_READ_BINARY	0x00001000		/* Set to 1 if last read was binary, otherwise conversion applied */
#define	JEDI_CONVERT_LAST_WRITE_BINARY	0x00002000		/* Set to 1 if last write was binary, otherwise conversion applied */
#define	JEDI_CONVERT_LAST_KEYS_BINARY	0x00004000		/* Set to 1 if last record key was binary, otherwise conversion applied */

/*
 * The following are bit fields sent in the Flags parameter to 
 * some of the function calls.
 *
 * To take a record lock, use EITHER the JEDI_RECORD_LOCKRECORD or the
 * JEDI_RECORD_LOCKRECORD_NOWAIT bits. The first will block and the
 * second will non-block, both for an update lock.
 *
 * You can modify either of the update locks by  adding the
 * JEDI_LOCKRECORD_READ to make it a read lock instead.
 *
 * These bits are also used for binary locks as well as record locks.
 */
#define		JEDI_RECORD_LOCKRECORD			0x00000001		/* Take a record lock and wait for it to become jfree */
#define		JEDI_RECORD_LOCKRECORD_NOWAIT	0x00000002		/* Take a record lock, do not wait if lock busy */
#define		JEDI_RECORD_UNLOCKRECORD		0x00000004		/* Unlock one or more record locks */
#define		JEDI_RECORD_LOCKRECORD_READ		0x00000008		/* Any locks applied should be read locks (note:
																the default is to apply update locks) */
#define		JEDI_RECORD_RESERVED_1			0x00000010		/* Was JEDI_RECORD_NOCRYPT */
#define		JEDI_RECORD_FIELDREADWRITE		0x00000020		/* Use the field number in passed parameters to read / write a single field only */
#define		JEDI_RECORD_SCRATCHWRITE		0x00000040		/* During a WRITE command, the data area supplied is a scratch area
																and can be over-written by the file driver. */
#define		JEDI_RECORD_LOCK_STATUS			0x00010000		/* No locks taken -- find status of the lock */
#define		JEDI_RECORD_MALLOC_SIZE			0x00100000		/* The READ mallocptr() requires size */
#define		JEDI_RECORD_MALLOC_BUFFPTR		0x00200000		/* The READ mallocptr() requires buffptr */
#define		JEDI_RECORD_MALLOC_DP			0x00400000		/* The READ mallocptr() requires 'dp' */
#define		JEDI_RECORD_DOCUMENT			0x00800000		/* The WRITE is a object document */
#define		JEDI_RECORD_IGNORE_DELIM_KEY	0x01000000		/* Ignore delimiters in the record key */

/*
	The following bits are in the 'FileFlags' header for HASHx files.
*/
#define		JEDI_FILE_NOLOG 					0x00000001		/* If set, do not log updates to the file with the logger */
#define		JEDI_FILE_NOTRANS				0x00000002		/* If set, do not make this part of a transaction */
#define		JEDI_FILE_NOBACKUP				0x00000004		/* If set, do not save this file during a backup operation */
#define		JEDI_FILE_NO_HOLE_FILL		0x00000008		/* If set, file not hole filled when created */
#define		JEDI_FILE_AUDIT_LOG				0x00000010		/* Set when a record of any file access is required ( Audit Logging ) */
#define		JEDI_FILE_NETWORK_ON		0x00000020		/* The file is specifically created to work over networks */
#define		JEDI_FILE_NETWORK_OFF		0x00000040		/* The file is specifically to NOT be on a network */
#define		JEDI_FILE_CONTROL				0x00000080		/* A control file containing indexes and/or schemas and/or triggers */
#define		JEDI_FILE_CACHE_WRITE		0x00000100		/* Where possible, cache the writes */
#define		JEDI_FILE_SECURE					0x00000200		/* Try to make file more secure */
#define		JEDI_FILE_CONTROL_TRIED		0x00000400		/* Set when we have tried once already to open the control file */
/*
 * The ASCII and EBCDIC flags are set only when we can be sure the data
 * is in one format or another. Without either of these, we assume the native code 
 * page of the platform it is running on.
 */
#define		JEDI_FILE_EBCDIC				0x00000800		/* Data in the file is in EBCDIC */
#define		JEDI_FILE_ASCII					0x00001000		/* Data in the file is in ASCII */
#define		JEDI_FILE_RESERVED1				0x00002000		/* NOW RESERVED - Set if file is a journaled file */
#define		JEDI_FILE_CRYPT_OLD				0x00004000		/* Was previously JEDI_FILE_CRYPT */
#define		JEDI_RESERVED_1  				0x00008000		/* Was previously JEDI_FILE_CASE_ID */
#define		JEDI_RESERVED_2					0x00010000		/* Was previously JEDI_FILE_CRYPT_LOCKED */
#define		JEDI_FILE_CASE_ID_INTERNAL		0x00020000		/* Case insensitive, but internal to driver without a lower case cross reference) */
#define		JEDI_FILE_LOCK_TAKEN			0x00040000		/* At some point a lock has been taken */
#define		JEDI_FILE_CRYPT					0x00080000		/* The data and key will be encrypted using newer security profile code */
/*
	The following are the sub-commands that can be used with the JediIOCTL() function.
*/
#define		JEDI_IOCTL_GETLABEL		4				/* Get description of the label last read */
#define		JEDI_IOCTL_FLUSH				5				/* Flush contents of tape buffer */
#define		JEDI_IOCTL_GETSTATUS		6				/* Get the status and return to the caller */
#define		JEDI_IOCTL_SETSTATUS		7				/* Set the status of parts of the tape system */
#define		JEDI_IOCTL_IGNORELABEL	8				/* Ignore labels on reads for durination of the program */
#define		JEDI_IOCTL_CONVERT			9				/* Set/Get type of record/recordkey conversions to do */
#define		JEDI_IOCTL_FILESTATUS		10				/* Get details about the file itself */
#define		JEDI_IOCTL_FINDRECORD	11				/* Test whether record exists */
#define		JEDI_IOCTL_ATTACH				12				/* Make this device JUST for use of a single user. */
#define		JEDI_IOCTL_DETACH			13				/* Stop this device being used by a single user. */
#define     JEDI_IOCTL_HASH_RECORD 15             /* Get the hash value used for reading records, and the bucket number */
#define		JEDI_IOCTL_GETLABELPARMS		16		/* Get details of label, such as time, date, parameters */
#define		JEDI_IOCTL_SETPARM1		17				/* Send a definition of the parameter field of the label */
#define		JEDI_IOCTL_SETREEL			18				/* Set the reel number to any value */
#define		JEDI_IOCTL_SETLABELTYPE		19		/* Set the label type to any valid value or none */
#define		JEDI_IOCTL_CHMOD				21				/* Change/display the file flags status, such as if to be logged etc. */
#define		JEDI_IOCTL_CACHE_WRITE	22				/* Cache the WRITE's if possible */
#define		JEDI_IOCTL_SETBLOCKSIZE	23				/* Ioctl to change the block size */
#define		JEDI_IOCTL_FINDRECORD_EXTENDED 	24				/* Extended info (size and update time) */
#define		JEDI_IOCTL_STATDIR			25		/* Display the flags for a directory */
/*
* SCSI tape IOCTL commands.
*/
#define		JEDI_IOCTL_REWIND				30				/* Rewind the file to the beginning - no arguments */
#define		JEDI_IOCTL_UNLOAD				31				/* Rewind AND UNLOAD the file - no arguments */
#define		JEDI_IOCTL_WEOF					32				/* Write an EOF to the tape device - 1 arg, the number of EOF's */
#define		JEDI_IOCTL_BACK					33				/* Move back a number of spaces, or to previous EOF mark */
#define		JEDI_IOCTL_FORWARD			34				/* Move forwards a number of spaces, or to next EOF mark */
#define		JEDI_IOCTL_CHECK				35				/* Check for parity errors up to next file mark */
#define		JEDI_IOCTL_ERASE				36				/* Erase entire contents of a SCSI QIC tape */
#define		JEDI_IOCTL_RETENSION		37				/* Retension a QIC tape */
#define		JEDI_IOCTL_FASTSCAN		38				/* Return details of fast scan if supported */

/* Ioctl commands for use on Remote files */
#define		JEDI_IOCTL_REMOTE_EXEC			40				/* Remote execute statement */
#define		JEDI_IOCTL_REMOTE_GETFILENAME	41				/* Get Remote File Name */
#define		JEDI_IOCTL_REMOTE_GETENV		42				/* Get Remote environment variable */
#define		JEDI_IOCTL_REMOTE_GETHOST		43				/* Get Hostname for connection */
#define		JEDI_IOCTL_REMOTE_GETUSER		44				/* Get Remote username */
#define		JEDI_IOCTL_REMOTE_FORK			45				/* Fork and execute command */
#define		JEDI_IOCTL_REMOTE_PUTENV		46				/* Update environment variables */
#define		JEDI_IOCTL_REMOTE_CLOSEALL		47				/* Close all remote files */
#define		JEDI_IOCTL_REMOTE_UNLOCKALL		48				/* Unlock all remote files */
#define		JEDI_IOCTL_REMOTE_DISCONNECTALL	49				/* Disconnect all remote files */


#define JEDI_IOCTL_SEQ_WEOF		 			50      		/* Sequential Write EOF  */
#define JEDI_IOCTL_SEQ_APPEND		 		51      		/* Force to end of file  */
#define JEDI_IOCTL_SEQ_FLUSH				52				/* Flush file */



#define JEDI_SEQ_READONLY 	0x00000001 	/* Open in readonly */
#define JEDI_SEQ_DEVICE 	0x00000002 	/* File is a Device */
#define JEDI_SEQ_CRLF	 	0x00000004 	/* Append CRLF to output */
#define JEDI_SEQ_APPEND		0x00000008	/* Flag must append */
#define JEDI_SEQ_EXISTING	0x00000010	/* Flag must be existing */
#define JEDI_SEQ_CREATE		0x00000020	/* Flag must create */
#define	JEDI_SEQ_TRANSLATE	0x00000040	/* Read/write operations perform crude character translation */

#define JEDI_SEQ_TYPE_SEQUENTIAL	0		/* Type zero are sequential files */
#define JEDI_SEQ_TYPE_SERIAL	 	1		/* Type one  are serial devices */
#define JEDI_SEQ_TYPE_PIPE			2		/* Type two  are pipe devices */

#define JEDI_IOCTL_SERIAL_SET_TIMEOUT		53		/* Set timeout */
#define JEDI_IOCTL_SERIAL_GET_TIMEOUT		54		/* Get current Timeout */
#define JEDI_IOCTL_SERIAL_SET_STATE			55		/* Set comm state */
#define JEDI_IOCTL_SERIAL_GET_STATE			56		/* Get comm state */
#define JEDI_IOCTL_SERIAL_SET_RAW			57		/* Get comm state */
#define JEDI_IOCTL_SERIAL_SET_CRLF			58		/* Get comm state */
#define JEDI_IOCTL_SERIAL_READ_LEN			59		/* Amount of data to read */

#define JEDI_IOCTL_SEQ_NO_DELIMITER			60		/* Suppress delimiter */
#define JEDI_IOCTL_SEQ_CHANGE_DELIMITER		61		/* Change delimiter */
#define JEDI_IOCTL_SEQ_CHANGE_RECORDSIZE	62		/* Change SEQ RecSz */
#define JEDI_IOCTL_SEQ_CHANGE_PADCHAR		63		/* Change SEQ Pad char */

#define JEDI_IOCTL_OPENSER_SET_STATE		64		/* Commstate OPENSER like */
#define JEDI_IOCTL_SERIAL_FLUSH				65		/* Force flush of serial device */
#define JEDI_IOCTL_SEEK						66		/* Seek to position */
#define JEDI_IOCTL_GENERATE_VIEWS			67		/* Generate SQL views for driver */
#define JEDI_IOCTL_SQL_SELECT_PREPARE		68		/* Prepare SQL select for SELECT FileVar statement */
#define JEDI_IOCTL_SQL_SELECT_FETCH			69		/* Execute SQL select and fetch results */
#define JEDI_IOCTL_SQL_SELECT_CLOSE			70		/* Close select cursor before exhausted */
#define JEDI_IOCTL_SQL_EXECUTE_DIRECT		71		/* Execute SQL direct */
#define JEDI_IOCTL_SQL_SET_READ_CURSOR		72		/* Set cursor for read with criteria */
#define JEDI_IOCTL_SQL_SET_JOIN_CURSOR		73		/* Set cursor for join used with above */
#define JEDI_IOCTL_SETCODEPAGE				74		/* Set code page for file conversion */
#define JEDI_IOCTL_REMOTE_CREATEFILE		75		/* Remote file creation */
#define JEDI_IOCTL_OPTION_AVAILABLE			76		/* driver cmd to return true if specified option available */
#define JEDI_IOCTL_JBASE_STAT				77		/* jstat output to stdout			*/

#define JEDI_IOCTL_JR_CHMOD					78		/* JR files own chmod - don't interfere with chmod */
#define	JEDI_IOCTL_VIRTUAL_ON				79		/* Turn on virtual (memory) I/O */
/*
 * These IOCTL functions are process in the run-time, rather than in the jedi drivers for each file.
 */
#define JEDI_IOCTL_GETFILENAME              100      // Get the name of the file associated with a file variable.
#define JEDI_IOCTL_CHANGEFLAGS              101      // Get and change the file flags for Indexes, Schemas and Triggers.
#define JEDI_IOCTL_AUDIT_SET            	102      // Append auditing logging definition
#define JEDI_IOCTL_AUDIT_GET		        103      // Get current logging options as string
#define	JEDI_IOCTL_AUDIT_LAST_KEY			104		// Get the audit logging last key written


#define JEDI_SERIAL_PARITY_NONE				0		/* No Parity */
#define JEDI_SERIAL_PARITY_EVEN				1		/* Even Parity */
#define JEDI_SERIAL_PARITY_ODD				2		/* Odd Parity */
#define JEDI_SERIAL_PARITY_MARK				3		/* Mark Parity */
#define JEDI_SERIAL_PARITY_STRIP			4		/* Strip top bit Parity */

#define JEDI_SERIAL_FLOW_XON				0x1		/* Xon/Xoff flow control */
#define JEDI_SERIAL_FLOW_CTS				0x2		/* RTS/CTS (Hardware) */
#define JEDI_SERIAL_FLOW_DSR				0x4		/* DTR/DSR (Hardware) */
#define JEDI_SERIAL_FLOW_INPUT				0x8		/* Input flow control */
#define JEDI_SERIAL_FLOW_OUTPUT				0x10	/* Output flow control */
#define JEDI_SERIAL_FLOW_NONE				0x20	/* No flow control */

/*
** Define struct for sequential file type
*/

typedef struct {
	FILE	*FilePtr;
	long	Offset;
	int		Flags;
	int		Type ;
	int		DelimFlg;
	unsigned char	DelimChar;
	unsigned char	PadChar;
	int 	RecSz;
	int		TimeOut;
} SEQFILE ;

/*
	The following structure defines the drivers for each different file device driver.
	See the source JediData.c for the drivers used on your system.
*/


struct JediDriverStruct
{
	char	*FileTypeDescr;								// Description of file type e.g. "UD" (Unix Directory) , "HASH3" 
	int		(*OpenCodePtr)(DPSTRUCT *, JediFileDescriptor * , char * , int , int );	// Address of function to attempt to open a file of that type 
	int		(*CreateCodePtr)(DPSTRUCT *, char *, char *, int, int, int, int,
							 char **, char **, char *, char *);										// Address of function to create a file of that type 
	int		(*JediClose)(DPSTRUCT *, JediFileDescriptor * , int );						// Close a file 
	int		(*JediSelect)(DPSTRUCT *, JediFileDescriptor *, 
						  struct JediSelectPtr **);													// Select a file in order to read all record keys from it 
	int		(*JediSelectEnd)(DPSTRUCT *, JediFileDescriptor *, 
						     struct JediSelectPtr * );												// Terminate prematurely a SELECT list 
	int		(*JediReadnext)(DPSTRUCT *, JediFileDescriptor *, struct JediSelectPtr *,
							char ** , int *);														// Read the next record key from a file 
	int		(*JediReadRecord)(struct jBASEDataAreas *,JediFileDescriptor *, 
							int, char *, int, char **, int *, int, void *(*)());
	int		(*JediWriteRecord)(DPSTRUCT *, JediFileDescriptor *, int, 
							   char *, int, char *, int, int);										// Write a record to a file 
	int		(*JediDeleteRecord)(struct jBASEDataAreas *, 
							JediFileDescriptor *, int, char *, int);
	int		(*JediLock)(DPSTRUCT *, JediFileDescriptor *, int, char *, int );			// Take a lock / release a lock 
	int		(*JediIOCTL)(DPSTRUCT *, JediFileDescriptor *, int, void *, int,
		void *, int *);															// Various control functions (see JEDI_IOCTL_xxx below) 
	int		(*JediClearFile)(DPSTRUCT *, JediFileDescriptor * );						// Delete ALL records from a file 
	int		(*JediDeleteFile)(DPSTRUCT *, JediFileDescriptor * , int );				// Delete a file 
	int		(*JediSync)(DPSTRUCT *, JediFileDescriptor * );							// Synchronise file I/O to disk 
	int		(*JediTransactionBegin)(DPSTRUCT *, int , char *);						// Set if driver has own transaction support -- begin a transaction 
	int		(*JediTransactionRollback)(DPSTRUCT *, int , char *);					// Set if driver has own transaction support -- roll back a transaction 
	int		(*JediTransactionCommit)(DPSTRUCT *, int , char *);						// Set if driver has own transaction support -- commit a transaction 
	int		(*JediGetJQLHandler)(DPSTRUCT *, JediFileDescriptor * );				// Set if driver has own transaction support -- commit a transaction 

			// Used by jEDI to keep track of if this driver is in a transaction 
#ifdef SEE_AARON
	int		(*JediTransactionPrepare)(DPSTRUCT *, int);								// Set if driver has 2PhaseCommit support -- prepare a transaction 
#endif
	} ;

/*
 * Define the commands sent to JediFileOp().
 */
#define	JEDI_FILEOP_CREATE_FILE		1			// Create a file
#define	JEDI_FILEOP_DELETE_FILE		2			// Delete a file
/*
 * Define the flags sent to JediFileOp()
 */
#define	JEDI_FILEOP_DICT_ONLY		0x0001		// Only the DICT section to be created
#define	JEDI_FILEOP_DATA_ONLY		0x0002		// Only the DATA section to be created
#define	JEDI_FILEOP_NO_RECURSIVE_DELETE	0x0004	// During a delete-file, don't recursively delete the directories
/*
 * This is the structure used to hold record lock information.
 * Win32: A file lock must be released using the file Handle that was
 * actually used to take the lock, even if we open a file more than once.
 * we therefore need to save this information.
 */
struct jfd_lock
{
	struct	jBASEDataAreas	* dp ;				// The dp that references this file
	off_t					hashvalue ;			// The hash value of the record lock
	char					key[64] ;			// The record key
	int						keylen ;			// Length of record key or -1 if a binary lock
	int						read_lock_flag ;	// Set to 1 if it was a read lock
} ;
/*
 * Structure for holding details of all dp's that reference a file.
 */
struct	jfd_ref_count
{
	struct	jBASEDataAreas	* dp ;				// The dp that references this file
	int						ref_count ;			// Number of times in the 'dp' the file is references
} ;
/*
 * This is the basic structure which is the handle
 * for all calls wihin these sources.
 */
struct jfd
{
	/*
	 * I-node information.
	 */
	ino_t           ino;						// The i-node number of a file 
	dev_t           dev;						// The device number of a file 
	mode_t          mode;						// The mode of a file 
	ino_t           lockid1;
	dev_t			lockid2;					// The identifiers to use in the locking scheme 
	/*
	 * Other.
	 */
	struct jfd_lock *locklist;					// The list of locks outstanding 
	int             locklist_qty;				// The number of entries allocated in'locklist' 
	struct	jfd_ref_count	* ref_count_list ;	// List of all dp's that reference us
	int				ref_count_qty ;				// Number of entries in ref_count_list
	void			* jfd_mutex ;				// Allows us to lock this file handle during updates
	int             lockcount;					// Current number of locks outstanding for this file 
	int			origmode ;						// The original mode we tried to open the file in 
	int			status ;						// General status of this file 
	char		* path ;						// The path associated with the file descriptor 
	char		* path_url ;					// The path described as a URL.
	int			filefd ;						// The unix/NT file descriptor for the opened file
	struct	jfd	* next ;						// The next jfd handle in our linked list 
	struct	jfd	* prev ;						// The previous jfd handle in our linked list 
	struct jflinkstat	 *linkstat ;			// Optional pointer to the information for the jfXXXlink() functions 
	struct	ramfile	* ramfile ;			// Used for virtual (memory) I/O
	/*
	 * The following 'linebuffer*' members are to allow the 'JRunFileIOGetLine()' function
	 * to work, which basically reads in 1 line at a time.
	 */
	char	* linebuffer ;						// Address of allocated space where data is read in 
	long	linebuffer_offset ;					// Next offset into the file to read in 
	int		linebuffer_allocated ;				// How much space at 'linebuffer' is allocated 
	int		linebuffer_used ;					// How much data has been read info 'linebuffer' 
	int		linebuffer_nxt ;					// Pointer into 'linebuffer' to get next line from 
	int		lockfd;						// Handle for psuedo lock file
	int		disable_64bit_io ;					// Set to 1 to disable files > 2GB in J4 files.
	struct	jdynamic	* persistent_locks ;	// Used for persistent locks
	jedi_security	* security ;				// Any security mode (encryption/decryption)
} ;
/*
 * Define general status bits.
 */
#define	JEDI_FILEIO_READ		0x00000001				/* File is opened for reading */
#define	JEDI_FILEIO_WRITE		0x00000002				/* File is opened for writing */
#define	JEDI_FILEIO_BINARY		0x00000004				/* File is opened in binary mode */
#define	JEDI_FILEIO_REGULAR		0x00000008				/* File is a regular Unix/NT/Win95 file */
#define	JEDI_FILEIO_DIR			0x00000010				/* File is a directory */
#define	JEDI_FILEIO_CREAT		0x00000080				/* The file MAY have been created from scratch */
#define	JEDI_FILEIO_TRUNC		0x00000100				/* The file should be truncated when opened */
#define	JEDI_FILEIO_LOCKRLA		0x00000200				/* Set of using jRLA locks instead of Unix/NT */
#define	JEDI_FILEIO_READONLY	0x00000800				/* The file is globally set to be a read only file */
#define	JEDI_FILEIO_APPEND		0x00001000				/* Open the file in 'append' mode */
#define	JEDI_FILEIO_PSEUDO		0x00002000				/* File is not a real file */
#define	JEDI_FILEIO_NOERROR_NOSPACE	0x00004000			/* If no space, don't call error function */
#define JEDI_FILEIO_SEQUENTIAL		0x00008000				/* Sequential file access required */

/*
 * Define the variable type for locks.
 */
typedef	off_t	 jflock_t ;
/*
 * Define the layout of the structure used by calls to the jfXXXlink() function call set.
 */
struct	jflink
{
	char	* frame_ptr ;				// Start of frame, including the header 
	int		frame_qty ;					// Number of frames allocated 
	char	* data_ptr ;				// Start of usable data 
	int		data_len ;					// Amount of usable data 
	off_t	base_fid ;					// Base frame id read in 
	off_t	* link_fid_list ;			// Pointer to a list of linked frame id of data read in 
	int		link_fid_qty ;				// Number of linked frames read in 
	struct	jfd	* fdinfo ;				// File pointer for who read in the block 
	int		mallocflag ;				// Set to 1 if data at 'frame_ptr' was malloc'ed 
	int		locktype ;					// The type of lock outstanding 
	int		update_seqid ;				// The incrementing sequence id 
	int		update_flags ;				// The general purpose flags 
} ;
/*
 * This is the structure we return from JRunFileIOReadLockStatus.
 */
struct	jflock
{
	pid_t		lastlock ;								// Process ID that blocked the last lock
	off_t		hashvalue ;								// The hash value of the supplied record key
} ;
#include <jediFile64.h>

int		JediBaseCreateSubDirectories (char *PathName, int Permissions );
int		JediBaseDeleteSubDirectories (char *FileName , int flags );
int		JediBaseIoctlConvert (DPSTRUCT * dp, int Current, void *InputAddr, int InputLen, void *OutputAddr, int *OutputLen);
int		JediBaseGetcwd(DPSTRUCT * dp,char * DirBuffer, int len);
int		JediClearFile (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor) ;
int		JediCreateFileRaw(DPSTRUCT * dp,char * name , char * operand , char * error_buffer , int file_flags , 
				char ** qualifiers , char * realpathname , void (*SigSetFunction)());
int		JediClose(DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Flags );
int		JediCloseFileCache(DPSTRUCT * dp);
int		JediConvertUnixFileName (DPSTRUCT * dp, char * InitialPath , char * InputFileName , char * OutputFileName, char *HeaderInfo);
int		JediDeleteFile (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor ,int Flags );
int		JediDeleteRecord(DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Flags, char *RecordKey, int RecordKeyLen );
int		JediIOCTL (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int SubCommand, void *IoctlPassAddr, 
				   int IoctlPassLen, void *IoctlReturnAddr, int *IoctlReturnLen );
int		JediLock (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Flags, char *RecordKey, int RecordKeyLen );
int		JediLockEntireFile (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Flags );
int		JediMsg (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int SizeSnd, int CommandSnd, int TimeOut );
int		JediOpen (DPSTRUCT * dp, JediFileDescriptor **FilePointer, char *PathName, char *FilePath , int flags  );
int		JediOpenSeq(DPSTRUCT * dp,JediFileDescriptor ** FilePointer, char * PathName, char *FileName, int Flags, int Type) ;
int		JediQuickOpenRead (DPSTRUCT * dp, JediFileDescriptor **FileDescriptor, char *FileName, char *RecordKey, char **BufferPtr, int *BufferLen);
int		JediReadRecord (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Flags, char *RecordKey, int RecordKeyLen, char **BufferPtr, int *BufferLenPtr, int FieldNumber , void * (*mallocptr)());
int		JediReadnext (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, struct JediSelectPtr *SelectPtr, char **RecordKeyPtr, int *RecordKeyLenPtr );
int		JediSelect (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, struct JediSelectPtr **SelectPtr );
int		JediSelectEnd (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, struct JediSelectPtr *SelectPtr );

int		JediSync (DPSTRUCT * dp,JediFileDescriptor *FileDescriptor); 
    
int		JediSystemLock (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Command, int Hash );
int		JediWriteRecord (DPSTRUCT * dp, JediFileDescriptor *FileDescriptor, int Flags, char *RecordKey, int RecordKeyLen, char *BufferPtr, int BufferLen, int FieldNumber );
int		JediBaseAddDriver(DPSTRUCT * dp,struct JediDriverStruct * DriverDetails);
int		JediFileOp(DPSTRUCT * dp, int command, int flags, char * FileSpec,
				   char * UnixOptions, char * LegacyOptions, char * ErrorMessage, 
				   void	(*interrupt)());
int		JediParseFileSpec(DPSTRUCT * dp, char * file_spec , char ** file_name , char ** section_name , char **type ,
				char **operand1 , char ** operand2 , int * dict_flag , int * data_flag , int * perm , int * hashmethod);
int		JediBaseUmemcmp(DPSTRUCT * dp,char * field1 , char * field2 , int len );
int		JediGetJQLHandler(DPSTRUCT * dp, JediFileDescriptor *);
char	* JediBaseReadMalloc(DPSTRUCT * dp , int flags , void *(mallocptr)() , char ** bufferptr , int newsize );

#ifdef WIN32
DllExport int	JediBaseCheckFileSizePath(DPSTRUCT * dp, char *FilePath, int requiredsizek , int * freek , int * actualk);
DllExport int	JBASEQueryDiskSpacePath(DPSTRUCT *dp, char *FilePath, int requiredsizeMB, int *freeMB, int *actualMB);
#else
int		JediBaseCheckFileSize(DPSTRUCT * dp, int fd , int requiredsizek , int * freek , int * actualk);
int		JBASEQueryDiskSpace(DPSTRUCT *dp, int fd, int requiredsizeMB, int *freeMB, int *actualMB);
#endif
int		JediBaseParseChmod(DPSTRUCT * dp, JediFileDescriptor * fd , char * args , int argslen , long * fileflags , char * restorespec , char * rtndetails , int * rtnlen );

unsigned char * Jedimemccpy ( unsigned char *s1, unsigned char *s2, int c, int n );
long 	JediBaseHash(char * RecordKey , int RecordLen , int HashType) ;
void	JediFreeFileDescriptor (DPSTRUCT * dp, JediFileDescriptor ** FileDescriptor );
void	JediBaseModifyPath(DPSTRUCT * dp, char ** qualifiers , char * PathName , long totalsize , char * realpathname);

int		(*JediObjectFindFunction(DPSTRUCT * dp, char * function_name , int convert_flag ))();
unsigned int 	JediBaseHashUINT(unsigned char * RecordKey , int RecordLen , int HashType) ;
char	* JediBaseFindQualifier(DPSTRUCT * dp,char ** qualifierlist , char * string , int scantype );
char	* JediBaseExtractAttribute(DPSTRUCT * dp,char * BufferPtr , int * Len , int fieldno );
int		JediBaseSignal(DPSTRUCT * dp);
FILE	* JediTrace(DPSTRUCT * dp) ;
void	JediGetAllFiles(DPSTRUCT * dp, void* handle, void * DVar,
						void Callback(	DPSTRUCT * dp, void* handle, void * DVar,
										JediFileDescriptor	*JediPtr, char* name, int casesens)
										);
void	JediWaitDatabase(DPSTRUCT * dp , int waitmask);
time_t	JediPauseDatabase(struct jBASEDataAreas *dp, int operation , int type , int verbose , char * DBName);
int     JediBaseStdDevMean(struct jBASEDataAreas *dp, int nitems, off_t TotalData, off_t SumSquares);
/*
 * The following definitions used with the fast scan 
 * mechanism available from certain database drivers.
 */


#define		JEDI_SCANFILE_FLAGS_NOLOCK		0x0001		/* Read in read-only mode, no updates by other processes */
#define		JEDI_SCANFILE_FLAGS_KEYONLY		0x0002		/* Only return the record key */
#define		JEDI_SCANFILE_FLAGS_LOCKGROUP	0x0004		/* Keep groups locked while scanning */
#define		JEDI_SCANFILE_FLAGS_MODIFY		0x0008		/* Make sure the data returned is modifyable */
#define     JEDI_SCANFILE_FLAGS_SKIP_READS  0x0010      /* Skip subsequent record reads */
/*
 * Flags passed to JediOpen()
 */
#define	JEDI_OPEN_NO_AUDIT					0x0001		/* Don't open the audit log for this file */



JediScanfile	* JediScanfileInit(DPSTRUCT * dp, JediFileDescriptor * fd , int flags , int threads);
int		JediScanfileRead(DPSTRUCT * dp, JediScanfile * handle );
void	JediScanfileEnd(DPSTRUCT * dp, JediScanfile * handle );

struct	jfd		* JRunFileIOOpen(DPSTRUCT * dp, char * filename , int status , int permissions );
int		JRunFileIOClose(DPSTRUCT * dp, struct jfd ** fdinfoptr , int closetype );
int		JRunFileIOSync(DPSTRUCT * dp, struct jfd * fdinfo );
int		JRunFileIORead(DPSTRUCT * dp, struct jfd * fdinfo , off_t offset , void * addr , int len );
int		JRunFileIOWrite(DPSTRUCT * dp, struct jfd * fdinfo , off_t offset , void * addr , int len );
off_t	JRunFileIOWriteEOF(DPSTRUCT * dp, struct jfd * fdinfo , void * buffer , int len );
int		JRunFileIOGetLine(DPSTRUCT * dp, struct jfd * fdinfo , char * buffer , int buffer_len);
int		JRunFileIOFrameRead(DPSTRUCT * dp, struct jfd * fdinfo , off_t base_fid , char ** frame_ptr , int * mallocflag );
int		JRunFileIOWriteFrame(DPSTRUCT * dp, struct jfd * fdinfo , off_t base_fid , void * frame_ptr , int frame_len );
int		JRunFileIOWriteFile(DPSTRUCT * dp, struct jfd * fdinfo , off_t offset , void * data_ptr , int data_len );
void	JRunFileIOLinkOverflowRelease(DPSTRUCT * dp, struct jfd * fdinfo , off_t base_fid );

int		JRunFileIOGetFd(DPSTRUCT * dp, struct jfd * fdinfo );
ino_t	JRunFileIOGetIno(DPSTRUCT * dp, struct jfd * fdinfo );
dev_t	JRunFileIOGetDevice(DPSTRUCT * dp, struct jfd * fdinfo );
mode_t	JRunFileIOGetFileMode(DPSTRUCT * dp, struct jfd * fdinfo );
ino_t	JRunFileIOGetLock1(DPSTRUCT * dp, struct jfd * fdinfo );
dev_t	JRunFileIOGetLock2(DPSTRUCT * dp, struct jfd * fdinfo );
int		JRunFileIOGetStatus(DPSTRUCT * dp, struct jfd * fdinfo );
int		JRunFileIOSetStatus(DPSTRUCT * dp, struct jfd * fdinfo , int newstatus );
char	* JRunFileIOGetFullPath(DPSTRUCT * dp, struct jfd * fdinfo );
void	JRunFileIOWriteLockStatus(DPSTRUCT * dp);
int		JRunFileIOSetRamFile(DPSTRUCT * dp , struct jfd * fdinfo, int flags);
int		JRunFileIOSleep(DPSTRUCT * dp, int microsecs);
void	jfualarm(DPSTRUCT * dp, int microsecs);

int 	JRunFileIOLockBinary(DPSTRUCT * dp, struct jfd * fdinfo , int cmd , off_t lockvalue );
int 	JRunFileIOLockGroup(DPSTRUCT * dp, struct jfd * fdinfo , int cmd , off_t lockvalue );
int 	JRunFileIOLockRecord(DPSTRUCT * dp, struct jfd * fdinfo , int cmd , char * recordkey , int keylen );
int		JRunFileIOUnlockAll(DPSTRUCT * dp);
struct	jflock	* JRunFileIOReadLockStatus(DPSTRUCT * dp, char * recordkey , int keylen );
int		JRunFileIOLockEntireFile(DPSTRUCT * dp, struct jfd * fdinfo , int flags );

void	JRunFileIOEnableLink(DPSTRUCT * dp, struct jfd * fdinfo , int framesize , int networkflag , int freespaceptr , off_t maxoffset );
int		JRunFileIOLinkRead(DPSTRUCT * dp, struct jfd * fdinfo , struct jflink * linkinfo , int lockflag , off_t base_fid );
void	JRunFileIOLinkRelease(DPSTRUCT * dp, struct jflink * linkinfo);
int		JRunFileIOWriteLink(DPSTRUCT * dp, struct jflink * linkinfo , time_t update_time);
int		JRunFileIOLinkResize(DPSTRUCT * dp, struct jflink * linkinfo , int newsize);
off_t	JRunFileIONewLink(DPSTRUCT * dp, struct jfd * fdinfo , void * data_ptr , int data_len , int data_flag , off_t base_fid , off_t * fidlist , int fidlistqty , time_t update_time , int update_flags , int update_seqid);
off_t	JRunFileIONewShortLink(DPSTRUCT * dp, struct jfd * fdinfo , void * data_ptr , int data_len);
int		JRunFileIODeleteLink(DPSTRUCT * dp, struct jfd * fdinfo , off_t base_fid);
void	JediLogError(DPSTRUCT * dp, char * msg , ...);
int 	JRunFileIOLockHash(DPSTRUCT * dp, char * recordkey , int keylen);
int		JediSObjectHash(DPSTRUCT * dp, char * recordkey , int keylen);
int		JediGetFilePath(DPSTRUCT * dp , char * rtnpath , size_t pathsize);

/*
 * Macros to get the file name from either
 * a jBC run-time variable or a jEDI file descriptor.
 */
#define	JEDI_PATHNAME(jFileDes) ((jFileDes)->fdinfo->path)

#define	JBC_PATHNAME(jVarAddr) ((jVarAddr)->VAR_FILE->fdinfo->path)

/*
 * Also, macros to get the actual names used to Open the file
 */
#define JEDI_OPEN_PATHNAME(jVarAddr) ((jFileDes)->OpenFileName)
#define JBC_OPEN_PATHNAME(jVarAddr) ((jVarAddr)->VAR_FILE->OpenFileName)
/*
 * The size of the check string in some headers.
 */
#define	JEDI_CHECK_STRING			"JBASE_FILE_CHECK"
#define	JEDI_CHECK_STRING_SIZE		16			// NOTE !! Doesn't include NULL delimiter
#define JEDI_CHECK_BUFFER_SIZE		(JEDI_CHECK_STRING_SIZE*3) // buffer in file header where check string is stored (leaving room for cipher padding)
#define JEDI_CHECK_HEADER_SIZE		(JEDI_CHECK_BUFFER_SIZE+JLEN_INT) // entire check string including length of encrypted string

/*
 * The following structure is used to pass messages TO the device driver
*/
struct	JediMessageSend {
	long	type ;										// Destination type of server, always 0x7fffffff 
	int		command ;									// Command to be executed 
	long	returntype ;								// Destination type of Jedi function, usually getpid() 
	long	openid ;									// The unique ID to pass in future requests 
	union {

		struct	{										// Sub-structure for the CLOSE request 
			int		Flags ;								// Flags allowing variations of the close command 
			} CLOSE ;

		struct	{										// Sub-structure for the WRITE request 
			int		Flags ;								// Flags allowing variations (such as locking) on the write command 
			int		RecordKeyLen;						// Length of the record key 
			int		RecordLen ;							// Total length of the record to be written 
			int		RecordThisMsg ;						// Number of bytes in 'RecordOffset' sent in this ipc message 
			int		RecordPrevious ;					// Number of bytes sent in previous ipc message for this same record 
			int		FieldNumber ;						// For field reads and writes, this is the field number 
			int		ConvertFlags;						// Conversion flags to perform, and those that are performed 
			} WRITE ;

		struct	{										// Sub-structure for the READ request 
			int		Flags ;								// Flags allowing variations (such as locking) on the read command 
			int		RecordKeyLen;						// Length of the record key 
			int		FieldNumber ;						// For field reads and writes, this is the field number 
			int		BlockNumber ;						// Starting at 0, an incrementing number per ipc message for each read 
			int		ConvertFlags;						// Conversion flags to perform, and those that are performed 
			} READ ;

		struct	{										// Sub-structure for the DELETE request 
			int		Flags ;								// Flags allowing variations (such as locking) on the delete command 
			int		RecordKeyLen;						// Length of the record key 
			} DELETE ;

		struct {										// Sub-structure for SELECT END (i.e. END of select list) 
			char	SelectId[8];						// Unique 8 byte identification of the select list 
			} SELECTEND ;

		struct {										// Sub-structure for READNEXT operation 
			char	SelectId[8];						// Unique 8 byte identification of the select list 
			} READNEXT ;

		struct {										// Sub-structure for LOCK operations (includes unlock) 
			int		Flags ;								// Type of operation -- see the relevant JEDI_RECORD_xxx bits defined 
			int		RecordKeyLen ;						// Length of any associated record key 
			} LOCK ;

		struct	{										// Sub-structure for the IOCTL request 
			int		SubCommand ;						// The actual command for the IOCTL to execute 
			int		TextLen ;							// Number of bytes that follow the Send structure as command modifier 
			} IOCTL ;

		} SendUnion ;
	} ;

/*
 * The ID of messages sent to the jedi External driver.  This must not
 * clash with a valid process id.
 */
#define	JEDI_MESSAGE_ID_SERVER		0x80000

/*
	The following commands may be entered into the Command member of the above structrue :
*/
#define	JEDI_MESSAGE_COMMAND_CLOSE			1			/* Close a file. */
#define	JEDI_MESSAGE_COMMAND_READ			2			/* Read a record from file */
#define	JEDI_MESSAGE_COMMAND_WRITE			3			/* Write a record to file */
#define	JEDI_MESSAGE_COMMAND_DELETE			4			/* Delete a record from file */
#define	JEDI_MESSAGE_COMMAND_CLEAR			5			/* Clear the file */
#define	JEDI_MESSAGE_COMMAND_SELECT			6			/* Start selecting all records in devices. */
#define	JEDI_MESSAGE_COMMAND_SELECT_END		7			/* Call when select processing completed for a device. */
#define	JEDI_MESSAGE_COMMAND_READNEXT		8			/* Read the next record key following a SELECT */
#define	JEDI_MESSAGE_COMMAND_IOCTL			9			/* Various IOCTL functions, see the 'JEDI_IOCTL_xxx' commands */
#define	JEDI_MESSAGE_COMMAND_SYNC			10			/* Synchronise device, flushing any data in cache/buffers out to device */
#define	JEDI_MESSAGE_COMMAND_LOCK			11			/* Various locking mechanisms, such as releasing locks */

	/*	The following are for internal use only */
#define	JEDI_MESSAGE_CHILD_EXIT				2000		/* Close down child on shared memory models */

/*
 * The following structure is used to recieve message back FROM the device driver.
 */
struct	JediMessageRecieve {
	long	type ;										// Destination of Jedi function, usually same as returntype 
	int		returncode ;								// Return code from the server process, 0 is success otherwise errno 
	union {

		struct	{										// Sub structure for the WRITE request 
			int		Dummy1;								// Just a dummy entry 
			} WRITE ;

		struct	{										// Sub-structure for the READ request 
			int		ConvertFlags;						// Conversion flags to perform, and those that are performed 
			int		RecordSize;							// Total length of the record that will eventually be passed back 
			int		RecordThisMsg ;						// Number of bytes sent in the record in this message 
			int		RecordPrevious ;					// Total number of bytes sent in previous messages (excluding this msg) 
			int		RecordOffset ;						// Offset from start of this message that the record data follows 
			} READ ;

		struct {										// Sub-structure for SELECT operation 
			char	SelectId[8];						// Unique 8 byte identification of the select list 
			} SELECT ;

		struct {										// Sub-structure for READNEXT operation 
			int		RecordKeyLen ;						// Length of record key returned 
			} READNEXT ;

		struct	{										// Sub-structure for the IOCTL request 
			int		ReturnLen ;							// Length of any returned data 
			} IOCTL ;

		} RecieveUnion ;
	} ;

/*
 * The following structure is returned by the external device driver to the caller
 *    when the device driver is first loaded.
 */
struct	JediMessageAcknowledge {
	long	type ;														// Type of message sent 
	int		ReturnCode ;												// 0 shows the OPEN is okay, +ve is an errno failure 
	long	openid ;													// Unique ID to use on all future calls 
	int		NewMessageId ;												// Message queue to use in future. 
	char	ErrorMessage[128] ;											// If open fails, this is the error message 
	} ;
/*
 * Do some #defines to make elements of the send and recieve buffer easier to understand.
 */
#define		WRITE_FLAGS								SendUnion.WRITE.Flags
#define		WRITE_RECORD_KEY_LEN					SendUnion.WRITE.RecordKeyLen
#define		WRITE_RECORD_LEN						SendUnion.WRITE.RecordLen
#define		WRITE_RECORD_THIS_MSG					SendUnion.WRITE.RecordThisMsg
#define		WRITE_RECORD_PREVIOUS					SendUnion.WRITE.RecordPrevious
#define		WRITE_FIELD_NUMBER						SendUnion.WRITE.FieldNumber
#define		WRITE_CONVERT_FLAGS						SendUnion.WRITE.ConvertFlags

#define		READ_FLAGS								SendUnion.READ.Flags
#define		READ_RECORD_KEY_LEN						SendUnion.READ.RecordKeyLen
#define		READ_FIELD_NUMBER						SendUnion.READ.FieldNumber
#define		READ_BLOCK_NUMBER						SendUnion.READ.BlockNumber
#define		READ_CONVERT_FLAGS						SendUnion.READ.ConvertFlags
#define		READ_RECORD_SIZE						RecieveUnion.READ.RecordSize
#define		READ_RECORD_THIS_MSG					RecieveUnion.READ.RecordThisMsg
#define		READ_RECORD_PREVIOUS					RecieveUnion.READ.RecordPrevious

#define		DELETE_FLAGS							SendUnion.DELETE.Flags
#define		DELETE_RECORD_KEY_LEN					SendUnion.DELETE.RecordKeyLen

#define		SELECT_ID								RecieveUnion.SELECT.SelectId

#define		SELECTEND_ID							SendUnion.SELECTEND.SelectId

#define		READNEXT_ID								SendUnion.READNEXT.SelectId
#define		READNEXT_RECORD_KEY_LEN					RecieveUnion.READNEXT.RecordKeyLen

#define		LOCK_FLAGS								SendUnion.LOCK.Flags
#define		LOCK_RECORD_KEY_LEN						SendUnion.LOCK.RecordKeyLen

#define		IOCTL_COMMAND							SendUnion.IOCTL.SubCommand
#define		IOCTL_TEXTLEN							SendUnion.IOCTL.TextLen
#define		IOCTL_RETURNLEN							RecieveUnion.IOCTL.ReturnLen

#endif

#define JEDI_JRFS_DRIVER "JediInitREMOTE"
