/*****************************************************************************
 *
 * Copyright (c) 2019 jBASE International. All rights reserved
 * 
 * Include file for jBASE embedded SQL.
 *
 * This header contains some small support functions, for jBASE embedded
 * SQL.  This file is automatically included when required.
 *
 * SQL_SETRESULT()
 *	Called automatically after each embedded SQL statement to Assign the
 *	result to SYSTEM(0).  This can then be tested from within jBASE 
 *	to get the status of each statement, eg
 *
 *		EXEC SQL connect :DataBase;
 *		IF SYSTEM(0) THEN CRT "Connect failed"; STOP
 *
 * SQL_ERROR()
 *	Prints the SQL error, this is only called if set to do so (see below)
 *
 * SQL_ABORT()
 *	Prints the SQL error and Exits (Result of 1).
 *
 * There is no default action on error, but this can be configured by
 * including the following embedded SQL statements in your source code.
 *
 * ingres *	exec sql whenever sqlerror call SQL_ERROR;
 * oracle *	exec sql whenever sqlerror do SQL_ERROR();
 * ibmdb2 *	exec sql whenever sqlerror goto proglab_DOSQLERR;
 * mssql  *	exec sql whenever sqlerror goto proglab_DOSQLERR;
 *        *	DOSQLERR:
 *        *	  DEFC INT SQL_ERROR()
 *        *	  SQLERROR();
 *****************************************************************************/
#include <jsystem.h>
#include <assert.h>

#ifdef WIN32

 /* error structure used by jbase error functions */
#ifndef JBC_SQLCODE
struct     sqlca
{
	unsigned char  sqlcaid[8];
	long           sqlcabc;
	long           sqlcode;
	short          sqlerrml;
	unsigned char  sqlerrmc[70];
	unsigned char  sqlerrp[8];
	long           sqlerrd[6];
	unsigned char  sqlwarn[11];
	unsigned char  sqlstate[5];
};
//#define      SQLCODE   sqlca.sqlcode
#define      SQLWARN0  sqlca.sqlwarn[0]
#define      SQLWARN1  sqlca.sqlwarn[1]
#define      SQLWARN2  sqlca.sqlwarn[2]
#define      SQLWARN3  sqlca.sqlwarn[3]
#define      SQLWARN4  sqlca.sqlwarn[4]
#define      SQLWARN5  sqlca.sqlwarn[5]
#define      SQLWARN6  sqlca.sqlwarn[6]
#define      SQLWARN7  sqlca.sqlwarn[7]
#define      SQLWARN8  sqlca.sqlwarn[8]
#define      SQLWARN9  sqlca.sqlwarn[9]
#define      SQLWARNA  sqlca.sqlwarn[10]
#define      SQLSTATE  sqlca.sqlstate

struct sqlca sqlca;

#endif

#endif


 #if defined(JBC_DB2SQL)||(JBC_DB2)
 #include <sql.h>
 #endif
/*****************************************************************************
*
* Function:             SQL_SETRESULT()
*
* Description:  Assign SYSTEM(0) to the result of the last SQL statement.
*
* Parameters:   None
*
* Return Value: Always 0.
*
* Notes:                None.
*
*******************************************************************************/
#define SQL_SETRESULT()	SQLSETRESULT(dp)
INT32   SQLSETRESULT(DPSTRUCT *dp)
{
        VAR             scratchvar ;
		assert( dp != NULL );
        VAR_INITIALISE_VB(&scratchvar);
        STORE_VBI(&scratchvar , sqlca.sqlcode);
        ASSIGNSYS_VBI(&scratchvar, 0);
        return(0);
}

/*****************************************************************************
*
* Function:             SQL_ERROR()
*
* Description:  Display The SQL Error
*
* Parameters:   None
*
* Return Value: None
*
* Notes:                None.
*
*******************************************************************************/
#define SQL_ERROR()	SQLERROR(dp)	/* Circumvent problem with call _ cfuncs */
INT32   SQLERROR(DPSTRUCT *dp)
{
char StateBuffer[1024];
char ErrorBuffer[1024];
int BuffLen = sizeof(ErrorBuffer);
int MsgLen = 0;

		assert( dp != NULL );
		printf ("\nError: sqlcode : %d\n",sqlca.sqlcode);

#if defined(JBC_DB2SQL)||(JBC_DB2)
		sqlogstt(StateBuffer,1024,80, sqlca.sqlstate);
		sqlaintp(ErrorBuffer,1024,80, &sqlca);
        printf ("Error: sqlstate: %s\n", StateBuffer);
        printf ("Error: sqlerror: %s\n", ErrorBuffer);
#endif

#if defined(JBC_ORACLE)
		/*sqlglm(ErrorBuffer, &BuffLen, &MsgLen);*/
		/*print ("Error: sqlerror: %.*s\n", MsgLen, ErrorBuffer);*/
		printf("Error: % .70s\n", sqlca.sqlerrm.sqlerrmc);
#endif

#if defined(JBC_INGRES)
		EXEC SQL BEGIN DECLARE SECTION;
			char Errmsg[101];
		EXEC SQL END DECLARE SECTION;
		EXEC SQL INQUIRE SQL (:Errmsg = ERRORTEXT) ;
		printf("Error: %s\n",Errmsg);
#endif

#if defined(JBC_INFORMIX)
		rgetmsg(sqlca.sqlcode, ErrorBuffer, sizeof(ErrorBuffer));
		printf("Error: %s\n",ErrorBuffer);
#endif

#if defined(JBC_MSSQL)
		printf("Error: %.*s\n",sqlca.sqlerrml,sqlca.sqlerrmc);
#endif

#if defined(JBC_SYBASE)
		printf("Error: %s\n", sqlca.sqlerrm.sqlerrmc);
#endif
        return(0);
}

/*****************************************************************************
*
* Function:             SQL_ABORT()
*
* Description:  Terminate with the SQL error message
*
* Parameters:   None
*
* Return Value: None
*
* Notes:                None.
*
*******************************************************************************/
#define SQL_ABORT()	SQLABORT(dp)	/* Circumvent problem with call _ cfuncs */
INT32   SQLABORT(DPSTRUCT *dp)
{
		assert( dp != NULL );
        SQL_ERROR();
        EXIT_VI(1);
	return(0);
}
