/*
 *	This #include file contains all the necessary definitions
 *	for the 'C' code generated by the jBASE BASIC compiler.
 *
 *Copyright (c) 2019 jBASE International. All rights reserved
 */

#ifndef	JSYSTEM_H
#define	JSYSTEM_H
/*
 *Define _REENTRANT for Posix threads conformance.
 */
#ifndef	_REENTRANT
#define	_REENTRANT
#endif

#ifdef __GNUC__
#ifndef GCC_VERSION
#define GCC_VERSION (__GNUC__ *1000 + __GNUC_MINOR__)
#endif /*GCC_VERSION */
#endif /*__GNUC__ */

#ifdef __OS400__

/*OS400 doesn't use PAGESIZE.  Dummy to 4096 and see if it still works */
#define PAGESIZE	4096
#endif

// Configure define to indicate jBASE Major and Minor Versions
#define JVER_MAJOR 5
#define JVER_MINOR 2

#ifndef JBASE_EMULATE_INT_GENERIC_PRIME
#define   JBASE_EMULATE_INT_GENERIC_PRIME  30    /* Offset of 'generic prime system' emulation setting */
#endif

#ifndef DPSTRUCT_DEF
typedef	struct jBASEDataAreas DPSTRUCT;
#define	DPSTRUCT_DEF
#endif


#ifdef WIN32
/*
 *Unfortunately, microsoft also have a typedef for FLOAT (which is a float
 *and not a double as in jBASE).  Therefore if a module includes "windows.h"
 *and "jsystem.h" then there will be a conflict.  There are two solutions,
 *1) we change ours to be jFLOAT (in *all*modules that use it, and in the
 *documentation, or
 *2) we #define it to be JFLOAT here. 
 */
#define FLOAT	JFLOAT

// Turn off warnigns about unreferenced formal parameters as there are too many
// of these to fix easilly.
//
#pragma warning( disable : 4100 )
#pragma warning( disable : 4996 )

#ifdef	__cplusplus
#ifdef	__DECCXX
#pragma message disable 161
#endif
extern	"C"	{
#endif

// This is strange, it seems to object to he use of dp in the protype but I thikn
// it is erroneoud (level 4 warning only with /Za).
//
#pragma warning( disable : 4115 )

int jBaseCallSubroutine(DPSTRUCT *dp, char *SubName, int ArgCount, ...);

#pragma warning( default : 4115 )

int  JLibGetPrecision(DPSTRUCT *dp);
#ifdef	__cplusplus
}
#endif
#endif

/*
 *Make sure we set up the proper export definition if we are
 *in the Win32 envirionment.
 */
#ifdef	WIN32
#define	JBASE_EXPORT __declspec(dllexport)
#else
#define	JBASE_EXPORT
#endif
/*
 *To ensure thread safety, the first include file
 *has to be pthread.h
 */
#ifdef WIN32
#include <winpthread.h>
#else
#include <pthread.h>
#endif
#include <stdio.h>
#ifdef	__cplusplus
extern	"C"	{
#endif
#include <jportC.h>
#include <sys/types.h>
#include <fcntl.h> /*include this before jedi.h else _SC_PAGESIZE is not defined. */
#include <jedi.h>
#include <stdarg.h>

#ifdef	JBC_SHDEBUG
int		SmartHeap_jmalloc;
#define	MEM_DEBUG
#include	<smrtheap.h>
#endif
typedef int					INT32;
typedef unsigned int		UINT32;
typedef short int  			INT16;
typedef unsigned short int  UINT16;
typedef	double				FLOAT;

/* 
 * IEE754 quaruple type.
 */
typedef	long double			LONG_FLOAT;

typedef	unsigned char 		STRING;
#define	JCONST const

#define	TRUE	1
#define	FALSE	0

struct JObjectWrapper;


/*******************************************************************************
* 
* Support for BIG_INTEGERS
*
*******************************************************************************/
/* Special cases that can go in BIG_INT flags. */
#define BIG_INT_NAN 0x0001
#define BIG_INT_INF 0x0002

/* Stuff related to the flags sign bit */
#define BIG_INT_NEG 0x0004
#define BI_SET_POS(x)	((x)->flags &= (BIG_INT_NEG ^ 0xffff))
#define BI_SET_NEG(x)	((x)->flags |= BIG_INT_NEG)
#define BI_IS_POS(x)	(!((x)->flags & BIG_INT_NEG))
#define BI_DUP_SIGN(x, y) 	if (BI_IS_POS(y)) \
								BI_SET_POS(x); \
							else \
								BI_SET_NEG(x);

/*******************************************************************************
*
* The BIG_INTEGER structure.
*
* This has been tuned to be the same size as other VAR union members. This 
* should mean that previously compiled BASIC programs will still work.
*
*******************************************************************************/
struct BIG_INT
{
    /*
     * Pointer to the number. This will actually be a pointer to BIGD as
     * defined in BigDigits however nothing other than jlibBIgInt.c has any
     * reason to mess with it so declare it opaque here.
     */
    void    *num;

    /* 
	 * Any flags. One day this will be used to indicate 'special' states of 
	 * other VAR_TYPES but for now just handles BIG_INTEGERS.
	 *
	 * Also includes sign bit
	 */
    UINT16  flags;

	/*
	 * Scale. This will eventually take over from FLOAT_SCALE. It is useful 
	 * for this to be part og BIG_INT so a BIG_INT can be passed around as a
	 * complete 'object'.
	 */
	INT16	scale;
};

/* Define size of VAR integer field */
#define VARINT INT64
#define UVARINT UINT64

/*******************************************************************************
* 
* Declaration or the VAR structure
*
*******************************************************************************/
struct	VARHeader
{
	unsigned short	Type ;					// See the VAR_TYPE_xxx values
	unsigned char	Flags ;					// See the VAR_FLAGS_xxx bits
	unsigned char	FloatScale ;			// The scaling (or precision) applied to a float
	unsigned char	Reserved ;				// Modified flag, CI

	// Used for storing floating point numbers, and other stuff as long as the header struct stays the same size.
	union {
		LONG_FLOAT	LongFloat;
		FLOAT		ShortFloat;
		void * UString;						// Stores an UPPER case version of the current value,  constant or VAR. keep here for backwards bin compat
	} Float ;
	VARINT			Integer ;				// Used for storing integer values
};

typedef	struct VAR_dummy
{
	struct	VARHeader	VarHeader ;
	/*
	 *The remainder of the layout depends upon the type in the 'Type' byte.
	 */
	union 
	{
		struct
		{
			unsigned short  StringConstLen ;			// Length of string
			unsigned char   Stringdata [6];				// a constant string
		}
		ConstString;
		struct
		{
			STRING			*StringAddress ;			// Address of any string
			INT32			StringLen ;					// Length of any string
		}
		String ;
        struct
		{
			JediFileDescriptor  	*FilePointer ;		// Pointer to the opened file descriptor
			struct JediSelectPtr	*SelectPointer ;	// Pointer to a select list
		}
		File ;
		struct	IndexFileVar		*IndexPointer ;	// Indexing functionality
		struct	JArrayDynamic		*ArrayPointer ;	// DIMensioned array pointer
		struct	JObjectWrapper      *ObjectPointer;	// Pointer to memory structure
		struct jdynamic	* DynamicObjectPointer ;			// Pointer to dynamic object

#ifdef JBUILD_OPTION_BIGINT_MATH
		// Used for storing BigDigit numbers
		struct BIG_INT	BigInt;	
#endif /* JBUILD_OPTION_BIGINT_MATH */				

	}
	VarUnion;

	/* Copied from CVar */
#ifdef WIN32
	UINT32		lastThreadId;
#else
	pthread_t	lastThreadId;
#endif
	struct	jBASEDataAreas * dp;
}
VAR;


/*
 * Mask for top half of VAR->Integer set. Used in overflow prediction.
 */
#define VAR_INTEGER_TOP_HALF_MASK 0xffffffff00000000UL

#ifdef JBUILD_OPTION_BIGINT_MATH
/* Function for freeing BIG_INT part of a variable */
void freeVARBIGINT (VAR *var);
#endif /* JBUILD_OPTION_BIGINT_MATH */

/*
 *Define the types in the 'Type' member.
 */
#define	VAR_TYPE_UNASSIGNED				0x0000		// VARiable is unassigned
#define	VAR_TYPE_INTEGER				0x0001		// The 'Integer' member has a valid value
#define	VAR_TYPE_FLOAT					0x0002		// The 'Float' member has a valid scaled number
#define	VAR_TYPE_STRING					0x0004		// The 'StringAddress' and 'StringLen' are valid
#define	VAR_TYPE_SELECT					0x0008		// The 'VarUnion.File.SelectPointer' is in use
#define	VAR_TYPE_FILE					0x0010		// The 'VarUnion.File.FilePointer' is in use
#define	VAR_TYPE_INDEX					0x0020		// The 'VarUnion.IndexPointer' is in use
#define	VAR_TYPE_ARRAY					0x0040		// The 'VarUnion.ArrayPointer' is in use
#define	VAR_TYPE_REAL					0x0080		// The 'Float' member has a real IEEE float
#define	VAR_TYPE_FIELD_NUMBER			0x0100		// Set when a string and field_number is valid
#define	VAR_TYPE_REMOVE_POINTER			0x0200		// Set when a string and remove_pointer is valid
#define	VAR_TYPE_STRING_CONSTANT		0x0800		// String constant
/*
 *The difference between VAR_TYPE_FLOAT_CONSTANT and
 *VAR_TYPE_FLOAT_CONSTANT_STRING is that the former, when converted
 *to a string, does not equal the latter. Hence "0.0" would be
 *the former as when converted to a string would be different, "".
 */
#define	VAR_TYPE_FLOAT_CONSTANT			0x1000		// Scaled float constant
#define	VAR_TYPE_FLOAT_CONSTANT_STRING	0x2000		// Scaled float constant with equivalent string
/*
 * These next two types share the same flag bit. They are separated
 * by VAR_FLAGS_DYNAMIC_OBJECT being set.
 */
#define	VAR_TYPE_OBJECT_WRAPPER			0x4000		// Pointer to memory structure (32 or 64 bit)
#define	VAR_TYPE_DYNAMIC_OBJECT			0x4000		// It is a dynamic object

/*
 * Pointer to a BigDigits number (arbitary sized int).
 */
#define	VAR_TYPE_BIG_INT				0x8000

/* String/Constant but USTRING ptr */
#define	VAR_TYPE_STRING_USTRING			0x0400
/* Dynamic object */

/*
 *Define some macros to easy access the members of VAR.
 */
#define	VAR_TYPE						VarHeader.Type
#define	VAR_FLAGS						VarHeader.Flags
#define	VAR_FLOAT						VarHeader.Float.ShortFloat
#define	VAR_FLOAT_SCALE					VarHeader.FloatScale
#define	VAR_INTEGER						VarHeader.Integer
#define	VAR_REAL						VarHeader.Float.ShortFloat
#define	VAR_SELECT						VarUnion.File.SelectPointer
#define	VAR_FILE						VarUnion.File.FilePointer
#define	VAR_INDEX						VarUnion.IndexPointer
#define	VAR_ARRAY						VarUnion.ArrayPointer
#define	VAR_STRING_LEN(Source)			JLibELEN_IB(dp, Source)
#define	VAR_STRING_ADDR(Source)			JLibEADDR_SB(dp, Source)
#define	VAR_OBJECT_WRAPPER				VarUnion.ObjectPointer
#define	VAR_DYNAMIC_OBJECT				VarUnion.DynamicObjectPointer

#ifdef JBUILD_OPTION_BIGINT_MATH
#define	VAR_BIG_INTEGER					VarUnion.BigInt
#define	VAR_BIG_INT_NUM					VarUnion.BigInt.num
#define	VAR_BIG_INT_FLAGS				VarUnion.BigInt.flags
#define	VAR_BIG_INT_SCALE				VarUnion.BigInt.scale
#endif /* JBUILD_OPTION_BIGINT_MATH */

#define	VAR_FLAGS_USER					0x0001			// The variable is a normal user defined variable
#define	VAR_FLAGS_CONSTANT				0x0002			// The variable is a program constant
#define	VAR_FLAGS_REGISTER				0x0004			// Tis a register variable
#define	VAR_FLAGS_SPECIAL				0x0008			// Special case such as @USER_THREAD
#define VAR_FLAGS_DYNAMIC_OBJECT		0x0010			// Shows the TYPE flag is a dynamic object

/*	
 *Define default depth the GOSUB stack.
 */
#define	GOSUB_STACK_MAX		512
/*
 *Define the structure for recording details of a dimensioned array.
 *This structure is used for non-dynamic arrays.
 */
struct	JArrayInfo
{
	int		VarsQty;									// Total number of variables this array occupies
	int		ArrayQty;									// Number of elements in dimensioned array
	int		ArrayDims[10];								// Size of each element
} ;
/*
 *Define the structure for recording details of a dimensioned array.
 *This is new for release 3.1 and is used for dynamic arrays, where applicable.
 */
struct	JArrayDynamic
{
	int		VarsQty ;				// Total number of variables this array occupies
	int		ArrayQty ;				// Number of dimensions
	int		ArrayDims[10] ;			// Size of each dimension
	VAR		*FirstVar ;				// Address of first variable
} ;
/*
 *Define the structures that defines the current subroutine .
 *The StaticLevelData is the static data that is initialised as initialised
 *   data in a user subroutine. It remains constant nomatter how many levels
 *   of recursion we are in.
 *The StackLevelData is the data allocated on the function stack, one copy of
 *  which exists per subroutine level.
 */
struct StaticLevelData
{
	int		compiled_precision ;						// The PRECISION the source was compiled at
	FLOAT	compiled_precmult ;							// Value of 10 ^ compiled_precision
	int		UserVarQty;									// Total number of user variables
	int		RegVarQty;									// Total number of register variables
	int		ParmVarQty ;								// Total number of subroutine parameters
	int		StaticFlags ;								// General usage flags
} ;

struct	StackLevelData
{
	int		GosubStack[GOSUB_STACK_MAX];				// The GOSUB stack
	int		GosubNext ;									// Pointer to next entry in GosubStack
	int		SaveDebugLineAndFile ;						// Save area for the debugger information
	va_list	ap ;										// Used for varargs
} ;
/*
 *Structure to complete the information required per CALL level.
 */
struct	CALLStack
{
	struct	StaticLevelData	*static_level_data ;		// The static constant data
	struct	StackLevelData	*stack_level_data ;		// The stack data used by the function
	VAR		**SubroutineParms ;							// Pointer to a list of VAR's where addresses of subroutine parms stored
	VAR		*FirstLocalVar ;							// Pointer to the first variable used by the function
	JCONST unsigned char *ConstantText ;				// Pointer to the constant text space
	int		current_precision ;							// The PRECISION currently in force
	FLOAT	current_precmult ;							// Value of 10 ^ current_precmult
	struct jvararg * vararg ;							// Any varargs in use
} ;

/*
	Define all the possible exit() values. Values 1 to 200 are
	reserved for use in the error message file.
*/
#define JBC_EXIT_OKAY						0			// Exit point normal exit
#define JBC_EXIT_FATAL 						201			// Non-determinal fatal error
#define	JBC_EXIT_DEBUGQUIT 					203			// 'Q' typed in debugger
#define	JBC_EXIT_DEBUGABORT					204			// 'A' typed in debugger
#define JBC_EXIT_INV_ERROR_MESSAGE 			205			// Cannot open the error message file
#define	JBC_EXIT_NOSPACE					206			// No space and continued from debugger
#define	JBC_EXIT_RUNTIME_ERROR 				207			// We unexpecteldly came out of the debugger
#define	JBC_EXIT_NOERRMSG		 			208			// There is no opened error message file for an error message
#define	JBC_EXIT_HEADER			 			209			// The user entered <CTRL> E or <CTRL> X during a HEADING pause
#define	JBC_EXIT_SIGNAL			 			210			// A 'SIGTERM' signal was caught
#define	JBC_EXIT_NOLOAD						211 		// Unable to load original shared object on AIX systems
#define	JBC_EXIT_CONNECTION					212			// Cannot connect to the requested database
#define	JBC_EXIT_JBASECONNECT				213			// Cannot connect to jBASE demon
#define	JBC_EXIT_DEMONKILL					214			// Killed by the demon (usually an ABORT etc in a sibling)
#define	JBC_EXIT_OFF						215			// User logged off by the OFF or LOGOFF command
#define	JBC_EXIT_NO_EMULATE					216			// Unable to find the Config_EMULATE file
/*
 *Define the bits in the debugger flag.
 */
#define	DB_REASON_TRACE_LINES			0x00000001			// We are tracing each line of source
#define	DB_REASON_INTERRUPT				0x00000002			// We have seen an interrupt, or -D option used
#define	DB_REASON_ERROR_MESSAGE			0x00000004			// An entry in the error file causes this to be called
#define	DB_REASON_OPCODE				0x00000008			// The DEBUG opcode in the source was seen
#define	DB_REASON_STEP_LINES			0x00000010			// Single step each source line
#define DB_REASON_STEP_LINES_SAME_CALL	0x00000020			// Single step each source lines, only if in same CALL level
#define DB_REASON_STEP_LINES_SAME_GOSUB	0x00000040			// Single step each source lines, only if in same GOSUB level
#define	DB_REASON_BPOINTS				0x00000080			// There is one or more breakpoint set
#define	DB_REASON_DOPTION				0x00000100			// -d option seen on command line
#define	DB_REASON_NEWFILE				0x00000200			// Break when a different file name is executed
#define	DB_REASON_FILECHANGED			0x00000400			// Bit is set whenever a CALL or RETURN from subroutine executed
#define	DB_REASON_HISTORY				0x00000800			// Keep rolling history of all the lines traced
#define	DB_REASON_UPDATE_STATUS			0x00002000			// Update the process status in shared memory
#define	DB_REASON_FILE_ERROR			0x00004000			// A file I/O operation generated an error.
#define	DB_REASON_JCOVER				0x00008000			// The jCOVER option is in use and each statement needs to be traced
#define	DB_REASON_EXIT					0x00010000			// Need to exit for some reason e.g. SIGHUP
#define	DB_REASON_UPDATE_TIMES			0x00020000			// Update the 'cpu' member of thread data
#define	DB_REASON_PROFILE				0x00040000			// Seen a profile tick.
#define DB_REASON_DINHOPTION			0x00080000			// -D option seen on command line, inherit the debug mode

#define		JBC_AT_DAY						0
#define		JBC_AT_MONTH					1
#define		JBC_AT_YEAR						2
#define		JBC_AT_SYS_BELL					5
#define		JBC_AT_UDTNO					6
#define		JBC_AT_WHO						7
#define		JBC_AT_ACCOUNT					8
#define		JBC_AT_SYSTEM_RETURN_CODE		9
#define		JBC_AT_USER_RETURN_CODE			10
#define		JBC_AT_LEVEL					11
#define		JBC_AT_COMMAND					12
#define		JBC_AT_PATH						13
#define		JBC_AT_APPLICATION_ID			14
#define		JBC_AT_TTY						15
#define		JBC_AT_SELECTED					16
#define		JBC_AT_LPTRHIGH					17
#define		JBC_AT_ID						18
#define		JBC_AT_RECORD					19
#define		JBC_AT_FILENAME					20
#define		JBC_AT_TERM_TYPE				21
#define		JBC_AT_UID						22
#define		JBC_AT_DATA						23
#define		JBC_AT_EOF						24
#define		JBC_AT_HEADER_BREAK				25
#define		JBC_AT_FOOTER_BREAK				26
#define		JBC_AT_LOCALE					27
#define		JBC_AT_CODEPAGE					28
#define		JBC_AT_TIMEZONE					29
#define		JBC_AT_CALLSTACK				30
#define		JBC_AT_PID						31
#define		JBC_AT_USER_ROOT				32
#define		JBC_AT_USER_THREAD				33
#define		JBC_AT_USERSTATS				34
#define		JBC_AT_USER_OSNAME				35
#define		JBC_AT_PIB						36
/*
 * Define the maximum for these
 */
#define		JBC_AT_MAX_VALUE				128
/*
**Define @var macros generated by compiler
*/
#define		_jbat_USER_RETURN_CODE			ATFUNC_BI(JBC_AT_USER_RETURN_CODE)
#define		_jbat_APPLICATION_ID			ATFUNC_BI(JBC_AT_APPLICATION_ID)
#define		_jbat_ID						ATFUNC_BI(JBC_AT_ID)
#define		_jbat_RECORD					ATFUNC_BI(JBC_AT_RECORD)
#define		_jbat_FILENAME					ATFUNC_BI(JBC_AT_FILENAME)
#define		_jbat_HEADER_BREAK				ATFUNC_BI(JBC_AT_HEADER_BREAK)
#define		_jbat_FOOTER_BREAK				ATFUNC_BI(JBC_AT_FOOTER_BREAK)
#define		_jbat_USER_ROOT					ATFUNC_BI(JBC_AT_USER_ROOT)
#define		_jbat_USER_THREAD				ATFUNC_BI(JBC_AT_USER_THREAD)
#define		_jbat_USERSTATS					ATFUNC_BI(JBC_AT_USERSTATS)

void	JLibINIT				(DPSTRUCT *dp);
void	JLibEXIT				(DPSTRUCT *dp);
FLOAT	JLibDACOS_FF			(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDASIN_FF			(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDATAN_FF			(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibBPREC_LOSE_FF		(DPSTRUCT *dp, FLOAT Source);
FLOAT	JLibBPREC_GAIN_FF		(DPSTRUCT *dp, FLOAT Source);
INT32	JLibBPREC_LOSE_II		(DPSTRUCT *dp, INT32 Source);
INT32	JLibBPREC_GAIN_II		(DPSTRUCT *dp, INT32 Source);
INT32	JLibBINT_IF				(DPSTRUCT *dp, FLOAT Value);
FLOAT	JLibBCONV_FB			(DPSTRUCT *dp, VAR *Source );
FLOAT	JLibBCONV_FI			(DPSTRUCT *dp, INT32 IntValue);
void	JLibCSetPrecision_VI	(DPSTRUCT *dp, int newprecision);
VAR		*JLibDABS_BBB			(DPSTRUCT *dp, VAR *target, VAR *source ) ;
VAR		*JLibDREUSE_BB			(DPSTRUCT *dp, VAR *DynamicArray) ;
VAR		*JLibDNEG_BBB			(DPSTRUCT *dp, VAR *target, VAR *source ) ;
VAR		*JLibDRND_BBB			(DPSTRUCT *dp, VAR *target, VAR *source ) ;
VAR		*JLibDMOD_BBBB			(DPSTRUCT *dp, VAR *target, VAR *source1, VAR *source2 ) ;
FLOAT	JLibDCOS_FF				(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDDIV_FFF			(DPSTRUCT *dp, FLOAT Value1, FLOAT Value2 );
FLOAT	JLibDEXP_FF				(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDLN_FF				(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDMUL_FFF			(DPSTRUCT *dp, FLOAT Value1, FLOAT Value2 );
VAR		*JLibDPOSTD_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibDPOSTI_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibDPRED_BB			(DPSTRUCT *dp, VAR *Source );
VAR		*JLibDPREI_BB			(DPSTRUCT *dp, VAR *Source );
void	JLibDPOSTD_VB			(DPSTRUCT *dp, VAR *Source );
void	JLibDPOSTI_VB			(DPSTRUCT *dp, VAR *Source );
FLOAT	JLibDPWR_FFF			(DPSTRUCT *dp, FLOAT Value1, FLOAT Value2 );
FLOAT	JLibDSIN_FF				(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDSQRT_FF			(DPSTRUCT *dp, FLOAT Value );
FLOAT	JLibDTAN_FF				(DPSTRUCT *dp, FLOAT Value );

/* 
 * 32 bit conversions.
 */
INT32	JLibBCONV_IB			(DPSTRUCT *dp, VAR *Source );
UINT32	JLibBCONV_UB			(DPSTRUCT *dp, VAR *Source );
INT32	JLibBCONVCOND_IB		(DPSTRUCT *dp, VAR *Source );
void	JLibBSTORE_VBI			(DPSTRUCT *dp, VAR *Target, INT32 Value );

/* 
 * 64 bit conversions.
 *
 * NOTE : NOT BACKWARDS COMPATIBLE WITH OLDER jBASE BUILDS
 */
VARINT	JLibBCONV_VARINT		(DPSTRUCT *dp, VAR *Source );
UVARINT	JLibBCONV_UVARINT		(DPSTRUCT *dp, VAR *Source );
VARINT	JLibBCONVCOND_VARINT	(DPSTRUCT *dp, VAR *Source );
void	JLibBSTORE_VARINT		(DPSTRUCT *dp, VAR *Target, VARINT Value );

INT32	JLibBCONV_IF			(DPSTRUCT *dp, FLOAT FloatValue);
void	JLibCFORINIT_VBII		(DPSTRUCT *dp, VAR *ControlVar, INT32 OriginatingValue, INT32 Step) ;
void	JLibCFORINIT_VBBB		(DPSTRUCT *dp, VAR *ControlVar, VAR *OriginatingValue, VAR *Step);
void	JLibCFORINIT_VBBBB		(DPSTRUCT *dp, VAR *ControlVar, VAR *OriginatingValue, VAR *Step, VAR *To);
int		JLibCFOR_IBII			(DPSTRUCT *dp, VAR *ControlVar, INT32 FinalValue, INT32 Step );
int		JLibCFOR_IBBB			(DPSTRUCT *dp, VAR *ControlVar, VAR *FinalValue, VAR *Step );
INT32	JLibCONGOSUB_III		(DPSTRUCT *dp, INT32 Value, INT32 Max );
INT32	JLibCONGOTO_III			(DPSTRUCT *dp, INT32 Value, INT32 Max );
INT32	JLibCPROCREAD_IB		(DPSTRUCT *dp, VAR *Result );
INT32	JLibCPROCWRITE_IB		(DPSTRUCT *dp, VAR *Source );
INT32 	JLibCOBJEXCALLBACK_IBB	(DPSTRUCT *dp, VAR *Param1, VAR *Param2 );
INT32	JLibDCOMPARE_IBBI		(DPSTRUCT *dp, VAR *Value1, VAR *Value2, INT32 CompareType );
INT32	JLibDNUM_IB				(DPSTRUCT *dp, VAR *Source );
INT32	JLibDCMP_IBBI			(DPSTRUCT *dp, VAR *result, VAR *source, INT32 compare_type) ;
INT32	JLibEALPHA_IB			(DPSTRUCT *dp, VAR *Source );
INT32	JLibECHDIR_IB			(DPSTRUCT *dp, VAR *NewDir );
INT32	JLibECOMPARES_IBBB		(DPSTRUCT *dp, VAR *Source1, VAR *Source2, VAR *Type);
INT32	JLibECOUNT_IBB			(DPSTRUCT *dp, VAR *Source, VAR *Delimiter );
INT32	JLibECOUNTS_IBB			(DPSTRUCT *dp, VAR *Source, VAR *Delimiter );
VAR		*JLibECOUNTS_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Delimiter );
INT32	JLibEDCOUNT_IBB			(DPSTRUCT *dp, VAR *Source, VAR *Delimiter );
INT32	JLibEFINDSTR_IBBIBBB	(DPSTRUCT *dp, VAR *Expression, VAR *Array, INT32 Count, VAR *AMNo, VAR *MVNo, VAR *SVNo );
INT32	JLibEFIND_IBBIBBB		(DPSTRUCT *dp, VAR *Expression, VAR *Array, INT32 Count, VAR *AMNo, VAR *MVNo, VAR *SVNo );
INT32	JLibEGETCWD_IB			(DPSTRUCT *dp, VAR *Result );
INT32	JLibEISTYPE_IBI			(DPSTRUCT *dp, VAR *Source, INT32 TestType) ;
INT32	JLibEGETENV_IBB			(DPSTRUCT *dp, VAR *EnvName, VAR *Result );
VAR		*JLibEGETENV_BBB		(DPSTRUCT *dp, VAR *Result, VAR *EnvName );
VAR		*JLibEGETUSERNAME_BBI	(DPSTRUCT *dp, VAR *Result, INT32 Uid );
VAR		*JLibEGETUSERGROUP_BBI	(DPSTRUCT *dp, VAR *Result, INT32 Uid );
VAR		*JLibECONTROLCHAR_BBB	(DPSTRUCT *dp, VAR *Result, VAR *Source) ;
VAR		*JLibEUNIQUEKEY_BB		(DPSTRUCT *dp, VAR *Result);
INT32	JLibEINDEX_IBBI			(DPSTRUCT *dp, VAR *Source, VAR *ParmString, INT32 Count );
INT32	JLibELEN_IB				(DPSTRUCT *dp, VAR *Source );
INT32	JLibELENC_IB			(DPSTRUCT *dp, VAR *Source );
INT32	JLibELENDP_IB			(DPSTRUCT *dp, VAR *Source );
STRING	*JLibEADDR_SB			(DPSTRUCT *dp, VAR *Source );
INT32	JLibELOCATE_IBBBIIIIB	(DPSTRUCT *dp, VAR *Result, VAR *SubString, VAR *Source, INT32 Attribute, INT32 MultiValue, INT32 SubValue, INT32 StartPos, VAR *Sequence );
INT32	JLibELOCK_IIII			(DPSTRUCT *dp, INT32 Flags, INT32 Action, INT32 LockNo );
INT32	JLibEMATCHES_IBB		(DPSTRUCT *dp, VAR *Source, VAR *Pattern );
INT32   JLibEParseMATCHES		(DPSTRUCT *dp, void **Patterns, STRING *PatternAddr, INT32 PatternLen, INT32 ExtendedPatternMatch );
INT32   JLibETestMATCHES		(DPSTRUCT *dp, void *Patterns, STRING *SourceAddr, INT32 SourceLen );
void    JLibEReleaseMATCHES		(DPSTRUCT *dp, void *Patterns);
INT32	JLibEParsePATTERN		(DPSTRUCT *dp, void **Patterns, STRING *PatternAddr, INT32 PatternLen );
INT32	JLibEPUTENV_IB			(DPSTRUCT *dp, VAR *NewName );
INT32	JLibEDELENV_IB			(DPSTRUCT *dp, VAR *EldName );
INT32	JLibEREGEXP_IBB			(DPSTRUCT *dp, VAR *Source, VAR *Expression );
INT32	JLibEREGEXP_IBBI		(DPSTRUCT *dp, VAR *Source, VAR *Expression , INT32 flags);
INT32	JLibESLEEP_II			(DPSTRUCT *dp, INT32 sleeptime );
INT32	JLibEBITCHANGE_II		(DPSTRUCT *dp, INT32 BitNo);
INT32	JLibEBITCHECK_II		(DPSTRUCT *dp, INT32 BitNo);
INT32	JLibEBITRESET_II		(DPSTRUCT *dp, INT32 BitNo);
INT32	JLibEBITSET_II			(DPSTRUCT *dp, INT32 BitNo);
INT32	JLibEFIELD_COL_II		(DPSTRUCT *dp, int ColNumber);
VAR		*JLibADebugCoreDump_BBBB	(DPSTRUCT *dp, VAR *result, VAR *filename , VAR * flags ) ;
int		JLibADebugCheckCase		(DPSTRUCT * dp , UINT32 LineAndFile);

INT32	JLibFDELETELIST_VB		(DPSTRUCT *dp, VAR *ListName );
INT32	JLibFINPUT_IBBBBBIII	(DPSTRUCT *dp, VAR *Result, VAR *CursorInfo, VAR *FormatMask, VAR *DelimMask, VAR *PadChar, INT32 Length, INT32 Flags, INT32 Timeout );
void	JLibFINPUTNULL_VB		(DPSTRUCT *dp, VAR *Character);
void	JLibFINPUTCLEAR_V		(DPSTRUCT *dp);
INT32	JLibFINPUTTRAP_IBI		(DPSTRUCT *dp, VAR *StringValue, INT32 Max);
void	JLibFCLEARDATA_V		(DPSTRUCT *dp);
INT32	JLibFIN_IBI(DPSTRUCT *dp, VAR *Result, INT32 TimeOut);
INT32	JLibFIN_IBII(DPSTRUCT *dp, VAR *Result, INT32 TimeOut, INT32 Options);
VAR		*JLibFKEYIN_BB			(DPSTRUCT *dp, VAR *Result );
INT32	JLibFIOCTL_IBBB			(DPSTRUCT *dp, VAR *FileVar, VAR *CommandVar, VAR *SourceReturn );
INT32	JLibFTA_II				(DPSTRUCT *dp, INT32 port);
INT32	JLibFGET_IIBIBBBBB		(DPSTRUCT *dp, INT32 flags, VAR *retvar, INT32 length, VAR *setting, VAR *from, VAR *until, VAR *returning, VAR *waiting ) ;
INT32	JLibFSEND_IIBB			(DPSTRUCT *dp, INT32 flags, VAR *senddata, VAR *from );
INT32	JLibFTTYCTL_IBBB		(DPSTRUCT *dp, VAR *VarDscb, VAR *Code, VAR *Setting);
INT32	JLibFTTYGET_IIBBB		(DPSTRUCT *dp, INT32 Type, VAR *VarDscb, VAR *Variable, VAR *Setting);
INT32	JLibFTTYSET_IIBBB		(DPSTRUCT *dp, INT32 Type, VAR *VarDscb, VAR *Variable, VAR *Setting);
INT32	JLibFOPEN_IBBB			(DPSTRUCT *dp, VAR *Dict, VAR *FileName, VAR *VarDscb);
INT32	JLibFOPEN_IBBBB			(DPSTRUCT *dp, VAR *Dict, VAR *FileName, VAR *VarDscb, VAR *Setting);
void	JLibFCLOSE_VB			(DPSTRUCT *dp, VAR *VarDscb);
INT32	JLibFCLOSE_IB			(DPSTRUCT *dp, VAR *VarDscb);
INT32	JLibFREADNEXT_IBBBBB	(DPSTRUCT *dp, VAR *ToVar, VAR *MultiValue, VAR *SubValue, VAR *List, VAR *Setting ) ;
INT32	JLibFREADINDEX_IIBBBBB	(DPSTRUCT *dp, INT32 direction, VAR *NextIndexValue, VAR *NextKey, VAR *NextVmCount, VAR *FromList, VAR *Setting ) ;
INT32	JLibFREADT_IBII			(DPSTRUCT *dp, VAR *Target, INT32 Channel, INT32 Flags );
INT32	JLibFRECORDLOCKED_IBB	(DPSTRUCT *dp, VAR *DSCB, VAR *key);
INT32	JLibFREWIND_II			(DPSTRUCT *dp, INT32 Channel );
INT32	JLibFSELECTATKEY_IBBBBBII(DPSTRUCT *dp, VAR *From, VAR *To, VAR *Setting, VAR *IndexKey, VAR *RecordKey, INT32 Vmc, INT32 Svmc);
INT32	JLibFWEOF_II			(DPSTRUCT *dp, INT32 Channel );
INT32	JLibFWRITELIST_IBB		(DPSTRUCT *dp, VAR *Source, VAR *ListName );
INT32	JLibFWRITELIST_IBBB		(DPSTRUCT *dp, VAR *Source, VAR *ListName, VAR *Setting );
INT32	JLibFWRITET_IBII		(DPSTRUCT *dp, VAR *Source, INT32 Channel, INT32 Flags );
INT32	JLibFTRANSACTION_IIB	(DPSTRUCT *dp, INT32 Flags, VAR *Parm);
INT32	JLibFModifySttyIB		(DPSTRUCT *dp, VAR *newstring);
INT32   JLibFNEWOPEN_IIBBBB		(DPSTRUCT *dp, INT32 Flags, VAR *Dict, VAR *FileName, VAR *VarDscb, VAR *Setting );
INT32	JLibFNEWREAD_IIBBBIB	(DPSTRUCT *dp, INT32 Flags, VAR *FileDescriptor, VAR *Record, VAR *Key, INT32 FieldNumber, VAR *Setting );
INT32	JLibFNEWWRITE_IIBBBIB	(DPSTRUCT *dp, INT32 Flags, VAR *FileDescriptor, VAR *Record, VAR *Key, INT32 FieldNumber, VAR *Setting );
INT32	JLibFNEWMATREAD_IIBBBBB	(DPSTRUCT *dp, INT32 Flags, VAR *FirstVariable, JCONST struct JArrayInfo *JCONST Desc, VAR *RecordKey, VAR *FileDescriptor, VAR *Setting );
INT32	JLibFNEWMATWRITE_IIBBBBB(DPSTRUCT *dp, INT32 Flags, VAR *FirstVariable, JCONST struct JArrayInfo *JCONST Desc, VAR *RecordKey, VAR *FileDescriptor, VAR *Setting );
INT32	JLibFNEWDELETE_IIBBB	(DPSTRUCT *dp, INT32 Flags, VAR *FileDescriptor, VAR *RecordKey, VAR *Setting);
INT32	JLibFNEWCLEARFILE_IIBB	(DPSTRUCT *dp, INT32 Flags, VAR *FileDescriptor, VAR *Setting );
INT32	JLibFNEWOPENINDEX_IIBBBB(DPSTRUCT *dp, INT32 Flags, VAR *FileName, VAR *RecordKey, VAR *FileDescriptor, VAR *Setting );
VAR		*JLibGTIMESTAMP_BB		(DPSTRUCT *dp, VAR *Result);
FLOAT	JLibGTIMESTAMP			(DPSTRUCT *dp, int precision);
VAR		*JLibGSEQUOIA_TIMESTAMP_BB	(DPSTRUCT *dp, VAR *Result);
VAR		*JLibGTIMEDIFF_BBBBI		(DPSTRUCT *dp, VAR *Result, VAR *Timestamp1, VAR *Timestamp2, INT32 Mask);
VAR		*JLibGMAKETIMESTAMP_BBIIB	(DPSTRUCT *dp, VAR *Result, INT32 intDate, INT32 intTime, VAR *TimeZone);
VAR		*JLibGCHANGETIMESTAMP_BBBB	(DPSTRUCT *dp, VAR *Result, VAR *Timestamp, VAR *ChangeArray);
VAR		*JLibGLOCALTIME_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *Timestamp, VAR *TimeZone);
VAR		*JLibGLOCALDATE_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *Timestamp, VAR *TimeZone);
VAR		*JLibReturnDefaultFile		(DPSTRUCT *dp);
VAR		*JLibReturnDefaultSelect	(DPSTRUCT *dp);
INT32	JLibGInternalLocalTime	(DPSTRUCT *dp);
INT32	JLibGInternalLocalDate	(DPSTRUCT *dp);
INT32	JLibGDateToInternal		(DPSTRUCT *dp, INT32 Year, INT32 Month, INT32 Day, INT32 julianDays, INT32 *pInternalDate);
void	JLibGUTCtoDateTime		(DPSTRUCT *dp, double  UTCTime, VAR *TimeZone, long *Date, long *Time);
double	JLibGUTCtoMilliseconds	(DPSTRUCT *dp, double  UTCTime, VAR *TimeZone);
INT32	JLibGSEQ_IB				(DPSTRUCT *dp, VAR *Source );
STRING	*JLibBCONV_SFB			(DPSTRUCT *dp, VAR *Source );
VAR		*JLibBARRAY_BB			(DPSTRUCT *dp, JCONST struct JArrayInfo *JCONST Desc, ... );
VAR		*JLibBDYNARRAY_BB		(DPSTRUCT *dp, VAR *ControlVar, ...) ;
VAR		*JLibBDYNARRAY1_BBI		(DPSTRUCT *dp, VAR *ControlVar, INT32 FirstElement );
VAR		*JLibBDYNARRAY2_BBII	(DPSTRUCT *dp, VAR *ControlVar, INT32 FirstElement , INT32 SecondElement );
VAR		*JLibEESORT_BBBBII		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Keys, INT32 NumAtts, INT32 NumSortKeys);
VAR		*JLibBSTORE_BBB			(DPSTRUCT *dp, VAR *Target, VAR *Source );
VAR		*JLibBSTORE_BBF			(DPSTRUCT *dp, VAR *Target, FLOAT Value );
VAR		*JLibBSTORE_BBI			(DPSTRUCT *dp, VAR *Target, INT32 Value );
VAR		*JLibBINT_BBB			(DPSTRUCT *dp, VAR *Target, VAR *Source );
VAR		*JLibBIFCONDOP_BBBBB	(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Value1, VAR *Value2);
VAR		*JLibDMATH_BBBBI		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2, INT32 type);
VAR		*JLibDMATHADD_BBBB		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2 ) ;
VAR		*JLibDMATHSUB_BBBB		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2 ) ;
VAR		*JLibDMATHMUL_BBBB		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2 ) ;
VAR		*JLibDMATHDIV_BBBB		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2 ) ;
VAR		*JLibDFMATH_BBBBII		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2, INT32 precision, INT32 command);
VAR		*JLibDSMATH_BBBBII		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2, INT32 precision, INT32 type);
VAR		*JLibDDYN_ARRAY_BBBBI	(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2, INT32 opcode );
VAR		*JLibDSPLICE_BBBBB		(DPSTRUCT *dp, VAR *Result, VAR *Source1, VAR *Separator, VAR *Source2);
VAR		*JLibDDROUND_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Precision);
VAR		*JLibDIFS_BBBBB			(DPSTRUCT *dp, VAR *Result, VAR *Conditional, VAR *True, VAR *False ) ;
VAR		*JLibECAT_BBBB			(DPSTRUCT *dp, VAR *Result, VAR *Source1, VAR *Source2 );
VAR		*JLibECHAR_BBI			(DPSTRUCT *dp, VAR *Target, INT32 Number );
VAR		*JLibECHECKSUM_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibEDELETE_BBBIII		(DPSTRUCT *dp, VAR *Result, VAR *Source, INT32 Attribute, INT32 MultiValue, INT32 SubValue );
VAR		*JLibEEXTRACT_BBBIII	(DPSTRUCT *dp, VAR *Result, VAR *Source, INT32 Attribute, INT32 MultiValue, INT32 SubValue );
VAR		*JLibEFIELD_BBBBI		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Delim, INT32 Start);
VAR		*JLibEGROUP_BBBBII		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Delim, INT32 Start, INT32 Total );
VAR		*JLibEGROUPS_BBBBII		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Delim, INT32 Start, INT32 Total );
VAR		*JLibEINMAT_BBBA		(DPSTRUCT *dp, VAR *target, VAR *source, JCONST struct JArrayInfo *sourcedesc);
VAR		*JLibEMATCHFIELD_BBBBI	(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Pattern, INT32 Count );
VAR		*JLibEINSERT_BBBBIII	(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *NewString, INT32 AMNumber, INT32 MVNumber, INT32 SVNumber );
VAR		*JLibEINSERTF_BBBBIII	(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *NewString, INT32 AMNumber, INT32 MVNumber, INT32 SVNumber );
VAR		*JLibEFOLD_BBBB			(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Length) ;
VAR		*JLibEFOLD_BBBBB			(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Length, VAR * Delim) ;
VAR		*JLibELOWER_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Array) ;
VAR		*JLibERAISE_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Array);
// Note that the MCAT function has the result as the first parameter so
// that the jbccom optimizer does not have to be modified.
//
VAR		*JLibEMCAT_BBI			(VAR *Result, DPSTRUCT *dp, ...);
VAR		*JLibEPERROR_BBB		(DPSTRUCT *dp, VAR *Result, VAR *StoreErrno );
VAR		*JLibEREPLACE_BBBBIII	(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *NewString, INT32 AMNumber, INT32 MVNumber, INT32 SVNumber );
VAR		*JLibESENTENCE_BBII		(DPSTRUCT *dp, VAR *Result, INT32 Position, INT32 Options );
VAR		*JLibENEWSENTENCE_BBII	(DPSTRUCT *dp, VAR *Result, INT32 Position, INT32 Options );
INT32	JLibEGETARG_IIB			(DPSTRUCT *dp, INT32 ArgNo, VAR *Result);
INT32	JLibEGETMSG_IIB			(DPSTRUCT *dp, INT32 ErrNo, VAR *TargetVar);
VAR		*JLibESORT_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibESPACE_BBI			(DPSTRUCT *dp, VAR *Target, INT32 Count );
VAR		*JLibE_DYNARRAY_BBBBIII	(DPSTRUCT *dp, VAR *Result, VAR *Source1, VAR *Source2, VAR *Source3, INT32 Operation, INT32 Count1, INT32 Count2 );
VAR		*JLibESPOOLER_BBIB		(DPSTRUCT *dp, VAR *Target, INT32 Function, VAR *Line) ;
VAR		*JLibESTR_BBBI			(DPSTRUCT *dp, VAR *Result, VAR *Source, INT32 Count );
VAR		*JLibESUBSTR_BBBII		(DPSTRUCT *dp, VAR *Result, VAR *Source, INT32 StartPos, INT32 Length );
VAR		*JLibESUBSTR_BBBI		(DPSTRUCT *dp, VAR *Result, VAR *Source, INT32 Length );
VAR		*JLibESYSTEM_BBI		(DPSTRUCT *dp, VAR *Result, INT32 SystemNo );
VAR		*JLibEDIR_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Source);
VAR		*JLibESUM_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Source);
VAR		*JLibESUMMATION_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source);
VAR		*JLibESOUNDEX_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source);
VAR		*JLibEQUOTE_BBBI		(DPSTRUCT *dp, VAR *Result, VAR *Source, int QuoteChar);
VAR		*JLibEMinMax_BBBI		(DPSTRUCT *dp, VAR *Result, VAR *Source, int Flags);
VAR		*JLibEBITLOAD_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source) ;
VAR		*JLibEBITFUNC_BBBBI		(DPSTRUCT *dp, VAR *result, VAR *source1, VAR *source2, INT32 test);
VAR		*JLibEREMOVE_BBBB		(DPSTRUCT *dp, VAR *Target, VAR *Source, VAR *Setting);
VAR		*JLibETRIM_BIBBBB		(DPSTRUCT *dp, INT32 Flags, VAR *Result, VAR *Source, VAR *Char, VAR *Type );
VAR		*JLibETRIMS_BIBB		(DPSTRUCT *dp, INT32 Flags, VAR *Target, VAR *Source);
VAR		*JLibEATFUNC_BI			(DPSTRUCT *dp, INT32 funcno);
VAR		*JLibEDECRYPT_BBBBI		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Key, INT32 Method);
VAR		*JLibEENCRYPT_BBBBI		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *Key, INT32 Method);
VAR		*JLibFCATTAB_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *Source1, VAR *Source2 );
VAR		*JLibFCURSOR_BBII		(DPSTRUCT *dp, VAR *Result, INT32 Column, INT32 Row );
VAR		*JLibFINDICES_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *filevar, VAR *Operand );
VAR		*JLibGASCII_BBB			(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibGUPCASE_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source );
STRING  *JLibGUPCASE_INTERNAL	(DPSTRUCT *dp, VAR *Source, INT32 mode ); // mode 0 = create, 1 = release
VAR		*JLibGDOWNCASE_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibGCHANGEF_BBBBB		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *From, VAR *To );
VAR		*JLibGSWAPF_BBBBBII		(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *From, VAR *To, INT32 Occurence, INT32 Start);
VAR		*JLibGCONVERTF_BBBBB	(DPSTRUCT *dp, VAR *Result, VAR *Source, VAR *From, VAR *To );
VAR		*JLibGItypeRun			(DPSTRUCT *dp, VAR *ItypeReg, VAR *Target, VAR *ItypeRef);
void	JLibGReleaseITypeCache	(DPSTRUCT *dp, void *ItypeCache);
void	JLibGReleaseConvCache	(DPSTRUCT *dp, void *ConvCache);
void	JLibGReleasejQLDictCache(DPSTRUCT *dp, void *jQLDictCache);
void	JLibEGROUPSTORE_VBBIIB	(DPSTRUCT *dp, VAR *SubString, VAR *Variable, INT32 Start, INT32 End, VAR *Delim ) ;
void	JLibGCONVERTTO_VBIIIBB	(DPSTRUCT *dp, VAR *SourceAndTarget, INT32 Attribute, INT32 MultiValue, INT32 SubValue, VAR *From, VAR *To);
void	JLibGCHANGETO_VBIIIBB	(DPSTRUCT *dp, VAR *SourceAndTarget, INT32 Attribute, INT32 MultiValue, INT32 SubValue, VAR *From, VAR *To);
VAR		*JLibGEBCDIC_BBB		(DPSTRUCT *dp, VAR *Result, VAR *Source );
VAR		*JLibGFORMAT_BBBBB		(DPSTRUCT *dp, VAR *Result, VAR *ConvReg, VAR *Source, VAR *FormatSpec );
VAR		*JLibGTIMEDATE_BB		(DPSTRUCT *dp, VAR *Result );
VAR		*JLibGTRANS_BBBBIB		(DPSTRUCT *dp, VAR *Result, VAR *filename, VAR *recordkey, INT32 field, VAR *ccode);
VAR		*JLibGTRANS_BBBBBB		(DPSTRUCT *dp, VAR *Result, VAR *filename, VAR *recordkey, VAR *field, VAR *ccode);
int		JLibSignalTest			(struct jBASEDataAreas * dp);
INT32	JLibGXTD_IB				(DPSTRUCT *dp, VAR *Source );
VAR		*JLibGDTX_BBI			(DPSTRUCT *dp, VAR *Result, INT32 Value );
VAR		*JLibGDTX_BBBB			(DPSTRUCT *dp, VAR * Result, VAR * Source, VAR * universe_mode);
VAR		*JLibGXTD_BBBB			(DPSTRUCT *dp, VAR * Result, VAR * Source, VAR * universe_mode);

#ifdef	__cplusplus

#ifdef __GNUC__
typedef  int ( *CsubPtr )(...);
#else
extern "C" typedef  int ( *CsubPtr )(...);
#endif

/*
 * Declarations for C++ compilers...
 */
int		(*JLibCCALL_DIR_DYN(DPSTRUCT * dp, int (**handle)(...), char *FunctionName ))(DPSTRUCT *dp, ...);
int		(*JLibCCALL_DIR_DYN_NOERROR(DPSTRUCT * dp, int (**handle)(...), char *FunctionName, int))(DPSTRUCT *dp, ...);
int		(*JLibCCALL_IND_DYN(DPSTRUCT * dp, VAR *FunctionName))(DPSTRUCT *dp, ...);

#else /* _cplusplus */
/*
 * Declarations for C compilers...
 */
int		(*JLibCCALL_DIR_DYN(DPSTRUCT * dp, int (**handle)(), char *FunctionName))();
int		(*JLibCCALL_DIR_DYN_NOERROR(DPSTRUCT * dp, int (**handle)(), char *FunctionName, int))();
int		(*JLibCCALL_IND_DYN(DPSTRUCT * dp, VAR *FunctionName))();

#endif /* _cplusplus */

int		(*JLibCCALLC_INDIRECT(DPSTRUCT * dp, VAR *FunctionName))();
int		(*JLibCCALLC_DIRECT(DPSTRUCT * dp, char *FunctionName))();
VAR             *JBASECALLC(VAR *Result, DPSTRUCT * dp, STRING *FunctionName, VAR *Source);
int		(*JLibCCALL_DIRC_DYN(DPSTRUCT * dp, int (**handle)(), char *FunctionName))();
int     (*JLibCCALL_FUNCTION(DPSTRUCT * dp, char *FunctionName))();
int		JLibCCALLJ				(DPSTRUCT *dp, VAR *Result, VAR *ClassName, VAR *MethodName, VAR *Params);
int		JLibCCALLdotNET			(DPSTRUCT *dp, VAR *Result, VAR *ClassName, VAR *MethodName, VAR *Params);
INT32	JLibCCALLONEXIT_IB		(DPSTRUCT *dp, VAR * subptr);
char	*JLibAGetGlobalDir		(void);
char	*JLibAGetReleaseDir		(void);
char	*JLibAGetDataDir		(void);
char	*JLibAGetProcDir		(void);
int		JLibFREADLIST_IBBB		(DPSTRUCT *dp, VAR *Result, VAR *ListName, VAR *Setting );
int     JLibFGETLIST_IBBB		(DPSTRUCT *dp, VAR *Result, VAR *ListName, VAR *Setting );
void	JLibARESET_FILE_ERROR_V	(DPSTRUCT *dp);
void	JLibBCLEARVAR_V			(DPSTRUCT *dp);
void	JLibBMATAA_VBB			(DPSTRUCT *dp, JCONST struct JArrayInfo *JCONST TargetDesc, 
								 VAR *Target, JCONST struct JArrayInfo *JCONST SourceDesc, VAR *Source );
void	JLibBMATAB_VBB			(DPSTRUCT *dp, JCONST struct JArrayInfo *JCONST Desc,
								 VAR *Target, VAR *Source );
void	JLibBDYNMATAA_VBB		(DPSTRUCT *dp, VAR *Target, VAR *Source );
void	JLibBDYNMATAB_VBB		(DPSTRUCT *dp, VAR *Target, VAR *Source );
void    JLibBDDIM_VIBII			(DPSTRUCT *dp, int Whence, VAR *ControlVar, INT32 ArrayQty, INT32 SizeFirst, ...) ;
void	JLibBSTORE_VBF			(DPSTRUCT *dp, VAR *Target, FLOAT Value );

/* The following really should take a BIG_INT * rather than a void *. However making
 * this opaque removes the requirement for all of jBASE to include 
 * BigDigits/bigd.h. As time progressed and BigDigits becomes more widely used
 * this can be changed.
 */
#ifdef JBUILD_OPTION_BIGINT_MATH
void	JLibBSTORE_VBD			(DPSTRUCT *dp, VAR *Target, struct BIG_INT * Value );
#endif /* JBUILD_OPTION_BIGINT_MATH */

void	JLibBSTORE_VBS			(DPSTRUCT *dp, VAR *Target, STRING *Source );
void	JLibBSTORE_VBSI			(DPSTRUCT *dp, VAR *Target, STRING *Source, int Length);
void	JLibBStoreFreeVar_VB	(DPSTRUCT *dp, VAR *Var, const char *filename, int lineno );
void	JLibBStoreResize_VBI	(DPSTRUCT *dp, VAR *Var, INT32 NewSize, const char *filename, int lineno );
void	JLibBStoreString_VBIS	(DPSTRUCT *dp, VAR *Var, INT32 NewSize, STRING *DataToCopy, const char *filename, int lineno );
void	JLibCCHAIN_VB			(DPSTRUCT *dp, VAR *Statement );
void	JLibCDATA_VIB			(DPSTRUCT *dp, INT32 DatQty, ... );
void	JLibCDEBUG_V			(DPSTRUCT *dp);
void	JLibCENTER_VB			(DPSTRUCT *dp, VAR *Statement );
void	JLibCEXIT_VI			(DPSTRUCT *dp, INT32 ExitCode, INT32 Flags );
void	JLibCGOSUB_STACK_FULL_V	(DPSTRUCT *dp);
INT32	JLibCPERFORM_IIBBBBBBB	(DPSTRUCT *dp, INT32 Flags, VAR *Statement, 
								 VAR *Setting, VAR *Capturing, VAR *RtnList, VAR *RtnData, 
								 VAR *PassList, VAR *PassData );
int		JLibCSTOP_VI			(DPSTRUCT *dp, INT32 Flags, ...);
void	JLibCSUBROUTINE_END_VI	(DPSTRUCT *dp, UINT64 * var_strings, JCONST int * lookups);
void	JLibCSUBROUTINE_INIT_VV	(DPSTRUCT *dp, struct StackLevelData *LevelData, 
								 JCONST unsigned char *ConstantText, int **debug_flag_pointer, 
								 UINT32 **debug_line_pointer, VAR *LocalVars, 
								 struct StaticLevelData *SubStaticLevelData, VAR **SubroutineArgs, 
								 char *ExpectedFlags, char *ActualFlags, ...);
void	JLibCSUBROUTINE_INIT2_VV	(DPSTRUCT *dp, struct StackLevelData *LevelData, 
								 JCONST unsigned char *ConstantText, int **debug_flag_pointer, 
								 UINT32 **debug_line_pointer, VAR *LocalVars, 
								 struct StaticLevelData *SubStaticLevelData, VAR **SubroutineArgs, 
								 char *ExpectedFlags, char *ActualFlags, ...);
void	JLibCAttachCommon_VSSVI	(DPSTRUCT *dp, char *CommonName, VAR **FirstVarPointer,
								 INT32 VarQty, int *user_data_key);
void	JLibCReloadCommon_VSSVI	(DPSTRUCT *dp, char *CommonName, VAR **FirstVarPointer,
								 INT32 VarQty, int *user_data_key);
void	JLibCRTNDATA_VB			(DPSTRUCT *dp, VAR *DataVar);
void	JLibCCOLLECTDATA_VB		(DPSTRUCT *dp, VAR *DataVar);
void	JLibCCLEARCOMMON_V		(DPSTRUCT *dp) ;
void	JLibCCLEARCOMMON_VB		(DPSTRUCT *dp, VAR *NamedCommon) ;
void	JLibEASSIGNSYS_VBI		(DPSTRUCT *dp, VAR *Source, INT32 SystemNo );
void	JLibECOMPLEX_STORE_VBIIIIIIB	(DPSTRUCT *dp, VAR *Result, INT32 AMNumber, 
										 INT32 MVNumber, INT32 SVNumber, INT32 SubStringStart, 
										 INT32 SubStringLength, INT32 Flags, VAR *Source );
void	JLibECOMPLEX_STORE_VBIIIBIIIB	(DPSTRUCT *dp, VAR *Result, INT32 AMNumber, 
										 INT32 MVNumber, INT32 SVNumber, VAR *Delim, INT32 SubStringStart, 
										 INT32 SubStringLength, INT32 Flags, VAR *Source );
void	JLibECOMPLEX_STORE_VBB			(DPSTRUCT *dp, VAR *Result, VAR * Source);
void	JLibECOMPLEX_STORE_VBIS			(DPSTRUCT *dp, VAR *Result, INT32 SourceLen , STRING * Source);
void	JLibEMATBUILD_VBBIIB			(DPSTRUCT *dp, VAR *ParmString, VAR *ParmArray, 
										 JCONST struct JArrayInfo *JCONST ArrayDesc, 
										 INT32 Start, INT32 End, VAR *Delim );
void	JLibEMATPARSE_VBIIBBB	(DPSTRUCT *dp, VAR *ParmArray, 
								 JCONST struct JArrayInfo *JCONST ArrayDesc, INT32 Start, INT32 End, 
								 VAR *ParmString, VAR *Delim, VAR *Setting );
void	JLibESLEEP_VB			(DPSTRUCT *dp, VAR *SleepVar );
void	JLibEMSLEEP_VB			(DPSTRUCT *dp, VAR *SleepVar );
void	JLibEREMOVE_VBBBB		(DPSTRUCT *dp, VAR *Target, VAR *Source, VAR *Colpos, VAR *Setting);
void	JLibFBREAKKEY_VF		(DPSTRUCT *dp, FLOAT BreakFlag );
void	JLibFCRTCR				(DPSTRUCT *dp);
void	JLibFCRTN_VB			(DPSTRUCT *dp, VAR *Source );
void	JLibFCRT_VB				(DPSTRUCT *dp, VAR *Source );
void	JLibFECHO_VF			(DPSTRUCT *dp, FLOAT Echo );
void	JLibFECHO_VIF			(DPSTRUCT *dp, INT32 Type, FLOAT Echo );
INT32	JLibFNEWFILELOCK_IIBBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *FileDesc, VAR *LockType, VAR *Setting);
INT32	JLibFFILEUNLOCK_IB		(DPSTRUCT *dp, VAR *FileDesc);
void	JLibFHEADING_FOOTING_VIIB	(DPSTRUCT *dp, INT32 Report, int Flag, VAR *Statement );
void	JLibFINPUTERR_VB			(DPSTRUCT *dp, VAR *Expression );
void	JLibFOUT_VI				(DPSTRUCT *dp, INT32 NumberToOutput );
void	JLibFPAUSE_VI			(DPSTRUCT *dp, INT32 TimeOut);
void	JLibFPAUSE_VB			(DPSTRUCT *dp, VAR * TimeOut);
void	JLibFPAGE_VII			(DPSTRUCT *dp, INT32 Report,  INT32 NewPage );
void	JLibFPRINTCR_VI			(DPSTRUCT *dp, INT32	Report );
void	JLibFPRINTER_VII		(DPSTRUCT *dp, INT32 Report, INT32 Flags );
void	JLibFPRINTN_VIB			(DPSTRUCT *dp, INT32 Report, VAR *Source );
void	JLibFPRINT_VIB			(DPSTRUCT *dp, INT32 Report, VAR *Source );
void	JLibFPRINTERR_VBB		(DPSTRUCT *dp, VAR *Message, VAR *OptionalFile);
void	JLibFPROMPT_VB			(DPSTRUCT *dp, VAR *Prompt );
void	JLibFRELEASE_VBB		(DPSTRUCT *dp, VAR *FileVar, VAR *RecordKey );
INT32	JLibFRELEASE_IBB		(DPSTRUCT *dp, VAR *FileVar, VAR *RecordKey );
INT32	JLibFRELEASEALL_I		(DPSTRUCT *dp) ;
void	JLibFRELEASEALL_VB		(DPSTRUCT *dp, VAR *FileVar) ;
INT32	JLibFSELECT_IBBB		(DPSTRUCT *dp, VAR *FromVariable, VAR *ToVariable , VAR *Setting );
INT32	JLibFSSELECT_IBBB		(DPSTRUCT *dp, VAR *FromVariable, VAR *ToVariable , VAR *Setting );
void	JLibFSELECTE_VBB		(DPSTRUCT *dp, VAR *ToVariable , VAR *Setting );
void	JLibFSELECTINDEX_VBBBBB	(DPSTRUCT *dp, VAR *IndexName, VAR *Start, VAR *DSCB, VAR *ToPos, VAR *Setting);
void	JLibFFORMLIST_VBBB		(DPSTRUCT *dp, VAR *FromVariable, VAR *ToVariable , VAR *Setting );
void	JLibFCLEARSELECT_VB		(DPSTRUCT *dp, VAR *SelectVar);
void	JLibFCLEARSELECTALL		(DPSTRUCT *dp);
void	JLibFCRTBEGIN			(DPSTRUCT *dp);
void	JLibFCRTEND				(DPSTRUCT *dp) ;

JediFileDescriptor	*JLibAGetTempFile	(DPSTRUCT *dp);
JediFileDescriptor	*JLibAGetPauseFile	(DPSTRUCT *dp);

INT32	JLibFNEWOPENSEQ_IIBBIBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *PathName, VAR *Filename, INT32 Action, VAR *FileDesc, VAR *Setting);
INT32	JLibFNEWOPENSER_IIBBBB	(DPSTRUCT *dp, INT32 FileFlags,  VAR *FilePath, VAR *Options, VAR *Desc, VAR *Setting);
INT32	JLibFNEWDELETESEQ_IIBBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *PathName, VAR *Filename, VAR *Setting);
INT32	JLibFNEWREADSEQ_IIBBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *Record, VAR *FileDesc, VAR *Setting);
INT32	JLibFNEWWRITESEQ_IIBIBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *Record, INT32 Action, VAR *FileDesc, VAR *Setting);
INT32	JLibFNEWWEOFSEQ_IIBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *FileDesc, VAR *Setting);

INT32	JLibFNEWOSREAD_IIBBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *Record, VAR *FileDesc, VAR *Setting);
INT32	JLibFNEWOSWRITE_IIBIBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *Record, INT32 Action, VAR *FileDesc, VAR *Setting);
INT32	JLibFNEWOSDELETE_IIBB	(DPSTRUCT *dp, INT32 FileFlags, VAR *PathName, VAR *Setting);

INT32	JLibFCLOSESEQ_IB		(DPSTRUCT *dp, VAR *FileDesc);
INT32	JLibFNOBUF_IBBB			(DPSTRUCT *dp, VAR *FileDesc, VAR *Setting);
INT32	JLibFSEEK_IBIIB			(DPSTRUCT *dp, VAR *FileDesc, INT32 Offset, INT32 RelPos, VAR *Setting);
INT32   JLibFREADBLK_IBBIB		(DPSTRUCT *dp, VAR *Record, VAR *FileDesc, INT32 BlockSize, VAR *Setting);
INT32   JLibFWRITEBLK_IBBB		(DPSTRUCT *dp, VAR *Record, VAR *FileDesc, VAR *Setting);
void	JLibFTIMEOUT_VBB		(DPSTRUCT *dp, VAR *TimeOut, VAR *FileDesc);
void	JLibFWAKE_VI			(DPSTRUCT *dp, INT32 Port);
INT32	JLibFSTATUS_IBBB		(DPSTRUCT *dp, VAR *Array, VAR *FileDesc, VAR *Setting);
VAR 	*JLibFFILEINFO_BBBB		(DPSTRUCT *dp, VAR *Result, VAR *FileDesc, VAR *Code);
INT32	JLibFSELECTINFO_IBI		(DPSTRUCT *dp, VAR *SelectList, INT32 Code);
INT32	JLibFCREATESEQ_IBB		(DPSTRUCT *dp, VAR *FileDesc, VAR *Setting);
INT32   JLibFOSBREAD_IBBIIB		(DPSTRUCT *dp, VAR *Record, VAR *FileDesc, INT32 Offset, INT32 Length, VAR *Setting);
INT32	JLibFOSBWRITE_IBBIB		(DPSTRUCT *dp, VAR *Record, VAR *FileDesc, INT32 Offset, VAR *Setting);
void 	JLibGInternalToDate		(DPSTRUCT *dp, INT32 InternalDate, INT32 *pYear, INT32 *pMonth, INT32 *pDay );
VAR		*JLibGConvRun			(DPSTRUCT *dp, VAR *CCBVar, VAR *Result, VAR *Source, VAR *pConversion, INT32 Type);
VAR		*JLibGConvRun2			(DPSTRUCT *dp, VAR *CCBVar, VAR *Result, VAR *Source, VAR *pConversion, INT32 Type, INT32 flags);
void	JLibGConvCCBFree		(DPSTRUCT *dp, VAR *CCBVar);
void	JLibGConv_MCAX			(DPSTRUCT *dp, VAR *Result, VAR*Source);
void	JLibGConv_MCXA			(DPSTRUCT *dp, VAR *Result, VAR*Source);
void	JLibOSTORE_VBO			(DPSTRUCT *dp, VAR *Target, void*objectPointer);

INT32	JLibCreateFile			(DPSTRUCT *dp, char *, INT32 );
INT32	JLibMkdir				(DPSTRUCT *dp, char *);

INT32	JLibUpdateMD			(DPSTRUCT *dp, char *);
INT32	JLibGetTrigger			(DPSTRUCT *dp, char *, VAR *);
INT32 	JLibDeleteTrigger		(DPSTRUCT *dp, char *FileName, char *eventName );

INT32 	JLibCreateOrUpdateTrigger	(DPSTRUCT *dp, char *FileName, char *description, 
									 char *subName, INT32 overwriteFlag, INT32 terminateFlag, 
									 INT32 modifyFlag, INT32 debugFlag );

INT32	JLibModifyHashFile		(DPSTRUCT *dp, char *, char *, INT32 );
INT32	JLibDelete				(DPSTRUCT *dp, char *);
INT32	JLibDeleteFile			(DPSTRUCT *dp, char *);
INT32	JLibMoveOrCopyFile		(DPSTRUCT *dp, STRING *, STRING *, STRING *, STRING *, INT32 );
INT32	JLibFileExists			(DPSTRUCT *dp, char *);
INT32	JLibSanitiseName		(DPSTRUCT *dp, char *SourceString, char *ResultString );
INT32	JLibDesanitiseName		(DPSTRUCT *dp, char *SourceString, char *ResultString );
void	*JLibCreateUserData		(DPSTRUCT *dp, const char *key, int size, int *arrayno, 
								 int flags, void (*)(DPSTRUCT *dp, void *, int ));
void	*JLibCreateUserDataEx	(DPSTRUCT *dp, const char *key, int size, int *arrayno, int flags,
								 void (*)(DPSTRUCT *dp, void *),
								 void (*)(DPSTRUCT *dp, void *, int ));

void	JLibCheckMD				(DPSTRUCT *dp, char *);


VAR		*JBASE_GetDevInode_BBB	( VAR *Target, DPSTRUCT *dp, VAR *Source );
INT32	JLibEmulateGetIntJBASE	( DPSTRUCT *dp, INT32 command );
INT32	JLibEmulateSetIntJBASE	( DPSTRUCT *dp, INT32 command, INT32 value);
void	JLibEmulateSwitchCase	( DPSTRUCT *dp, FLOAT value);
INT32   JLibFNOBUF_IBB			( DPSTRUCT *dp, VAR *FileDesc, VAR *Setting );

INT32	PatternMatch			(struct jBASEDataAreas * dp, STRING * PatternAddr, INT32 PatternLen,
								STRING * SourceAddr, INT32 SourceLen, INT32 MatchField,
								VAR *MatchFieldVAR, INT32 ExtendedPatternMatch);

INT32   JBASE_utf8_set_mode			(DPSTRUCT *dp, INT32 Value);		// Set value of international mode
void	*JBASE_utf8_open_converter	(DPSTRUCT *dp, char *CodePage);
void	JBASE_utf8_close_converter	(DPSTRUCT *dp, void *Converter);
INT32   JBASE_utf8_displen_var_IB	(DPSTRUCT *dp, VAR *Source);		// Get display length of variable
INT32   JBASE_utf8_charlen_var_IB	(DPSTRUCT *dp, VAR *Source);		// Get character len of UTF-8 variable

INT32   JBASE_utf8_charlen_string_ISI	(DPSTRUCT *dp, STRING *sourcePtr, INT32 sourceLen );		// Get character length of a string
INT32   JBASE_utf8_bytelen_string_ISI	(DPSTRUCT *dp, STRING *sourcePtr, INT32 sourceCharLen );	// Get byte length of a string of n characters
INT32   JBASE_utf8_bytelen_char_IS	(DPSTRUCT *dp, STRING *sourcePtr );		// Get byte length of a character
INT32   JBASE_utf8_displen_char_IS	(DPSTRUCT *dp, STRING *sourcePtr );

STRING	*JBASE_utf8_next_char_SS		(DPSTRUCT *dp, STRING *sourcePtr);				// Find start of next UTF-8 character sequence
STRING	*JBASE_utf8_prev_char_SS		(DPSTRUCT *dp, STRING *sourcePtr);				// Find start of prev UTF-8 character sequence
UINT32	JBASE_utf8_get_unicode_char_US	(DPSTRUCT *dp, STRING *sourcePtr);				// Convert UTF-8 Sequence to Unicode 32 bit char
STRING	*JBASE_utf8_get_utf8_string_SI	(DPSTRUCT *dp, UINT32 unicodeValue);			// Convert Unicode 32 bit char to UTF-8 Sequence
INT32	JBASE_utf8_setpage_IIS			(DPSTRUCT *dp, INT32 Mode, STRING *CodePage);	// Set Code Page
VAR 	*JBASE_utf8_getpage_BBI			(VAR *Result, DPSTRUCT *dp,  INT32 Mode);		// Get Code Page
VAR	*JBASE_utf8_gettimezone_BB		(VAR *Result, DPSTRUCT *dp);				// Get Time Zone
VAR	*JBASE_utf8_getlocale_BB		(VAR *Result, DPSTRUCT *dp);				// Get Locale
INT32	JBASE_utf8_setlocale_IIS		(DPSTRUCT *dp, INT32 Mode, STRING *Locale);	// Set locale
INT32	JBASE_utf8_properties_IIS		(DPSTRUCT *dp, INT32 Type, STRING *sourcePtr);	// Check properties

INT32 	JBASE_utf8_convert_var_codepage_to_utf8_BBB	(DPSTRUCT *dp, VAR *Source, int bom_flag);	// Convert var from code page to UTF-8 */
INT32	JBASE_utf8_convert_var_utf8_to_codepage_BBB	(DPSTRUCT *dp, VAR *Source);	// Convert var from utf8 to code page */

INT32	JBASE_utf8_convert_utf8_to_codepage_IIVSISI		(DPSTRUCT *dp, INT32 Mode, void *Converter, STRING **TargetPtr, 
														 INT32 *TargetLen, STRING *SourcePtr, INT32 SourceLen);	// Convert String from UTF-8 to CodePage */
INT32	JBASE_utf8_convert_codepage_to_utf8_IVSISI		(DPSTRUCT *dp, void *Converter, STRING **TargetPtr, INT32 *TargetLen,
														 STRING *SourcePtr, INT32 sourceLen);					// Convert String from Codepage to UTF-8 */


void	*JBASE_utf8_store_UTF8_In_Unicode_UUB		(DPSTRUCT *dp,
													 void *Result, VAR *Source);		// Convert UTF-8 to Unicode
VAR		*JBASE_utf8_convert_var_UTF8_To_Unicode_BB	(VAR *Result, DPSTRUCT *dp,
													 VAR *Source);						// Convert UTF-8 to Unicode
VAR		*JBASE_utf8_convert_var_UTF8_To_UCS4_BBB	(VAR *Result, DPSTRUCT *dp,
													 VAR *Source);			// Convert UTF-8 to UCS4
INT32	JBASE_utf8_convert_UTF8_To_Unicode_IUISI	(DPSTRUCT *dp,
													 void **targetPtr, INT32 *targetLen, STRING *sourcePtr, INT32 sourceLen);
INT32	JBASE_utf8_convert_UTF8_To_Unicode_IUISINM	(DPSTRUCT *dp,
													 void **targetPtr, INT32 *targetLen, STRING *sourcePtr, INT32 sourceLen);

VAR		*JBASE_utf8_store_Unicode_In_UTF8_BBIU		(VAR *Result, DPSTRUCT *dp,  
													 INT32 sourceLen, void *sourcePtr);				// Convert Unicode to UTF-8
VAR		*JBASE_utf8_convert_var_Unicode_To_UTF8_BBB	(VAR *Result, DPSTRUCT *dp, 
													 VAR *Source);									// Convert Unicode to UTF-8
VAR		*JBASE_utf8_convert_var_UCS4_To_UTF8_BBB	(VAR *Result, DPSTRUCT *dp, 
													 VAR *Source,int startOffset);					// Convert UCS4 to UTF-8
VAR		*JBASE_utf8_convert_var_UTF8_To_Unicode_BBB	(VAR *Result, DPSTRUCT *dp, 
													 VAR *Source);									// Convert UTF-8 to Unicode
INT32	JBASE_utf8_convert_Unicode_To_UTF8_ISIUI	(DPSTRUCT *dp, STRING **targetPtr,
													 INT32 *targetLen, void *sourcePtr, INT32 sourceLen);
VAR		*JBASE_utf8_convert_UTF8_To_Latin1_BBB		(VAR *Result, DPSTRUCT *dp, 
													 VAR *Source);									// Convert UTF-8 to latin1
VAR		*JBASE_utf8_convert_Latin1_To_UTF8_BBB		(VAR *Result, DPSTRUCT *dp, 
													 VAR *Source);									// Convert latin1 to UTF-8

VAR		*JBASE_utf8_getaliases		(VAR *Result, DPSTRUCT *dp);		// Get aliases for code pages
VAR		*JBASE_utf8_getlocales		(VAR *Result, DPSTRUCT *dp);		// Get available locales
VAR		*JBASE_utf8_gettimezones	(VAR *Result, DPSTRUCT *dp);		// Get available timezones
INT32	JBASE_utf8_compare_IBB		(DPSTRUCT *dp, VAR *firstVar,
									 VAR *secondVar);								// Compare two UTF-8 variables
VAR		*JBASE_utf8_case_BBIB		(VAR *resultVar, DPSTRUCT *dp, 
									 INT32 Type, VAR *sourceVar);					// Convert variable to upper/lower case

//UCS2 <-> UTF-8 routines.
VAR		*JBASE_utf8_convert_var_UCS2_To_UTF8_BBB(VAR *Result, struct jBASEDataAreas *dp, VAR *Source,int startOffset);
VAR		*JBASE_utf8_convert_var_UTF8_To_UCS2_BBB(VAR *Result, struct jBASEDataAreas *dp, VAR *Source);
UINT16	JBASE_utf8_get_unicode_char_USC2_US(struct jBASEDataAreas *dp, STRING *s,INT16 *utf8_char_length);
#define JBASE_UTF8_GET_UNICODE_USC2_CHAR(var,utf8_char_length) JBASE_utf8_get_unicode_char_USC2_US(dp,var,utf8_char_length) 



// jQL jBC interface functions
INT32	jQLCompile(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* cmdLine, VAR* Options, VAR* MessagesVar);
INT32	jQLGetProperty(struct jBASEDataAreas *dp, VAR* propValue, VAR* StmtVar, VAR* columnNo, VAR* property);
INT32	jQLPutProperty(struct jBASEDataAreas *dp, VAR* propValue, VAR* StmtVar, VAR* columnNo, VAR* property);
INT32	jQLExecute(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* SelectListVar);
INT32	jQLFetch(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* ControlVar, VAR* DataVar);
INT32	jQLCloseStatment(struct jBASEDataAreas * dp, VAR * StmtVar);

// jSQL jBC interface functions
INT32 jSQLCompile(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* cmdLine, VAR* Options,VAR* ErrorStringVar,INT32 ErrorNumber);
INT32 jSQLExecute(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* SelectListVar,VAR* ErrorStringVar,INT32 ErrorNumber);
VAR **jSQLFetch(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* ControlVar, VAR* ErrorStringVar,INT32 *ErrorNumber);
INT32 jSQLFetchFromBasic(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* ControlVar,VAR* ErrorStringVar,INT32 ErrorNumber);
INT32 jSQLPrepare(struct jBASEDataAreas *dp, VAR* StmtVar, VAR* cmdLine, VAR* Options,VAR* ErrorStringVar,INT32 ErrorNumber);
INT32 jSQLBind(struct jBASEDataAreas *dp, VAR* StmtVar, INT32 inteface,VAR** SelectListVarArray,VAR* ErrorStringVar,INT32 ErrorNumber);
INT32 jSQLFetchMetaData(struct jBASEDataAreas *dp, VAR* StmtVar, VAR *MetaData);
INT32 jSQLElementAt(struct jBASEDataAreas *dp, VAR* StmtVar, INT32 pos, VAR *data);
INT32 jSQLElementSize(struct jBASEDataAreas *dp, VAR* StmtVar);
INT32 jSQLAddBindElement(struct jBASEDataAreas *dp, VAR* StmtVar, VAR *newItem);
INT32 jSQLFlushBindData(struct jBASEDataAreas *dp, VAR* StmtVar);
INT32 jSQLGetLastTotals(struct jBASEDataAreas *dp, VAR* StmtVar);

// XSL / XML interface function(s)
VAR *	JLibXSLTRANSLATE_BBBBI(DPSTRUCT * dp, VAR * Target, VAR * Input, VAR * XSLDoc, VAR * ReturnCode, INT32 flag);
VAR *	JLibXMLFUNC_BIBB(DPSTRUCT* dp, VAR * vResult, INT32 request, VAR * vArg1, VAR * vArg2);


// License interface function(s)
VAR *	JLibLicense_BBBBI( DPSTRUCT * dp, VAR * vResult, VAR * vProductId, VAR * vAmount, INT32 nFlag );

/*
 * sort key gen routines
 */
void genLeftKey(struct jBASEDataAreas *dp, VAR* KeyVar, VAR* InputVar, int descending);
void genRightKey(struct jBASEDataAreas *dp,VAR* KeyVar, VAR* InputVar, int descending);
void genStringFromKey(struct jBASEDataAreas *dp, VAR *Target, VAR *InputVar, int descending);

// Misc functions

// Function to get the Pid of the process running on a specified port and vice versa
//

INT32	JBASEGetPidFromPort(DPSTRUCT *dp, INT32 Port);
INT32	JBASEGetPortFromPid(DPSTRUCT *dp, INT32 Pid);

/*
 * Determine file access permissions
 */

INT32	JBASEAccess(DPSTRUCT *dp, VAR *filepath, INT32 mode);

/*
 * Flags for JLibCreateUserData
 */
#define	USER_DATA_FLAGS_ERRMSG			0x0000001
#define	USER_DATA_FLAGS_COMMON			0x0000002
#define	USER_DATA_FLAGS_ROOT_DATA		0x0000004

/*
	'C' code produced by the compiler.
*/

/*
	Category (A) - Fundamental macros.
*/

#define		DEBUG_CHECK		if (DEBUG_FLAG) JLibCDEBUG_V(dp);

#define		DEBUG_FLAG		(*debug_flag_pointer)

#define		DEBUG_LINE		(*debug_line_pointer)

/*RESET_FILE_ERROR_V(void) - Reset the error condition for a file operation error */
#define	RESET_FILE_ERROR_V()	JLibARESET_FILE_ERROR_V(dp)

#define	RETURN_SWITCH									\
return(0);												\
_jl_return_switch:										\
DEBUG_CHECK												\
if (StackData.GosubNext <= 0) {goto _jl_function_end;}	\
switch(StackData.GosubStack[--StackData.GosubNext]) {

#define	RETURN_END(Identifier, IdLabel) case Identifier: goto IdLabel;

#define	PROGRAM_END										\
default: ;					\
 } return 0; }

#define	RETURN_FUNCTION_SWITCH							\
return(FuncResult);										\
_jl_return_switch:										\
DEBUG_CHECK												\
if (StackData.GosubNext <= 0 ) {goto _jl_function_end;}	\
switch(StackData.GosubStack[--StackData.GosubNext]) {

#define	PROGRAM_FUNCTION_END							\
default: ;					\
 } return(FuncResult); }

#define	CHECK_MD(String)	JLibCheckMD(dp, String)

#define	COL1_I()	JLibEFIELD_COL_II(dp, 0)
#define	COL2_I()	JLibEFIELD_COL_II(dp, 1)

/*
	Category (B) - Simple load and store commands, numeric and string
				   extraction and so on.
*/

#define VAR_RELEASE2_VB(jvptr)									\
if (jvptr->VAR_TYPE & (VAR_TYPE_STRING | VAR_TYPE_SELECT | VAR_TYPE_FILE | VAR_TYPE_INDEX | VAR_TYPE_ARRAY | VAR_TYPE_OBJECT_WRAPPER | VAR_TYPE_BIG_INT | VAR_TYPE_DYNAMIC_OBJECT))      \
{																\
    JLibBStoreFreeVar_VB(dp, jvptr,__FILE__,__LINE__) ;								\
}
#define VAR_RELEASE_VB(jvptr) JLibBStoreFreeVar_VB(dp, jvptr,__FILE__,__LINE__)

	/*Convert UTF-8 Sequence to Unicode 32 bit char */
#define	JBASE_UTF8_GET_UNICODE_CHAR(String)			JBASE_utf8_get_unicode_char_US(dp, String)

	/*Set Code Page */
#define	JBASE_UTF8_SET_CODEPAGE_INPUT(String)		JBASE_utf8_setpage_IIS(dp, 0, String)
#define JBASE_UTF8_SET_CODEPAGE_OUTPUT(String)		JBASE_utf8_setpage_IIS(dp, 1, String)
#define JBASE_UTF8_SET_CODEPAGE_SPOOLER(String)		JBASE_utf8_setpage_IIS(dp, 2, String)

	/*Get Code Page */
#define JBASE_UTF8_GET_CODEPAGE_INPUT(Result)		JBASE_utf8_getpage_BBI(Result, dp, 0)
#define JBASE_UTF8_GET_CODEPAGE_OUTPUT(Result)		JBASE_utf8_getpage_BBI(Result, dp, 1)	
#define JBASE_UTF8_GET_CODEPAGE_SPOOLER(Result)		JBASE_utf8_getpage_BBI(Result, dp, 2)

	/*Set locale */
#define JBASE_UTF8_SET_LOCALE_GENERAL(String)		JBASE_utf8_setlocale_IIS(dp, 0,String)
#define JBASE_UTF8_SET_LOCALE_CURRENCY(String)		JBASE_utf8_setlocale_IIS(dp, 1,String)

	/*Convert variable via Unicode to upper or lower case */
#define JBASE_UTF8_TOLOWER(Variable)			JBASE_utf8_case_BBIB(Variable, dp, 0,Variable)
#define JBASE_UTF8_TOUPPER(Variable)			JBASE_utf8_case_BBIB(Variable, dp, 1,Variable)

	/*Compare two variables via Unicode */
#define JBASE_UTF8_COMPARE(FirstVar,SecondVar)		JBASE_utf8_compare_IBB(dp, FirstVar, SecondVar)


/*
**Allow build with forcing ICU Package
*/

#if defined(JBUILD_OPTION_NO_ICU)

	/*Check properties normal */
#define JBASE_UTF8_ISDIGIT(String)					isdigit(*String)
#define JBASE_UTF8_ISALPHA(String)					isalpha(*String)
#define JBASE_UTF8_ISPRINT(String)					isprint(*String)
#define JBASE_UTF8_ISCNTRL(String)					iscntrl(*String)
#define JBASE_UTF8_ISLOWER(String)					islower(*String)
#define JBASE_UTF8_ISUPPER(String)					isupper(*String)
#define JBASE_UTF8_ISALNUM(String)					isalnum(*String)
#define JBASE_UTF8_ISSPACE(String)					isspace(*String)

#define JBASE_UTF8_DISPLEN_VAR(Variable)			JLibELEN_IB(dp, Variable)	// Get byte length of variable for default non utf8 build
#define JBASE_UTF8_CHARLEN_VAR(Variable)			JLibELEN_IB(dp, Variable)	// Get byte length of variable

#define JBASE_UTF8_CHARLEN_STRING(String,ByteLen)	JBASE_utf8_charlen_string_ISI(dp, String, ByteLen)		// Get character length of Utf8 string

#define JBASE_UTF8_LEN_CHAR(String)					1							// Get length of current character
#define	JBASE_UTF8_DISPLEN_CHAR(String)                                 1

#define JBASE_UTF8_BYTELEN_STRING(String,CharLen)	JBASE_utf8_bytelen_string_ISI(dp, String, CharLen)		// Get byte length of character string */ 

#define JBASE_UTF8_NEXT_CHAR(String)				(String + 1)	// Increment to next byte
#define JBASE_UTF8_PREV_CHAR(String)				(String - 1)	// Decrement to previous byte

#else

	/*Check properties ala Unicode ICU Package */
#define JBASE_UTF8_ISDIGIT(String)					JBASE_utf8_properties_IIS(dp, 1,String)
#define JBASE_UTF8_ISALPHA(String)					JBASE_utf8_properties_IIS(dp, 2,String)
#define JBASE_UTF8_ISPRINT(String)					JBASE_utf8_properties_IIS(dp, 3,String)
#define JBASE_UTF8_ISCNTRL(String)					JBASE_utf8_properties_IIS(dp, 4,String)
#define JBASE_UTF8_ISLOWER(String)					JBASE_utf8_properties_IIS(dp, 5,String)
#define JBASE_UTF8_ISUPPER(String)					JBASE_utf8_properties_IIS(dp, 6,String)
#define JBASE_UTF8_ISALNUM(String)					JBASE_utf8_properties_IIS(dp, 7,String)
#define JBASE_UTF8_ISSPACE(String)					JBASE_utf8_properties_IIS(dp, 8,String)

#define JBASE_UTF8_DISPLEN_VAR(Variable)			JBASE_utf8_displen_var_IB(dp, Variable)				// Get display character length of variable
#define JBASE_UTF8_CHARLEN_VAR(Variable)			JBASE_utf8_charlen_var_IB(dp, Variable)				// Get character length of variable
#define JBASE_UTF8_CHARLEN_STRING(String,ByteLen)	JBASE_utf8_charlen_string_ISI(dp, String, ByteLen)	// Get character length of vtf8 string
#define JBASE_UTF8_LEN_CHAR(String)					JBASE_utf8_bytelen_char_IS(dp, String)				// Get byte length of character
#define	JBASE_UTF8_DISPLEN_CHAR(String)			JBASE_utf8_displen_char_IS(dp, String)				// Get byte length of character
#define JBASE_UTF8_BYTELEN_STRING(String,CharLen)	JBASE_utf8_bytelen_string_ISI(dp, String, CharLen)	// Get byte length of character string
#define JBASE_UTF8_NEXT_CHAR(String)				JBASE_utf8_next_char_SS(dp, String)					// Find start of next char
#define JBASE_UTF8_PREV_CHAR(String)				JBASE_utf8_prev_char_SS(dp, String)					// Find start of previous char
#endif


#define JBASEGetDevInode(Target, dp, Source)			JBASE_GetDevInode_BBB(Target, dp, Source)


	/*Convert String from UTF-8 to CodePage */
#define CONV_UTF8_TO_OUTPUT_ISISI(TargetPtr, TargetLen, SourcePtr, SourceLen) 	JBASE_utf8_convert_utf8_to_codepage_IIVSISI(dp, 0, NULL, TargetPtr, TargetLen, SourcePtr, SourceLen)
#define CONV_UTF8_TO_SPOOLER_ISISI(TargetPtr, TargetLen, SourcePtr, SourceLen)	JBASE_utf8_convert_utf8_to_codepage_IIVSISI(dp, 1, NULL, TargetPtr, TargetLen, SourcePtr, SourceLen)

	/*Convert String from Codepage to UTF-8 */
#define CONV_INPUT_TO_UTF8_ISISI(TargetPtr, TargetLen, SourcePtr, SourceLen)	JBASE_utf8_convert_codepage_to_utf8_IVSISI(dp, NULL, TargetPtr, TargetLen, SourcePtr, SourceLen)

	/*Convert UTF-8 to Unicode */
#define STORE_VUB(Result, Source)						JBASE_utf8_store_UTF8_In_Unicode_UUB(dp, Result, Source)
#define CONV_UTF8_TO_UNICODE_VBB(Result,Source)			JBASE_utf8_convert_var_UTF8_To_Unicode_BBB(Result, dp, Source)

#define CONV_UTF8_TO_UNICODE_IUISI(Target, TargetLen, Source, SourceLen)	JBASE_utf8_convert_UTF8_To_Unicode_IUISI(dp, Target, TargetLen, Source, SourceLen)

	/*Convert Unicode to UTF-8 */
#define STORE_VBIU(Result, SourceLen, Source)					JBASE_utf8_store_Unicode_In_UTF8_BBIU(Result, dp, SourceLen, Source)
#define CONV_UNICODE_TO_UTF8_VBB(Result, Source)				JBASE_utf8_convert_var_Unicode_To_UTF8_BBB(Result, dp, Source)
#define CONV_UNICODE_TO_UTF8_ISIUI(Target, TargetLen, Source, SourceLen)	JBASE_utf8_convert_Unicode_To_UTF8_ISIUI(dp, Target, TargetLen, Source, SourceLen)

	/*Convert latin1 to UTF-8 */
#define STORE_UTF8_FROM_LATIN1_VBB(Target,Source)				JBASE_utf8_convert_Latin1_To_UTF8_BBB(Target, dp, Source)

	/*UTF8_BBB(Target, Source) - Convert from latin1 to UTF-8 */
#define	UTF8_BBB(Target, Source) 						JBASE_utf8_convert_Latin1_To_UTF8_BBB(Target, dp, Source)

	/*Convert UTF-8 to latin1 */
#define STORE_LATIN1_FROM_UTF8_VBB(Target,Source)				JBASE_utf8_convert_UTF8_To_Latin1_BBB(Target, dp, Source)

	/*LATIN1_BBB(Target, Source) - Convert from UTF-8 to latin1 */
#define	LATIN1_BBB(Target, Source) 						JBASE_utf8_convert_UTF8_To_Latin1_BBB(Target, dp, Source)

	/*STORE_BBF(Target,Numeric) - Store a numeric value into a variable */
#define	STORE_BBF(Target, Source) JLibBSTORE_BBF(dp, Target, Source)
#define	STORE_VBF(Target, Source) JLibBSTORE_VBF(dp, Target, Source)

	/*STORE_BBB(Target,Source) - Copy from one variable to another. */
#define	STORE_BBB(Target, Source)  JLibBSTORE_BBB(dp, Target,Source)
#define	STORE_VBB(Target, Source)  JLibBSTORE_BBB(dp, Target,Source)

	/*STORE_BBI(Target,Integer) - Store a 32 bit integer into a variable */
#define	STORE_BBI(Target, Source)	JLibBSTORE_BBI(dp, Target, Source)

#ifdef JBUILD_OPTION_BIGINT_MATH
	/*STORE_VBD(Target,Integer) - Store a BIG INTEGER into a variable */
#define	STORE_VBD(Target, Source)	JLibBSTORE_VBD(dp, Target, Source)
#endif /* JBUILD_OPTION_BIGINT_MATH */

	/*STORE_VBS(Target, String) - Store null terminated (STRING*) */
#define	STORE_VBS(Target, Source)	JLibBSTORE_VBS(dp, Target, Source)
#define	STORE_VBC(Target, Source)	JLibBSTORE_VBS(dp, Target, (STRING*)Source)

	/*STORE_VBSI(Target, String, Length) - Store null terminated (STRING*) */
#define	STORE_VBSI(Target, String, Length)	JLibBSTORE_VBSI(dp, Target, String, Length)

	/*STORE_BBS(Target, String) - Store null terminated (STRING*) and return the VAR*	*/
#define	STORE_BBS(Target, String) (STORE_VBS(Target, String), Target)

	/*STORE_VBO(Target, String) - Store object pointer */
#define	STORE_VBO(Target, String)	JLibOSTORE_VBO(dp, Target, String)

	/* STORE_NULL(Targe) -- Store a null string into a variable */
#define	STORE_NULL(Target)	JRunBStoreNull_VB(dp, Target)


	/*PREC_LOSE_II(Integer)	- Return the integer argument - PrecMult correction	*/
#define	PREC_LOSE_II(Integer)	JLibBPREC_LOSE_II(dp, Integer)

	/*PREC_LOSE_FF(Float)		- Return the float argument - PrecMult correction	*/
#define	PREC_LOSE_FF(Float)		JLibBPREC_LOSE_FF(dp, Float)

	/*PREC_GAIN_II(Integer)	- Return the integer argument - PrecMult correction	*/
#define	PREC_GAIN_II(Integer)	JLibBPREC_GAIN_II(dp, Integer)

	/*PREC_GAIN_FF(Float)		- Return the float argument - PrecMult correction	*/
#define	PREC_GAIN_FF(Float)		JLibBPREC_GAIN_FF(dp, Float)

	/*CONV_FI(Integer) - Given a 32 bit integer, return the Floating point equivalent (with PRECISION) */
#define	CONV_FI(Integer) JLibBCONV_FI(dp, Integer)

	/*CONV_IF(Float) - Given a floating point value, return the integer equivalent (without PRECISION) */
#define	CONV_IF(Float) JLibBCONV_IF(dp, Float)

	/*CONV_FB(Source) - Return the floating point value from a source variable */
#define	CONV_FB(vptr) JLibBCONV_FB(dp, vptr)

/*
 * 32 bit conversion macros
 */
	/*CONV_IB(Source) - Return the integer value from a source variable */
#define	CONV_IB(vptr)	JLibBCONV_IB(dp, vptr)

	/*CONVCOND_IB(Source) - Return 1 or 0 for conditional evaluations. */
#define	CONVCOND_IB(Source) 	JLibBCONVCOND_IB(dp, Source)

	/*STORE_VBI(Target,Integer) - Store a 32 bit integer into a variable */
#define	STORE_VBI(Target, Source)	JLibBSTORE_VBI(dp, Target, Source)

/* 
 * 64 bit conversion macros.
 *
 * NOTE : NOT BACKWARDS COMPATIBLE WITH OLDER jBASE BUILDS
 */
	/*CONV_IB(Source) - Return the integer value from a source variable */
#define	CONV_VARINT(vptr)	JLibBCONV_VARINT(dp, vptr)

	/*CONVCOND_IB(Source) - Return 1 or 0 for conditional evaluations. */
#define	CONVCOND_VARINT(Source) 	JLibBCONVCOND_VARINT(dp, Source)

	/*STORE_VARINT(Target,Integer) - Store a 64 bit integer into a variable */
#define	STORE_VARINT(Target, Source)	JLibBSTORE_VARINT(dp, Target, Source)

	

	/*IFCONDOP_BBBBB(Result, Source, Value1, Value2) - Return value1 or value2 dependent upon test of source */
#define IFCONDOP_BBBBB(Result, Source, Value1, Value2)	JLibBIFCONDOP_BBBBB(dp, Result, Source, Value1, Value2)

	/*	CONV_SFB(Source) - Return the NULL terminated string address from a source variable */
#define	CONV_SFB(Source) JLibBCONV_SFB(dp, Source)
#define	CONV_CFB(Source) (char*)JLibBCONV_SFB(dp, Source)

	/*	CONVTYPE_IB(Source) - Return the type of variable */
#define	CONVTYPE_IB(Source)	(dp, (Source)->VAR_TYPE)

	/*CLEARVAR_V() - Clear all variables back to null */
#define	CLEARVAR_V() JLibBCLEARVAR_V(dp)

	/*DECLARE_DYN_DIM(Whence, FirstVar, NumElements, FirstElement, ... ) Allocate dynamic dimensioned array */
#define DECLARE_DYN_DIM     JLibBDDIM_VIBII

	/*Array_BB(MatrixDescriptor,FirstMatrix,FirstElement, ...) Resolve matrix address */
#define	Array_BB	JLibBARRAY_BB

	/*DYNARRAY_BB(ControlVariable, ...., -1) Resolve matrix address for dynamic dimensioned array */
#define	DYNARRAY_BB	JLibBDYNARRAY_BB
#define	DYNARRAY1_BBI	JLibBDYNARRAY1_BBI
#define	DYNARRAY2_BBII	JLibBDYNARRAY2_BBII

/*AJM
 *SVR4, for arrays of two dimensions, we must cast the Array reference 
 *to a (VAR *).
#define	MATAB_VBB JLibBMATAB_VBB
#define	MATAA_VBB JLibBMATAA_VBB
 */
	/*MATAB_VBB(TargetDescriptor,Target,Source) - MAT X = Y */
#define	MATAB_VBB(A,B,C)	 JLibBMATAB_VBB(dp, A, (VAR *)B, C)

	/*MATAA_VBB(TargetDescriptor,Target,SourceDescriptor,Source) - MAT X = MAT Y */
#define	MATAA_VBB(A,B,C,D)	 JLibBMATAA_VBB(dp, A, (VAR *)B, C, (VAR *)D)

	/*DYNMATAA_VBB(Target, Source ) - Copy a dynamic dimensioned array, MAT X = Y */
#define	DYNMATAA_VBB(Target, Source )		JLibBDYNMATAA_VBB(dp, Target, Source )

	/*DYNMATAB_VBB(Target, Source ) - Copy a dynamic dimensioned array, MAT X = MAT Y */
#define	DYNMATAB_VBB(Target, Source )		JLibBDYNMATAB_VBB(dp, Target, Source )

	/*VAR_RESIZE_VBI(Variable,NewSize) - Make a variable of type string, keep contents of any
		existing variable data */
#define	VAR_RESIZE_VBI(jvarname, jnewsize) JLibBStoreResize_VBI(dp, jvarname,jnewsize,__FILE__,__LINE__)

	/*VAR_MAKE_STRING_VBIS(Variable,NewSize,DataToCopy) - Make a variable of type string, lose any
		existing data  and optionally copy new data to newly allocated space (DataToCopy can be NULL) */
#define	VAR_MAKE_STRING_VBIS(jvarname,jnewsize,jdataptr) JLibBStoreString_VBIS(dp, jvarname,jnewsize,jdataptr,__FILE__,__LINE__)

/* VAR_INITIALISE_VB(Variable) - Initialise a variable to a jbc program variable */
#define	VAR_INITIALISE_VB(Variable)						\
{														\
	(Variable)->VAR_TYPE=VAR_TYPE_UNASSIGNED;			\
	(Variable)->VAR_FLAGS=VAR_FLAGS_USER;				\
	(Variable)->VarUnion.String.StringAddress=NULL;		\
	(Variable)->VarUnion.String.StringLen=0;			\
	(Variable)->VarHeader.Reserved = 0;					\
	(Variable)->VarHeader.Float.UString = NULL;			\
}

/* VAR_INITIALISE_REG_VB(Variable) - Initialise a variable to a register variable */
#define	VAR_INITIALISE_REG_VB(Variable)					\
{														\
	(Variable)->VAR_TYPE = VAR_TYPE_UNASSIGNED;			\
	(Variable)->VAR_FLAGS = VAR_FLAGS_REGISTER;			\
	(Variable)->VarUnion.String.StringAddress = NULL;	\
	(Variable)->VarUnion.String.StringLen = 0;			\
	(Variable)->VarHeader.Reserved = 0;					\
	(Variable)->VarHeader.Float.UString = NULL;			\
}


/*
	Category (C) - Program flow control transfer and COMMON variables.
*/

	/*DEBUG_TRACE(Line, File) - conditionally enter the debugger */
#define	DEBUG_TRACE(Line, File ) {DEBUG_LINE = (Line|(File<<24)) ; DEBUG_CHECK}

	/*CASE_DEBUG_TRACE(Line, File) - conditionally enter the debugger -- used in the CASE statements*/
#define	CASE_DEBUG_TRACE(Line, File ) JLibADebugCheckCase(dp , Line|(File<<24)) 

	/*DEBUG_LINE_TRACE(Line, File) -  macro used to keep the line number accurate, when  no debug */
#define	DEBUG_LINE_TRACE(Line, File ) {DEBUG_LINE = (Line|(File<<24)) ; }

	/*DEBUG_ENTER() - Unconditionally enter the debugger as DEBUG statement was compiled */
#define	DEBUG_ENTER (DEBUG_FLAG) |= DB_REASON_OPCODE ; JLibCDEBUG_V(dp) ;

	/*GOTO_V(Address) - Unconditional branch to a label */
#define	GOTO_V(Address) DEBUG_CHECK ; goto Address;

	/*GOSUB_V(Address) - Unconditional branch to a subroutine at address 'Address'  */
#define	GOSUB_V(Address,Identifier,IdLabel) 				\
DEBUG_CHECK													\
if (StackData.GosubNext >= GOSUB_STACK_MAX) JLibCGOSUB_STACK_FULL_V(dp);	\
StackData.GosubStack[StackData.GosubNext++]=Identifier;		\
goto Address;												\
IdLabel: ;

	/*ONGOTO_III(Value, Max) - Computed GOTO
	   ONGOTO(Vase,Label)
	   ONGOTOEND		*/
#define	ONGOTO_III(Value, Max) DEBUG_CHECK ; switch(JLibCONGOTO_III(dp, Value,Max)) {
#define	ONGOTO(Case,Label) case Case: goto Label;
#define	ONGOTOEND	default:break;}

	/* INPUTTRAP_IBI(Value, Max) - Computed INPUTTRAP GOTO */
#define	INPUTTRAP_IBI(Value, Max) DEBUG_CHECK ; switch(JLibFINPUTTRAP_IBI(dp, Value, Max)) {

	/*ONGOSUB_IIII(Value, Max, Identifier ) - Computed GOSUB
	   ONGOSUB(Vase,Label)
	   ONGOSUBEND(Identifier)		*/
#define	ONGOSUB_IIII(Value, Max, Identifier )				\
DEBUG_CHECK													\
if (StackData.GosubNext >= GOSUB_STACK_MAX) JLibCGOSUB_STACK_FULL_V(dp);	\
StackData.GosubStack[StackData.GosubNext++]=Identifier;			\
switch(JLibCONGOSUB_III(dp, Value, Max)) {

	/* INPUTTRAP_IBII(Value, Max, Identifier) - Computed INPUTTRAP GOSUB
	   ONGOSUB(Case,Label)
	   ONGOSUBEND(Identifier)		*/

#define	INPUTTRAP_IBII(Value, Max, Identifier )				\
DEBUG_CHECK													\
if (StackData.GosubNext >= GOSUB_STACK_MAX) JLibCGOSUB_STACK_FULL_V(dp);	\
StackData.GosubStack[StackData.GosubNext++]=Identifier;			\
switch(JLibFINPUTTRAP_IBI(dp, Value, Max)) {

#define	ONGOSUB(Case,Label) case Case: goto Label;

#define	ONGOSUBEND(IdLabel)	default:--StackData.GosubNext;break;}		\
IdLabel: ;
		/*OBjEX call back function */
#define OBJEXCALLBACK(param1,param2) JLibCOBJEXCALLBACK_IBB (dp, param1,param2)

	/*RETURNF_BB - Unconditional return from a Function call */
#define	RETURNF_BB(Var) STORE_VBB(FuncResult, Var);goto _jl_return_switch;

	/*RETURNF_BS - Unconditional return from a Function call */
#define	RETURNF_BS()  STORE_VBS(FuncResult,(STRING *)"");goto _jl_return_switch;

	/*RETURN_V - Unconditional return from a subroutine call */
#define	RETURN_V  goto _jl_return_switch;

	/*RETURNTO_V(Label) - Return from a subroutine to a different point */
#define	RETURNTO_V(Label)									\
DEBUG_CHECK													\
if (StackData.GosubNext <= 0) {goto _jl_function_end;}	\
StackData.GosubNext--;									\
goto Label;	

	/*CLEARCOMMON_V() - Zero initialise unnamed common area */
#define	CLEARCOMMON_V() JLibCCLEARCOMMON_V(dp)

	/*CLEARCOMMON_VB() - Pass unnamed common area */
#define	CLEARCOMMON_VB(name) JLibCCLEARCOMMON_VB(dp, name)

	/*FOR1_VBII(Source, From, To ) - FOR loop with step of 1 */
#define	FOR1_VBII(Source,From,To)							\
if (!( JLibEmulateGetIntJBASE(dp, JBASE_EMULATE_INT_GENERIC_PRIME) && ( From > To ))) \
{ \
for (JLibCFORINIT_VBII(dp, (Source),(From),1) ; !JLibCFOR_IBII(dp, (Source),(To),1) ; )		\
{

	/*FOR_VBIII(Source, From, To, Step) - FOR loop with ANY sort of step */
#define	FOR_VBIII(Source,From,To,Step)						\
if (!(( JLibEmulateGetIntJBASE(dp, JBASE_EMULATE_INT_GENERIC_PRIME)) &&  ( ((Step <= 0) && ( To > From )) || (( Step>=0 ) && ( To < From )) ))) \
{ \
for (JLibCFORINIT_VBII(dp, (Source),(From),(Step)) ; !JLibCFOR_IBII(dp, (Source),(To),(Step)) ; )	\
{

	/*FOR1_VBBB(Source, From, To ) - FOR loop with step of 1 */
#define	FOR1_VBBB(Source, From, To)							\
if (!(JLibEmulateGetIntJBASE(dp, JBASE_EMULATE_INT_GENERIC_PRIME) && (CONV_IB(From) > CONV_IB(To)))) \
{\
for (JLibCFORINIT_VBBBB(dp, (Source), (From), NULL, (To)); !JLibCFOR_IBBB(dp, (Source), (To), NULL) ; )		\
{

	/*FOR_VBBBB(Source, From, To, Step) - FOR loop with ANY sort of step */
#define	FOR_VBBBB(Source, From, To, Step)						\
if (!((JLibEmulateGetIntJBASE(dp, JBASE_EMULATE_INT_GENERIC_PRIME)) &&  ( ((CONV_IB(Step) <= 0) && ( CONV_IB(To) > CONV_IB(From) )) || (( CONV_IB(Step)>=0 ) && ( CONV_IB(To) < CONV_IB(From) )) ))) \
{\
for (JLibCFORINIT_VBBBB(dp, (Source), (From), (Step), (To)); !JLibCFOR_IBBB(dp, (Source),(To),(Step)) ; )	\
{


	/*NEXT_V - Simple NEXT statement associated with a FOR loop */
#define	NEXT_V	DEBUG_CHECK ; } }


	/*SUBROUTINE_DATA  - Initial subroutine data */
#define	SUBROUTINE_DATA(ArgCount,jNumVars , jNumRegs)							\
	struct	StackLevelData	StackData ;						\
	VAR		*SubroutineArgs[ArgCount+1];					\
	VAR		GlobalBasicVars[jNumVars+jNumRegs+1];			\
	int		*debug_flag_pointer ;							\
	UINT32	*debug_line_pointer ;


	/*SUBROUTINE_INIT	- Initialise a function call. */
#define	SUBROUTINE_INIT JLibCSUBROUTINE_INIT_VV
#define	SUBROUTINE_INIT2 JLibCSUBROUTINE_INIT2_VV


	/*SUBROUTINE_END	- The last line in a SUBROUTINE statement */
#define	SUBROUTINE_END(var_strings, lookups)					JLibCSUBROUTINE_END_VI(dp, var_strings, lookups);

	/*COLLECTDATA_VB and RTNDATA_VB - Pass and return data to/from programs. */
#define	COLLECTDATA_VB(Source)	JLibCCOLLECTDATA_VB(dp, Source)
#define	RTNDATA_VB(Source)		JLibCRTNDATA_VB(dp, Source)

	/*DATA_VIB(DataQty, FirstElement, ...) - The DATA statement */
#define	DATA_VIB	JLibCDATA_VIB

	/*PERFORM_VBBBBBBB(Statement, Setting, Capturing, RtnList, RtnData, PassList, PassData) */
#define	PERFORM_VBBBBBBB(Statement,Setting,Capturing,RtnList,RtnData,PassList,PassData) JLibCPERFORM_IIBBBBBBB(dp, 0,Statement,Setting,Capturing,RtnList,RtnData,PassList,PassData)

	/*STOP_VI(Flags, NumberElements {, Var1 {, Var2, ... }} - Terminate a program */
#define	STOP_VI	JLibCSTOP_VI 

	/*EXIT_VI(ExitCode) - Similar to Unix exit() function */
#define	EXIT_VI(ExitCode) JLibCEXIT_VI(dp, ExitCode,0)

	/*PROCREAD_IB(Result) - The PROCREAD statement */
#define	PROCREAD_IB(Result) JLibCPROCREAD_IB(dp, Result)

	/*PROCWRITE_IB(Source) - The PROCWRITE statement */
#define	PROCWRITE_IB(Source) JLibCPROCWRITE_IB(dp, Source)

#define	CALL_IND_DYN	JLibCCALL_IND_DYN
#define	CALL_DIR_DYN	JLibCCALL_DIR_DYN
#define	CALL_DIRC_DYN	JLibCCALL_DIRC_DYN
#define CALL_FUNCTION	JLibCCALL_FUNCTION

#define CALLC_INDIRECT	JLibCCALLC_INDIRECT
#define CALLC_DIRECT	JLibCCALLC_DIRECT

#define	CALLJ_IBBBB(Result, ClassName, MethodName, Params)	JLibCCALLJ(dp, Result, ClassName, MethodName, Params)
#define	CALLdotNET_IBBBB(Result, ClassName, MethodName, Params)	JLibCCALLdotNET(dp, Result, ClassName, MethodName, Params)
	/*	CHAIN_VB(Command) and ENTER_VB(Command) - exec() to another program. */
#define	CHAIN_VB(Command)	JLibCCHAIN_VB(dp, Command)
#define	ENTER_VB(Command)	JLibCENTER_VB(dp, Command)

/*
 * CALLONEXIT_IB -- Define a function to call when program exits. */
#define	CALLONEXIT_IB(Var) JLibCCALLONEXIT_IB(dp, Var)

#define JBASE_CRYPT_GEN			0
#define	JBASE_CRYPT_ROT13		1	
#define	JBASE_CRYPT_XOR11		2	
#define	JBASE_CRYPT_RC2			3	
#define	JBASE_CRYPT_BASE64		4
#define	JBASE_CRYPT_DES			5
#define	JBASE_CRYPT_3DES		6
#define JBASE_CRYPT_BLOWFISH	7
#define JBASE_CRYPT_AES			8
#define JBASE_CRYPT_SECURE_FILE	9
#define JBASE_CRYPT_AES_CTR		10

#define JBASE_SECUREHASH_SHA256  32

#define	JBASE_CRYPT_BASE64_MASK	0x40000000


int JRunEncModeNarr(char *);				/* Encryption mode narratives */

#define	DECRYPT_BBBBI(Result, Source, Key, Method)	JLibEDECRYPT_BBBBI(dp, Result, Source, Key, Method)
#define	ENCRYPT_BBBBI(Result, Source, Key, Method)	JLibEENCRYPT_BBBBI(dp, Result, Source, Key, Method)

	/*ATTACH_COMMON(NameOfCommon, FirstPtr, CommonQty, CommonThreadKey ) - Attach common to global area */
#define	ATTACH_COMMON(Common, Pointer, Qty, Key)	JLibCAttachCommon_VSSVI(dp, Common, Pointer, Qty, Key)
	/*RELOAD_COMMON(NameOfCommon, FirstPtr, CommonQty, CommonThreadKey ) - Reload common to global area, following Subroutine call */
#define	RELOAD_COMMON(Common, Pointer, Qty, Key)	JLibCReloadCommon_VSSVI(dp, Common, Pointer, Qty, Key)
/*
	Category (D) - Mathematical functions.
*/
#define	DYN_FUNCTION_ADD		0
#define	DYN_FUNCTION_SUB		1
#define	DYN_FUNCTION_MUL		2
#define	DYN_FUNCTION_DIV		3
#define	DYN_FUNCTION_MOD		4
#define	DYN_FUNCTION_OR			10
#define	DYN_FUNCTION_AND		11
#define	DYN_FUNCTION_CAT		12
#define	DYN_FUNCTION_EQ			13
#define	DYN_FUNCTION_NE			14
#define	DYN_FUNCTION_GE			15
#define	DYN_FUNCTION_GT			16
#define	DYN_FUNCTION_LE			17
#define	DYN_FUNCTION_LT			18
#define	DYN_FUNCTION_SPACES		100
#define	DYN_FUNCTION_ABSS		101
#define	DYN_FUNCTION_CHARS		102
#define	DYN_FUNCTION_LENS		103
#define	DYN_FUNCTION_NEGS		104
#define	DYN_FUNCTION_NUMS		105
#define	DYN_FUNCTION_SEQS		106
#define	DYN_FUNCTION_NOTS		107
#define	DYN_FUNCTION_STRS		108
#define	DYN_FUNCTION_LEFT		109
#define	DYN_FUNCTION_RIGHT		110
#define	DYN_FUNCTION_ICONVS		111
#define	DYN_FUNCTION_OCONVS		112
#define	DYN_FUNCTION_FMTS		113
#define	DYN_FUNCTION_SUBSTRINGS	114
#define	DYN_FUNCTION_LENSDP		115
#define	DYN_FUNCTION_SUBSTRINGSEND	116


	/*ACOS_FF(Value) - Return the inverse cosine of Value */
#define	ACOS_FF(Value) JLibDACOS_FF(dp, Value)

	/*ASIN_FF(Value) - Return the inverse sine of Value */
#define	ASIN_FF(Value) JLibDASIN_FF(dp, Value)

	/*ATAN_FF(Value) - Return the inverse tangent of Value */
#define	ATAN_FF(Value) JLibDATAN_FF(dp, Value)

	/*FCMP_IBB(Var1,Var2) - Return the real floating point compare of Var1 Var2 */
#define	FCMP_IBB(Var1, Var2) JLibDCMP_IBBI(dp, Var1, Var2, 1 )

	/*SCMP_IBB(Var1,Var2) - Return the string compare of Var1 Var2 */
#define	SCMP_IBB(Var1, Var2) JLibDCMP_IBBI(dp, Var1, Var2, 2 )

	/*FADD_BBBB(target, source1, source2) - Floating point ADD */
#define	FADD_BBBB(target,source1,source2) JLibDFMATH_BBBBII(dp, target,source1,source2,0,DYN_FUNCTION_ADD)

	/*FSUB_BBBB(target, source1, source2) - Floating point SUBTRACT */
#define	FSUB_BBBB(target,source1,source2) JLibDFMATH_BBBBII(dp, target,source1,source2,0,DYN_FUNCTION_SUB)

	/*FDIV_BBBB(target, source1, source2) - Floating point DIVIDE */
#define	FDIV_BBBB(target,source1,source2) JLibDFMATH_BBBBII(dp, target,source1,source2,0,DYN_FUNCTION_DIV)

	/*FMUL_BBBB(target, source1, source2) - Floating point MULTIPLY */
#define	FMUL_BBBB(target,source1,source2) JLibDFMATH_BBBBII(dp, target,source1,source2,0,DYN_FUNCTION_MUL)

	/*FFIX_BBBI(target, source, precision) - Convert real floating point to fixed floating point */
#define	FFIX_BBBI(target,source,precision)	JLibDFMATH_BBBBII(dp, target,source,NULL,precision,4)

	/*FFLT_BBB(target, source) - Convet to a real floating point value */
#define	FFLT_BBB(target, source)	JLibDFMATH_BBBBII(dp, target, source, NULL, 0, 5)

	/*GETUSERGROUP_BBI(target, uid) - Get Unix group id from Uid */
#define	GETUSERGROUP_BBI(target, uid)	JLibEGETUSERGROUP_BBI(dp, target, uid)

	/*GETUSERNAME_BBI(target, uid) - Get Unix user name from Uid */
#define	GETUSERNAME_BBI(target, uid)	JLibEGETUSERNAME_BBI(dp, target, uid)

	/*DROUND_BBBB(target, source, precision) - Double precision rounding */
#define	DROUND_BBBB(target, source, precision)		JLibDDROUND_BBBB(dp, target, source, precision)

	/*ADD_BBBB(Result,Var1,Var2) - Return the calculation Var1+Var2 */
#define	ADD_BBBB(Result,Var1,Var2) JLibDMATHADD_BBBB(dp, Result,Var1,Var2)

	/*SUB_BBBB(Result,Var1,Var2) - Return the calculation Var1-Var2 */
#define	SUB_BBBB(Result,Var1,Var2) JLibDMATHSUB_BBBB(dp, Result,Var1,Var2)

	/*MUL_BBBB(Result,Var1,Var2) - Return the calculation Var1*Var2 */
#define	MUL_BBBB(Result,Var1,Var2) JLibDMATHMUL_BBBB(dp, Result,Var1,Var2)

	/*DIV_BBBB(Result,Var1,Var2) - Return the calculation Var1/Var2 */
#define	DIV_BBBB(Result,Var1,Var2) JLibDMATHDIV_BBBB(dp, Result,Var1,Var2)

	/*
	 *String arithmetic :
	 *SADD_BBBB(target, source1, source2)
	 *SSUB_BBBB(target, source1, source2)
	 *SMUL_BBBB(target, source1, source2)
	 *SDIV_BBBB(target, source1, source2)
	 */
#define	SADD_BBBBI(target, source1, source2, precision) JLibDSMATH_BBBBII(dp, target, source1, source2, precision, DYN_FUNCTION_ADD)
#define	SSUB_BBBBI(target, source1, source2, precision) JLibDSMATH_BBBBII(dp, target, source1, source2, precision, DYN_FUNCTION_SUB)
#define	SMUL_BBBBI(target, source1, source2, precision) JLibDSMATH_BBBBII(dp, target, source1, source2, precision, DYN_FUNCTION_MUL)
#define	SDIV_BBBBI(target, source1, source2, precision) JLibDSMATH_BBBBII(dp, target, source1, source2, precision, DYN_FUNCTION_DIV)

	/*DIV_FFF(Var1,Var2) - Return the calculation Var1/Var2 */
#define	DIV_FFF(Var1, Var2) JLibDDIV_FFF(dp, Var1, Var2)

	/*PWR_FFF(Var1,Var2) - Return the calculation Var1^Var2 */
#define	PWR_FFF(Var1, Var2) JLibDPWR_FFF(dp, Var1, Var2)

	/*MOD_BBBB(Target,Var1,Var2) - Return the calculation Var1%Var2 */
#define	MOD_BBBB(Target,Var1,Var2) JLibDMOD_BBBB(dp, Target,Var1,Var2)

	/*LN_FF(Value) - Return the of natural logarithm Value */
#define	LN_FF(Value) JLibDLN_FF(dp, Value)

	/*SIN_FF(Value) - Return the of sine Value */
#define	SIN_FF(Value) JLibDSIN_FF(dp, Value)

	/*SQRT_FF(Value) - Return the square root of Value */
#define	SQRT_FF(Value) JLibDSQRT_FF(dp, Value)

	/*TAN_FF(Value) - Return the tangent of Value */
#define	TAN_FF(Value) JLibDTAN_FF(dp, Value)

	/*EXP_FF(Value) - Return the exponential of Value */
#define	EXP_FF(Value) JLibDEXP_FF(dp, Value)

	/*COS_FF(Value) - Return the cosine of Value */
#define	COS_FF(Value) JLibDCOS_FF(dp, Value)

	/*INT_IF(Value) - Return the integer portion of Value */
#define	INT_IF(Value) JLibBINT_IF(dp, Value)

	/*INT_BBB(Target, Source) - Replaces INT_IF, Return the integer portion of Source */
#define	INT_BBB(Target, Source)	JLibBINT_BBB(dp, Target, Source)

	/*INDICES_BBBB(Result, FileVar, IndexName) - Returns names of assocaited indexes */
#define INDICES_BBBB(Result, FileVar, IndexName)	JLibFINDICES_BBBB(dp, Result, FileVar, IndexName)

    /*ITYPE_BBBB(Result, ItypeRef) - Returns evaluated Itype value */
#define ITYPE_BBBB(Target, ItypeReg, ItypeRef) JLibGItypeRun(dp, ItypeReg, Target, ItypeRef)

	/*FOLD_BBBB(Result, Source, Length) - Returns evaluated Itype value */
#define FOLD_BBBB(Result, Source, Length)	JLibEFOLD_BBBB(dp, Result, Source, Length)
#define FOLD_BBBBB(Result, Source, Length, Delim)	JLibEFOLD_BBBBB(dp, Result, Source, Length, Delim)

	/*LOWER_BBB(Result, DynamicArray) - Returns arry with delimiters lowered */
#define LOWER_BBB(Result, Array) JLibELOWER_BBB(dp, Result, Array)

	/*NUM_IB(Value) - Given a variable, return 1 if it is a numeric, 0 if not */
#define	NUM_IB(Value) JLibDNUM_IB(dp, Value)

	/*RND_BBB(Target,Source) - Given an integer seed, return a random floating point number */
#define RND_BBB(Target,Source) JLibDRND_BBB(dp, Target,Source)

	/*NEG_BBB(Target,Source) - Return the negative value of Value */
#define	NEG_BBB(Target,Source) JLibDNEG_BBB(dp, Target,Source)

	/*ABS_BBB(Target,Source) - Return the ABSolute value */
#define	ABS_BBB(Target,Source) JLibDABS_BBB(dp, Target,Source)

	/*EQ_IBB(Source1, Source2) - Return 1 if Source1 EQ Source2 else return 0 */
#define	EQ_IBB(Source1,Source2) JLibDCOMPARE_IBBI(dp, Source1,Source2,0)

	/*NE_IBB(Source1, Source2) - Return 1 if Source1 NE Source2 else return 0 */
#define	NE_IBB(Source1,Source2) JLibDCOMPARE_IBBI(dp, Source1,Source2,1)

	/*LT_IBB(Source1, Source2) - Return 1 if Source1 LT Source2 else return 0 */
#define	LT_IBB(Source1,Source2) JLibDCOMPARE_IBBI(dp, Source1,Source2,2)

	/*LE_IBB(Source1, Source2) - Return 1 if Source1 LE Source2 else return 0 */
#define	LE_IBB(Source1,Source2) JLibDCOMPARE_IBBI(dp, Source1,Source2,3)

	/*GT_IBB(Source1, Source2) - Return 1 if Source1 GT Source2 else return 0 */
#define	GT_IBB(Source1,Source2) JLibDCOMPARE_IBBI(dp, Source1,Source2,4)

	/*GE_IBB(Source1, Source2) - Return 1 if Source1 GE Source2 else return 0 */
#define	GE_IBB(Source1,Source2) JLibDCOMPARE_IBBI(dp, Source1,Source2,5)

	/*IFS_BBBBB(Result, Conditional, True, False) - Dynamic array testing */
#define	IFS_BBBBB(Result, Conditional, True, False)	JLibDIFS_BBBBB(dp, Result, Conditional, True, False)

	/*Pre and post increment and decrement of numerics */
#define	POSTI_BBB(Target, Source)	JLibDPOSTI_BBB(dp, Target, Source)
#define	POSTD_BBB(Target, Source)	JLibDPOSTD_BBB(dp, Target, Source)
#define	PREI_BB(Source)		JLibDPREI_BB(dp, Source)
#define	PRED_BB(Source)		JLibDPRED_BB(dp, Source)
#define	POSTI_VB(Source)	JLibDPOSTI_VB(dp, Source)
#define	POSTD_VB(Source)	JLibDPOSTD_VB(dp, Source)
#define	PREI_VB(Source)		JLibDPOSTI_VB(dp, Source)
#define	PRED_VB(Source)		JLibDPOSTD_VB(dp, Source)

	/*
	 *Dynamic array functions.
	 */
#define		ADDS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_ADD)
#define		SUBS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_SUB)
#define		MULS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_MUL)
#define		DIVS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_DIV)
#define		MODS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_MOD)
#define		ANDS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_AND)
#define		ORS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_OR)
#define		CATS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_CAT)
#define		EQS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_EQ)
#define		NES_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_NE)
#define		GES_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_GE)
#define		GTS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_GT)
#define		LES_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_LE)
#define		LTS_BBBB(result, source1, source2 ) JLibDDYN_ARRAY_BBBBI(dp, result, source1, source2, DYN_FUNCTION_LT)

/*
	Category (E) - Intrinsic functions.
*/

	/*LEN_FB(Source) - Returns the length of a string. */
#define	LEN_IB(vptr) JLibELEN_IB(dp, vptr)
#define	BYTELEN_IB(vptr) JLibELEN_IB(dp, vptr)

	/*LENDP_IB(Source) - Returns the display length of a string. */
#define	LENDP_IB(vptr) JLibELENDP_IB(dp, vptr)

	/*LENC_FB(Source) - Returns the length of a string in characters. */
#define	LENC_IB(vptr) JLibELENC_IB(dp, vptr)

	/*CHAR_BBI(Number) - Store the 'Number' as a 1 byte text string */
#define	CHAR_BBI(Target, Number) JLibECHAR_BBI(dp, Target, Number)

	/*CAT_BBBB(Target, Source1, Source2) - Concatenate two variables onto a third */
#define	CAT_BBBB(Target, Source1, Source2) JLibECAT_BBBB(dp, Target, Source1, Source2)

	/*BBI(Target, VarQty, ...) Concatenate 'VarQty' variables */
#define	MCAT_BBI JLibEMCAT_BBI

	/*INDEX_IBBI(Source, String, Count) - Find the 'n'th position of a sub-string */
#define	INDEX_IBBI(Source, String, Count) JLibEINDEX_IBBI(dp, Source, String, Count)

	/*RAISE_BBB(Result, DynamicArray) - Returns array with delimiters raised */
#define	RAISE_BBB(Result, DynamicArray) JLibERAISE_BBB(dp, Result,DynamicArray)

	/*REUSE_BB(DynamicArray) - Returns last delimited element */
#define	REUSE_BB(Source) JLibDREUSE_BB(dp, Source)

	/*SPACE_BBI(Target, Count) - Create a string made up of 'Count' spaces */
#define	SPACE_BBI(Target, Count) JLibESPACE_BBI(dp, Target, Count)

	/*SPACES_BBB(Result, Source) - Create array elements spaced as source */
#define	SPACES_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_SPACES,0,0)

	/*ABSS_BBB(Result, Source) - Create array elements ABS as source */
#define	ABSS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_ABSS,0,0)

	/*CHARS_BBB(Result, Source) - Create array elements CHARS as source */
#define	CHARS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_CHARS,0,0)

	/*LENS_BBB(Result, Source) - Create array elements LENS as source */
#define	LENS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_LENS,0,0)

	/*LENSDP_BBB(Result, Source) - Create array elements LENSDP as source */
#define	LENSDP_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_LENSDP,0,0)

	/*NEGS_BBB(Result, Source) - Create array elements NEGS as source */
#define	NEGS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_NEGS,0,0)

	/*NUMS_BBB(Result, Source) - Create array elements NUMS as source */
#define	NUMS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_NUMS,0,0)

	/*SEQS_BBB(Result, Source) - Create array elements SEQS as source */
#define	SEQS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_SEQS,0,0)

	/*NOTS_BBB(Result, Source) - Create array elements NOTS as source */
#define	NOTS_BBB(Result, Source)	JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_NOTS,0,0)

	/*STRS_BBBI(Result, Source, Count) - Duplicate array 'Count' times */
#define	STRS_BBBI(Result, Source, Count) JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL, NULL,DYN_FUNCTION_STRS,Count,0)

	/*LEFT_BBBI(Result, Source, Count) - Leave just LHS of string */
#define	LEFT_BBBI(Result, Source, Count) JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_LEFT,Count,0)

	/*RIGHT_BBBI(Result, Source, Count) - Leave just RHS of string */
#define	RIGHT_BBBI(Result, Source, Count) JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_RIGHT,Count,0)

	/*OCONVS_BBBBB(Result, Source, Conversion) -- Multiple OCONV conversions */
#define	OCONVS_BBBBB(Result, Convreg, Source, Conversion) JLibE_DYNARRAY_BBBBIII(dp, Result, Source, Conversion, Convreg, DYN_FUNCTION_OCONVS, 0, 0 )

	/*ICONVS_BBBBB(Result, Source, Conversion) -- Multiple ICONV conversions */
#define	ICONVS_BBBBB(Result, Convreg, Source, Conversion) JLibE_DYNARRAY_BBBBIII(dp, Result, Source, Conversion, Convreg, DYN_FUNCTION_ICONVS, 0, 0 )

	/*FMTS_BBBBB(Result, Source, Conversion) -- Multiple FMT conversions */
#define	FMTS_BBBBB(Result, Convreg, Source, Conversion) JLibE_DYNARRAY_BBBBIII(dp, Result, Source, Conversion, Convreg, DYN_FUNCTION_FMTS, 0, 0 )

	/*SUBSTRINGS_BBBII(Result, Source, Start, Length) -- Extract sub-strings */
#define	SUBSTRINGS_BBBII(Result, Source, Start, Length) JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_SUBSTRINGS,Start,Length)

	/*SUBSTRINGS_BBBI(Result, Source, Length) -- Extract sub-strings from end*/
#define	SUBSTRINGS_BBBI(Result, Source, Length) JLibE_DYNARRAY_BBBBIII(dp, Result,Source,NULL,NULL,DYN_FUNCTION_SUBSTRINGSEND,Length,0)

	/*STR_BBBI(Target, Source, Count) - Duplicate a string 'Count' times */
#define	STR_BBBI(Target, Source, Count) JLibESTR_BBBI(dp, Target, Source, Count)

	/*SPLICE_BBBBB(Result, Source1, Separator, Source2) - Splice two dynamic arrays */
#define	SPLICE_BBBBB(Result, Source1, Separator, Source2)	JLibDSPLICE_BBBBB(dp, Result, Source1, Separator, Source2)

	/*TRIM_xxx(..., ...) - Trim functions ... */
#define	TRIM_BBB(Target, Source)					JLibETRIM_BIBBBB(dp, 0x17	, Target, Source, 0, 0)
#define	TRIMF_BBB(Target, Source)					JLibETRIM_BIBBBB(dp, 0x11	, Target, Source, 0, 0)
#define	TRIMB_BBB(Target, Source)					JLibETRIM_BIBBBB(dp, 0x12	, Target, Source, 0, 0)
#define	TRIM_BBBB(Target, Source, Char )			JLibETRIM_BIBBBB(dp, 0x27	, Target, Source, Char, 0 )
#define	TRIM_BBBBB(Target, Source, Char, Type )	JLibETRIM_BIBBBB(dp, 0x120	, Target, Source, Char, Type )
#define	TRIMS_BBB(Target, Source)					JLibETRIMS_BIBB(dp, 0x17	, Target, Source)
#define	TRIMBS_BBB(Target, Source)					JLibETRIMS_BIBB(dp, 0x12	, Target, Source)
#define	TRIMFS_BBB(Target, Source)					JLibETRIMS_BIBB(dp, 0x11	, Target, Source)

	/* DEPRECATED -- REGEXP_IBB(Source, Expression) - Regular expression */
#define	REGEXP_IBB(Source, Expression) JLibEREGEXP_IBB(dp, Source, Expression)

	/*REGEXP_IBBI(Source, Expression, Flags) - Regular expression */
#define	REGEXP_IBBI(Source, Expression,Flags) JLibEREGEXP_IBBI(dp, Source, Expression,Flags)

	/*ALPHA_IB(Source) - Return 1 if string is alphabetic characters else return 0 */
#define	ALPHA_IB(Source) JLibEALPHA_IB(dp, Source)

	/*MATCHES_IBB(Source, Pattern) - Return 1 if Source matches Pattern else return 0 */
#define	MATCHES_IBB(Source,Pattern) JLibEMATCHES_IBB(dp, Source,Pattern)

	/*MATCHFIELD_BBBBI(Result, Source, Pattern, Count) - Returns portion of string that matches specified pattern */
#define	MATCHFIELD_BBBBI(Result, Source, Pattern, Count) JLibEMATCHFIELD_BBBBI(dp, Result, Source, Pattern, Count)

	/*COUNT_IBB(Source, Delimiter) - Return number of occurences of Delimiter in Source */
#define	COUNT_IBB(Source,Delimiter) JLibECOUNT_IBB(dp, Source,Delimiter)

	/*COUNTS_IBB(Source, Delimiter) - Return number of occurences of Delimiter in Source */
#define	COUNTS_IBB(Source,Delimiter) JLibECOUNTS_IBB(dp, Source,Delimiter)

	/*COUNTS_BBBB(Source, Delimiter) - Return number of occurences of Delimiter in Source Array, in a dynamic array*/
#define	COUNTS_BBBB(Result, Source, Delimiter) JLibECOUNTS_BBBB(dp,Result,Source,Delimiter)

	/*DCOUNT_IBB(Source, Delimiter) - Return number of fields delimiter by Delimiter in Source */
#define	DCOUNT_IBB(Source,Delimiter) JLibEDCOUNT_IBB(dp, Source,Delimiter)

	/*EXTRACTAM_BBBI(Result, Source, Attribute) - Extract an attribute */
#define	EXTRACTAM_BBBI(Result, Source, Attribute) JLibEEXTRACT_BBBIII(dp, Result, Source, Attribute, 0, 0)

	/*EXTRACTMV_BBBII(Result, Source, Attribute, MultiValue ) - Extract a multi-value */
#define	EXTRACTMV_BBBII(Result, Source, Attribute, MultiValue ) JLibEEXTRACT_BBBIII(dp, Result, Source, Attribute, MultiValue, 0)

	/*EXTRACTSV_BBBII(Result, Source, Attribute, MultiValue, SubValue ) - Extract a sub-value */
#define	EXTRACTSV_BBBIII(Result, Source, Attribute, MultiValue, SubValue) JLibEEXTRACT_BBBIII(dp, Result, Source, Attribute, MultiValue, SubValue)

	/*DELETETAM_BBBI(Result, Source, Attribute) - Delete an attribute */
#define	DELETEAM_BBBI(Result, Source, Attribute) JLibEDELETE_BBBIII(dp, Result, Source, Attribute, 0, 0)

	/*DELETETMV_BBBII(Result, Source, Attribute, MultiValue ) - Delete a multi-value */
#define	DELETEMV_BBBII(Result, Source, Attribute, MultiValue ) JLibEDELETE_BBBIII(dp, Result, Source, Attribute, MultiValue, 0)

	/*DELETETSV_BBBII(Result, Source, Attribute, MultiValue, SubValue ) - Delete a sub-value */
#define	DELETESV_BBBIII(Result, Source, Attribute, MultiValue, SubValue) JLibEDELETE_BBBIII(dp, Result, Source, Attribute, MultiValue, SubValue)

	/*INSERTAM_BBBIB(Result,Source,Attribute,NewString) - Insert before specified attribute */
#define	INSERTAM_BBBIB(Result,Source,Attribute,NewString) JLibEINSERT_BBBBIII(dp, Result,Source,NewString,Attribute,0,0)

        /*As above but different function to differentiate between INSERT function and INS statement Note 'F' */
#define INSERTFAM_BBBIB(Result,Source,Attribute,NewString) JLibEINSERTF_BBBBIII(dp, Result,Source,NewString,Attribute,0,0)

	/*INSERTMV_BBBIIB(Result,Source,Attribute,MultiValue,NewString) - Insert before specified multi-value */
#define	INSERTMV_BBBIIB(Result,Source,Attribute,MultiValue,NewString) JLibEINSERT_BBBBIII(dp, Result,Source,NewString,Attribute,MultiValue,0)
        /*As above but different function to differentiate between INSERT function and INS statement Note 'F' */
#define INSERTFMV_BBBIIB(Result,Source,Attribute,MultiValue,NewString) JLibEINSERTF_BBBBIII(dp, Result,Source,NewString,Attribute,MultiValue,0)
	/*INSERTSV_BBBIIIB(Result,Source,Attribute,MultiValue,SubValue,NewString) - Insert before specified sub-value */
#define	INSERTSV_BBBIIIB(Result,Source,Attribute,MultiValue,SubValue,NewString) JLibEINSERT_BBBBIII(dp, Result,Source,NewString,Attribute,MultiValue,SubValue)

        /*As above but different function to differentiate between INSERT function and INS statement Note 'F' */
#define INSERTFSV_BBBIIIB(Result,Source,Attribute,MultiValue,SubValue,NewString) JLibEINSERTF_BBBBIII(dp, Result,Source,NewString,Attribute,MultiValue,SubValue)

	/*SUBSTR_BBBII(Result, Source, StartPos, Length) - Return a sub-string from the Source */
#define	SUBSTR_BBBII(Result, Source, StartPos, Length) JLibESUBSTR_BBBII(dp, Result, Source, StartPos, Length)

	/*SUBSTR_BBBI(Result, Source, Length) - Return a sub-string from the end of Source */
#define	SUBSTR_BBBI(Result, Source, Length) JLibESUBSTR_BBBI(dp, Result, Source, Length)

	/*
	**Old Complex Store Macro still referenced in code
	**COMPSTORE(Target,AM,MV,SV,StStart,StEnd,Flags,Source) - Complex store
	*/
#define	COMPSTORE(Target,AM,MV,SV,StStart,StEnd,Flags,Source)	JLibECOMPLEX_STORE_VBIIIIIIB(dp, Target,AM,MV,SV,StStart,StEnd,Flags,Source)

	/*
	**New Complex Store Macro renamed to COMPSTORX as old macro used 
	**COMPSTORX(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)
	*/
#define	COMPSTORX(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source) JLibECOMPLEX_STORE_VBIIIBIIIB(dp, Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)

/*AJM
   These macros are for Complex store with assignment operator.
   Note: The values chosen for Assign op must not overlap with the 
   the other ASSIGN bits (currently only Least Significant Byte)
*/
#define		ASSIGN_OP			0xf000		/*Mask for Assign op */
#define		ASSIGN_OP_ADD		0x1000		/*+= */
#define		ASSIGN_OP_SUB		0x2000		/*-= */
#define		ASSIGN_OP_MUL		0x3000		/**= */
#define		ASSIGN_OP_DIV		0x4000		/*/= */
#define		ASSIGN_OP_CAT		0x5000		/*:= */

		/*COMPSTORA(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source) - Complex store with ADD ie += */
#define	COMPSTORA(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)	\
	JLibECOMPLEX_STORE_VBIIIBIIIB(dp, Target,AM,MV,SV,Delim,StStart,StEnd,Flags|ASSIGN_OP_ADD, Source)

		/*COMPSTORS(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source) - Complex store with Subtract ie -= */
#define	COMPSTORS(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)	\
	JLibECOMPLEX_STORE_VBIIIBIIIB(dp, Target,AM,MV,SV,Delim,StStart,StEnd,Flags|ASSIGN_OP_SUB, Source)

		/*COMPSTORM(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source) - Complex store with Multiply ie *= */
#define	COMPSTORM(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)	\
	JLibECOMPLEX_STORE_VBIIIBIIIB(dp, Target,AM,MV,SV,Delim,StStart,StEnd,Flags|ASSIGN_OP_MUL, Source)

		/*COMPSTORD(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source) - Complex store with Divide /= */
#define	COMPSTORD(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)	\
	JLibECOMPLEX_STORE_VBIIIBIIIB(dp, Target,AM,MV,SV,Delim,StStart,StEnd,Flags|ASSIGN_OP_DIV, Source)

		/*COMPSTORC(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source) - Complex store with Concatinate := */
#define	COMPSTORC(Target,AM,MV,SV,Delim,StStart,StEnd,Flags,Source)	\
	JLibECOMPLEX_STORE_VBIIIBIIIB(dp, Target,AM,MV,SV,Delim,StStart,StEnd,Flags|ASSIGN_OP_CAT, Source)

		/* COMPSTORAPPEND_VBB(Target , Source) -- Simple 'Target<-1> = Source' */
#define	COMPSTORAPPEND_VBB(Target , Source)								\
	JLibECOMPLEX_STORE_VBB(dp , Target , Source)

		/* COMPSTORAPPEND_VBIS(Target , Source) -- Simple 'Target<-1> = Source' */
#define	COMPSTORAPPEND_VBIS(Target , SourceLen , SourceAddr )								\
	JLibECOMPLEX_STORE_VBIS(dp , Target , SourceLen , SourceAddr)


	/*REPLACEAM_BBBIB(Result,Source,Attribute,NewString) - Replace one string with another */
#define	REPLACEAM_BBBIB(Result,Source,Attribute,NewString) JLibEREPLACE_BBBBIII(dp, Result,Source,NewString,Attribute,0,0)

	/*REPLACEMV_BBBIIB(Result,Source,Attribute,MultiValue,NewString) - Replace one string with another */
#define	REPLACEMV_BBBIIB(Result,Source,Attribute,MultiValue,NewString) JLibEREPLACE_BBBBIII(dp, Result,Source,NewString,Attribute,MultiValue,0)

	/*REPLACESV_BBBIIIB(Result,Source,Attribute,MultiValue,SubValue,NewString) - Replace one string with another */
#define	REPLACESV_BBBIIIB(Result,Source,Attribute,MultiValue,SubValue,NewString) JLibEREPLACE_BBBBIII(dp, Result,Source,NewString,Attribute,MultiValue,SubValue)

	/*LOCATE_IBBBIIIB(Result,SubString,Source,Attribute,MultiValue,StartPos,Sequence) - Locate a SubString in the Source */
#define LOCATE_IBBBIIIB(Result,SubString,Source,Attribute,MultiValue,StartPos,Sequence) JLibELOCATE_IBBBIIIIB(dp, Result,SubString,Source,Attribute,MultiValue,0,StartPos,Sequence)
#define LOCATE_IBBBIIIIB(Result,SubString,Source,Attribute,MultiValue,SubValue,StartPos,Sequence) JLibELOCATE_IBBBIIIIB(dp, Result,SubString,Source,Attribute,MultiValue,SubValue,StartPos,Sequence)

	/*GETENV_IBB(EnvName, Result ) - our equivalent of getenv() */
#define	GETENV_IBB(EnvName, Result ) JLibEGETENV_IBB(dp, EnvName, Result )

	/*GETENV_BBB(Result, EnvName ) - Unidate equivalent of getenv() */
#define	GETENV_BBB(Result, EnvName)		 JLibEGETENV_BBB(dp, Result, EnvName)

	/*PUTENV_IB(NewEnv) - equivalent to putenv() */
#define	PUTENV_IB(NewEnv) JLibEPUTENV_IB(dp, NewEnv)

	/*DELENV_IB(NewEnv) - equivalent to delenv() */
#define	DELENV_IB(EldEnv) JLibEDELENV_IB(dp, EldEnv)

	/*GETCWD_IB(Result) - equivalent to getcwd() */
#define	GETCWD_IB(Result) JLibEGETCWD_IB(dp, Result)

	/*CHDIR_IB(NewDir) - equivalent to chdir() */
#define	CHDIR_IB(NewDir) JLibECHDIR_IB(dp, NewDir)

	/*PERROR_BBB(Message, Errno) - equivalent to perror() */
#define	PERROR_BBB(Message, Errno) JLibEPERROR_BBB(dp, Message,Errno)

	/*ERRTEXT_BBBB( Result, ErrNum {, Var1 {, Var2, ... }}  - Returnsspecified errmsg */
#define ERRTEXT_BBBB

	/*SLEEP_VB(SleepTime) - sleep for n seconds or until certain time */
#define	SLEEP_VB(SleepTime) JLibESLEEP_VB(dp, SleepTime)

	/*MSLEEP_VB(SleepTime) - sleep for n milliseconds */
#define	MSLEEP_VB(SleepTime)	JLibEMSLEEP_VB(dp, SleepTime)

	/*CLEARDATA_V() - Clear stacked data */
#define	CLEARDATA_V()	JLibFCLEARDATA_V(dp)

	/*COMPARES_IBBB(Source1, Source2, Type) - Compare two strings */
#define	COMPARES_IBBB(Source1, Source2, Type)	JLibECOMPARES_IBBB(dp, Source1, Source2, Type)

	/*REMOVE_VBBBB(Target, Source, Colpos, Setting) Remove a substring from a variable */
#define	REMOVE_VBBBB(Target, Source, Colpos, Setting)	JLibEREMOVE_VBBBB(dp, Target, Source, Colpos, Setting)
#define	REMOVE_BBBB(Target, Source, Setting)	JLibEREMOVE_BBBB(dp, Target, Source, Setting)

	/*ASSIGNSYS_VBI(Source, SystemNumber) - Assign values to SYSTEM() values */
#define	ASSIGNSYS_VBI(Source, SystemNumber) JLibEASSIGNSYS_VBI(dp, Source, SystemNumber)

	/*INMAT_BBBA(target, source) Support the INMAT function */
#define	INMAT_BBBA(target, source, sourcedesc)	JLibEINMAT_BBBA(dp, target, source, sourcedesc)

	/*SYSTEM_BBI(Result, SystemNumber) - The SYSTEM() function */
#define	SYSTEM_BBI(Result, SystemNumber) JLibESYSTEM_BBI(dp, Result, SystemNumber)

	/*LOCK_III(Flags, LockNumber) - Take a SYSTEM lock */
#define	LOCK_III(Flags, LockNumber) JLibELOCK_IIII(dp, Flags, 0, LockNumber)

	/*UNLOCK_VI(LockNumber) - Unlock a single SYSTEM lock */
#define	UNLOCK_VI(LockNumber) JLibELOCK_IIII(dp, 0, 1, LockNumber)

	/*UNLOCK_V() - Unlock ALL current SYSTEM locks */
#define	UNLOCK_V() JLibELOCK_IIII(dp, 0, 2, 0)

	/*MATPARSE(Array,ArrayInfo,Start,End,String,Delim,Setting) - Build a dimensioned array from a string */
#define	MATPARSE_VBIIBBB(Array,ArrayInfo,Start,End,String,Delim,Setting)	JLibEMATPARSE_VBIIBBB(dp, Array,ArrayInfo,Start,End,String,Delim,Setting)

	/*MATBUILD(String,Array,ArrayInfo,Start,End,Delim) - Build a string from a dimensioned array */
#define	MATBUILD_VBBIIB(String,Array,ArrayInfo,Start,End,Delim)	JLibEMATBUILD_VBBIIB(dp, String,Array,ArrayInfo,Start,End,Delim)

	/*	ESORT_BBBBII(Result, Source, Keys, NumAtts, NumKeys ) - Exploding sort */
#define	ESORT_BBBBII(Result, Source, Keys, NumAtts, NumKeys )	JLibEESORT_BBBBII(dp, Result, Source, Keys, NumAtts, NumKeys)

	/*	SORT_BBB(Result, Source) - Ascending attribute sort */
#define	SORT_BBB(Result, Source) JLibESORT_BBB(dp, Result, Source)

	/*	CHECKSUM_IB(Result, Source) - Return a checksum */
#define	CHECKSUM_BBB(Result, Source) JLibECHECKSUM_BBB(dp, Result, Source)

	/*	GROUP_BBBBII(Result, Source, Delimiter, First, Last)  - Return a group of delimited string */
#define	GROUP_BBBBII(Result, Source, Delimiter, First, Last) JLibEGROUP_BBBBII(dp, Result, Source, Delimiter, First, Last)

	/*	GROUPSTORE_VBBIIB(SubString, Variable, Start, Qty, Delim ) - Replace a string of variables */
#define	GROUPSTORE_VBBIIB(SubString, Variable, Start, Qty, Delim )	JLibEGROUPSTORE_VBBIIB(dp, SubString, Variable, Start, Qty, Delim )

	/*	FIELD_BBBBII(Result, Source, Delimiter, First, Last)  - Return a group of delimited string */
#define	FIELD_BBBBII(Result, Source, Delimiter, First, Last) JLibEGROUP_BBBBII(dp, Result, Source, Delimiter, First, Last)
#define	FIELDS_BBBBII(Result, Source, Delimiter, First, Last) JLibEGROUPS_BBBBII(dp, Result, Source, Delimiter, First, Last)

	/*	GETARG_IIB(ArgNo, TargetVar) - Return TCL argument */
#define	GETARG_IIB(ArgNo, TargetVar)	JLibEGETARG_IIB(dp, ArgNo, TargetVar)

	/*	GETMSG_IIB(ErrNo, TargetVar) - Return System Error message */
#define	GETMSG_IIB(ErrNo, TargetVar) JLibEGETMSG_IIB(dp, ErrNo, TargetVar)

	/*	SENTENCE_BBI(Result, Position) - Return all or part of the command line */
#define	SENTENCE_BBI(Result, Position)  JLibENEWSENTENCE_BBII(dp, Result, Position, 0x02)
#define	SENTENCE_BB(Result)  JLibENEWSENTENCE_BBII(dp, Result, -1, 0x02 )

	/*	FIND_IBBIBBB(Expr,DynArray,Count,Attr,Mv,Sv) - Find a sub-string in a dynamic array */
#define	FIND_IBBIBBB(Expr,DynArray,Count,Attr,Mv,Sv) JLibEFIND_IBBIBBB(dp, Expr,DynArray,Count,Attr,Mv,Sv)

	/*	FINDSTR_IBBIBBB(Expr,DynArray,Count,Attr,Mv,Sv) - Find a sub-string in a string */
#define	FINDSTR_IBBIBBB(Expr,DynArray,Count,Attr,Mv,Sv) JLibEFINDSTR_IBBIBBB(dp, Expr,DynArray,Count,Attr,Mv,Sv)

	/*	ASSIGNED_IB and UNASSIGNED_IB - has a variable been assigned before */
#define	ASSIGNED_IB(Source) ((Source)->VAR_TYPE != VAR_TYPE_UNASSIGNED)
#define	UNASSIGNED_IB(Source) ((Source)->VAR_TYPE == VAR_TYPE_UNASSIGNED)

	/*	SPOOLER_BBIB(Target, Function, Line) - Return details about spooler status */
#define	SPOOLER_BBIB(Target, Function, Line)	JLibESPOOLER_BBIB(dp, Target, Function, Line)

	/*DIR_BBB - Return statistics about unix file */
#define	DIR_BBB(Result, Source)		JLibEDIR_BBB(dp, Result, Source)

	/*SUM_BBB - Sum all components of a variable */
#define	SUM_BBB(Result, Source)		JLibESUM_BBB(dp, Result, Source)

	/*SUMMATION_BBB - Sum all components of a variable */
#define	SUMMATION_BBB(Result, Source)		JLibESUMMATION_BBB(dp, Result, Source)

	/*SOUNDEX_BBB - Find matching sounding words */
#define	SOUNDEX_BBB(Result, Source)		JLibESOUNDEX_BBB(dp, Result, Source)

	/*DQUOTE_BBB - Enclose variable with double quotes */
#define	DQUOTE_BBB(Target, Source)		JLibEQUOTE_BBBI(dp, Target, Source, '"')

	/*SQUOTE_BBB - Enclose variable with single quotes */
#define	SQUOTE_BBB(Target, Source)		JLibEQUOTE_BBBI(dp, Target, Source, '\'')

	/*MAXIMUM_BBB - Return maximum value in dynamic variable */
#define	MAXIMUM_BBB(Target, Source)	JLibEMinMax_BBBI(dp, Target, Source, 1)

	/*MINIMUM_BBB - Return minimum value in dynamic variable */
#define	MINIMUM_BBB(Target, Source)	JLibEMinMax_BBBI(dp, Target, Source, 0)

	/*ATFUNC_BI(FuncNo)		New function to return address of common */
#define	ATFUNC_BI(FuncNo)	JLibEATFUNC_BI(dp, FuncNo)

	/*BITCHANGE, BITCHECK, BITLOAD, BITSET, BITRESET - manipulating bit strings. */
#define	BITCHANGE_II(BitNo)	JLibEBITCHANGE_II(dp, BitNo)
#define	BITCHECK_II(BitNo)	JLibEBITCHECK_II(dp, BitNo)
#define	BITLOAD_BBB(Result, Source)		JLibEBITLOAD_BBB(dp, Result, Source)
#define	BITSET_II(BitNo)	JLibEBITSET_II(dp, BitNo)
#define	BITRESET_II(BitNo)	JLibEBITRESET_II(dp, BitNo)

	/*BITAND, BITOR, BITNOT, BITXOR - manipulating 32 bit values */
#define BITAND_BBBB(result, source1, source2 ) JLibEBITFUNC_BBBBI(dp, result, source1, source2, 1 )
#define BITOR_BBBB(result, source1, source2 ) JLibEBITFUNC_BBBBI(dp, result, source1, source2, 2 )
#define BITXOR_BBBB(result, source1, source2 ) JLibEBITFUNC_BBBBI(dp, result, source1, source2, 3 )
#define BITNOT_BBB(result, source ) JLibEBITFUNC_BBBBI(dp, result, source, NULL, 4 )

	/*ISALNUM_IB(source) -- is the string alpha-numeric */
	/*ISALPHA_IB(source) -- is the string upper or lower case alphabetic */
	/*ISCNTRL_IB(source) -- is the string control characters */
	/*ISDIGIT_IB(source) -- is the string made up of digits 0-9 */
	/*ISGRAPH_IB(source) -- is the string printable characters except space */
	/*ISLOWER_IB(source) -- is the string lower case alphabetic */
	/*ISPRINT_IB(source) -- is the string any printable character except space */
	/*ISPUNCT_IB(source) -- is the string any printable character except space or alpha-numeric */
	/*ISSPACE_IB(source) -- is the string white space such as space, tab, form-feed etc. */
	/*ISUPPER_IB(source) -- is the string upper case alphabetic */
	/*ISXDIGIT_IB(source) -- is the string hex digit 0-9, a-f or A-F */
#define	ISALNUM_IB(source) JLibEISTYPE_IBI(dp, source,1)
#define	ISALPHA_IB(source) JLibEISTYPE_IBI(dp, source,2)
#define	ISCNTRL_IB(source) JLibEISTYPE_IBI(dp, source,3)
#define	ISDIGIT_IB(source) JLibEISTYPE_IBI(dp, source,4)
#define	ISGRAPH_IB(source) JLibEISTYPE_IBI(dp, source,5)
#define	ISLOWER_IB(source) JLibEISTYPE_IBI(dp, source,6)
#define	ISPRINT_IB(source) JLibEISTYPE_IBI(dp, source,7)
#define	ISPUNCT_IB(source) JLibEISTYPE_IBI(dp, source,8)
#define	ISSPACE_IB(source) JLibEISTYPE_IBI(dp, source,9)
#define	ISUPPER_IB(source) JLibEISTYPE_IBI(dp, source,10)
#define	ISXDIGIT_IB(source) JLibEISTYPE_IBI(dp, source,11)

	/*CONTROLCHAR_BBB(Result, Source) - Convert the ctrl<A> to ctrl<Z> 's */
#define	CONTROLCHAR_BBB(Result, Source)		JLibECONTROLCHAR_BBB(dp, Result, Source)

	/*UNIQUEKEY_BB(result) - generate a unique key */
#define UNIQUEKEY_BB(result)		JLibEUNIQUEKEY_BB(dp, result)

	/*JBASECOREDUMP(result , filename flags) -- dump details to a file */
#define	JBASECOREDUMP_BBBB(result, filename, flags)	JLibADebugCoreDump_BBBB( dp, result, filename, flags)
/*
	Category (F) - General Input / output including terminal I/O, file I/O, tape I/O.
*/
	/*
	 *File IO.
	 */
#define	NEWREAD_IIBBBB(Flags,Record,Key,FileDescriptor,Setting) JLibFNEWREAD_IIBBBIB(dp, Flags,FileDescriptor,Record,Key,0,Setting)
#define	NEWREADV_IIBBBIB(Flags,Record,FileDescriptor,Key,FieldNumber,Setting) JLibFNEWREAD_IIBBBIB(dp, Flags,FileDescriptor,Record,Key,FieldNumber,Setting)
#define	NEWWRITE_IIBBBB(Flags,Record,Key,FileDescriptor,Setting) JLibFNEWWRITE_IIBBBIB(dp, Flags,FileDescriptor,Record,Key,0,Setting)
#define	NEWWRITEV_IIBBBIB(Flags,Record,FileDescriptor,Key,FieldNumber,Setting) JLibFNEWWRITE_IIBBBIB(dp, Flags,FileDescriptor,Record,Key,FieldNumber,Setting)
#define	NEWMATREAD_IIBBBBB(Flags, FirstVariable, Desc, Key, FileDescriptor, Setting)	JLibFNEWMATREAD_IIBBBBB(dp, Flags, FirstVariable, Desc, Key, FileDescriptor, Setting)
#define	NEWMATWRITE_IIBBBBB(Flags, FirstVariable, Desc, Key, FileDescriptor, Setting)	JLibFNEWMATWRITE_IIBBBBB(dp, Flags, FirstVariable, Desc, Key, FileDescriptor, Setting)
#define	NEWDELETE_IIBBB(Flags, Desc, Key, Setting)		JLibFNEWDELETE_IIBBB(dp, Flags, Desc, Key, Setting)
#define	NEWCLEARF_IIBB(Flags, Desc, Setting)			JLibFNEWCLEARFILE_IIBB(dp, Flags, Desc, Setting)
#define	NEWOPENINDEX_IIBBBB(Flags, File, Name, Desc, Setting)	JLibFNEWOPENINDEX_IIBBBB(dp, Flags, File, Name, Desc, Setting)

	/*RECORDLOCKED_IBB(FileVar, ItemId) - Return status of record lock */
#define RECORDLOCKED_IBB(FileVar, ItemId)	JLibFRECORDLOCKED_IBB(dp, FileVar, ItemId)

	/*PRINT_VIN(Report, Source) - Print without CR/LF */
#define	PRINT_VIB(Report,Source) JLibFPRINT_VIB(dp, Report,Source)

	/*PRINTN_VIB(Report, Source) - Print with trailing CR/LF */
#define	PRINTN_VIB(Report, Source) JLibFPRINTN_VIB(dp, Report, Source)

	/*PRINTCR_VI(Report) - Print CR/LF */
#define PRINTCR_VI(Report) JLibFPRINTCR_VI(dp, Report)

	/*PRINTERR_VBB(Message,File) - Print a message to status line, or message from status file. */
#define	PRINTERR_VBB(Message, File)	JLibFPRINTERR_VBB(dp, Message, File)

	/*CRT_VB(Source) - Print without CR/LF */
#define	CRT_VB(Source) { JLibFCRTBEGIN(dp) ;JLibFCRT_VB(dp, Source) ; JLibFCRTEND(dp); }

	/*CRTN_VB(Source) - Print with trailing CR/LF */
#define	CRTN_VB(Source) { JLibFCRTBEGIN(dp) ;JLibFCRTN_VB(dp, Source) ; JLibFCRTEND(dp); }

	/*CRTCR - Print CR/LF */
#define CRTCR { JLibFCRTBEGIN(dp) ;JLibFCRTCR(dp) ; JLibFCRTEND(dp); }

	/*CATTAB_BBBB(Result, Source1, Source2) - Concatenate two strings with a TAB in the middle */
#define	CATTAB_BBBB(Result,Source1,Source2) JLibFCATTAB_BBBB(dp, Result,Source1,Source2)

#define	CASING_VIF(value) JLibEmulateSwitchCase(dp, value);

	/*BREAK_VF(BreakFlag) - Turn break key on or off */
#define	BREAK_VF(BreakFlag) JLibFBREAKKEY_VF(dp, (FLOAT)BreakFlag)

	/*HEADING_VIB(Report, Statement) - The HEADING statement */
#define	HEADING_VIB(Report, Statement) JLibFHEADING_FOOTING_VIIB(dp, Report, 0,Statement)

	/*HEADINGE_VIB(Report, Statement) - The HEADINGE statement */
#define	HEADINGE_VIB(Report, Statement) JLibFHEADING_FOOTING_VIIB(dp, Report, 2,Statement)

	/*HEADINGN_VIB(Report, Statement) - The HEADINGN statement */
#define	HEADINGN_VIB(Report, Statement) JLibFHEADING_FOOTING_VIIB(dp, Report, 3,Statement)

	/*FOOTING_VIB(Report, Statement) - The FOOTING statement */
#define	FOOTING_VIB(Report, Statement) JLibFHEADING_FOOTING_VIIB(dp, Report, 1,Statement)

	/*PAGE_VII(ReportNo, PageNo) - Start a new PAGE */
#define	PAGE_VII(ReportNo, PageNo) JLibFPAGE_VII(dp, ReportNo,PageNo)

	/*ECHO_VF(EchoFlag) - Turn echo to terminal on or off */
#define	ECHO_VF(EchoFlag) JLibFECHO_VF(dp, (FLOAT)EchoFlag)

	/*ECHO_VIF(Type, EchoFlag) - Turn hush/echo to terminal on or off */
#define	ECHO_VIF(Type, EchoFlag) JLibFECHO_VIF(dp, (INT32) Type, (FLOAT)EchoFlag)

	/*INPUT_IBBBBBIII(Result,CursorInfo,FormatMask,DelimMask,PadChar,Length,Flags,Timeout) - Input from terminal */
#define	INPUT_IBBBBBIII(Result,CursorInfo,FormatMask,DelimMask,PadChar,Length,Flags,Timeout) JLibFINPUT_IBBBBBIII(dp,Result,CursorInfo,FormatMask,DelimMask,PadChar,Length,Flags,Timeout)

	/*
	**INPUT_IBBBBBIIII( Result, CursorInfo, FormatMask, DelimMask, PadChar, Length, Flags, Timeout, UnFiltered );
	**Override with new macro containing additional flag for UNFILTERED option
	*/
#define	INPUT_IBBBBBIIII(Result, CursorInfo, FormatMask, DelimMask, PadChar, Length, Flags, Timeout, UnFiltered ) JLibFINPUT_IBBBBBIII (dp, Result, CursorInfo, FormatMask, DelimMask, PadChar, Length, (Flags | UnFiltered), Timeout )

	/*INPUTERR_VB(Expression) - Display on last line after input crap */
#define	INPUTERR_VB(Expression) JLibFINPUTERR_VB(dp, Expression)

	/*INPUTNULL_VB(Char) - Character to count as a null character */
#define	INPUTNULL_VB(Char)	JLibFINPUTNULL_VB(dp, Char)

	/*INPUTCLEAR_V - Clear type ahead from input */
#define	INPUTCLEAR_V()	JLibFINPUTCLEAR_V(dp)

	/*CURSOR_BBII(Result, Column, Row) - Reposition the cursor with the @() function */
#define	CURSOR_BBII(Result, Column, Row) JLibFCURSOR_BBII(dp, Result, Column, Row)

	/*OUT_VI(OutputInteger) - The OUT statement */
#define	OUT_VI(OutputInteger) JLibFOUT_VI(dp, OutputInteger)

	/*IN_IBI(InputVar,TimeOut) - Input a single character as the SEQ() value of it */
#define	IN_IBI(InputVar,TimeOut) JLibFIN_IBI(dp, InputVar,TimeOut)

	/*
	**IN_IBII(InputVar,TimeOut,Unfiltered) - Input a single character as the SEQ() value of it 
	**New macro to cater for UNFILTERED option on INPUT/IN in this case ignore
	*/
#define	IN_IBII(InputVar,TimeOut, Unfiltered) JLibFIN_IBII(dp, InputVar, TimeOut, Unfiltered)

	/*KEYIN_BB(InputVar) - Input a single character as the value of it */
#define	KEYIN_BB(InputVar) JLibFKEYIN_BB(dp, InputVar)

	/*PAUSE_VI(TimeOut) or PAUSE_VB(TimeOut) - Suspend processs */
#define	PAUSE_VI(TimeOut) JLibFPAUSE_VI(dp, TimeOut)
#define	PAUSE_VB(TimeOut) JLibFPAUSE_VB(dp, TimeOut)

	/*WAKE_VI(Port) - Wake process */
#define	WAKE_VI(Port)	JLibFWAKE_VI(dp, Port)

	/*CLEARSELECT_VB(SelectVar) - Clear select list */
#define	CLEARSELECT_VB(SelectVar) JLibFCLEARSELECT_VB(dp, SelectVar)

	/*CLEARSELECTALL() - Clear ALL select lists */
#define	CLEARSELECT_ALL() JLibFCLEARSELECTALL(dp)

	/*PRINTER_VII(Report,Flags) - PRINTER ON | OFF | CLOSE (Flags == 1, 0, -1 respectively) */
#define	PRINTER_VII(Report,Flags) JLibFPRINTER_VII(dp, Report, Flags)

	/*PROMPT_VB(PromptString) - Use a new string with terminal input */
#define	PROMPT_VB(PromptString) JLibFPROMPT_VB(dp, PromptString)

	/*SELECTINDEX_VBBBBB(IndexName, Start, Desc, SelectVar, Setting) - Select Index to select list */
#define SELECTINDEX_VBBBBB(IndexName, Start, Desc, SelectVar, Setting) JLibFSELECTINDEX_VBBBBB(dp, IndexName, Start, Desc, SelectVar, Setting)

	/*OPEN_IBBBB(Dict,FileName,Dscb,Setting) - Open a file */
#define	OPEN_IBBBB(Dict,FileName,Dscb,Setting) JLibFOPEN_IBBBB(dp, Dict,FileName,Dscb,Setting)

	/*CLOSE_VB(Dscb) - Close a file; set Dscb to unassigned */
#define	CLOSE_VB(Dscb) JLibFCLOSE_VB(dp, Dscb)

	/*CLOSE_IB(Dscb) - Close a file; set Dscb to unassigned */
#define	CLOSE_IB(Dscb) JLibFCLOSE_IB(dp, Dscb)

	/*SELECT_IBBB(From,To, Setting) - Select from a file to a variable */
#define	SELECT_IBBB(From,To, Setting) JLibFSELECT_IBBB(dp, From, To, Setting)

	/*SSELECT_IBBB(From,To, Setting) - Sort Select from a file to a variable */
#define	SSELECT_IBBB(From,To, Setting) JLibFSSELECT_IBBB(dp, From, To, Setting)
#define	SSELECTN_IBBB(From,To, Setting) JLibFSSELECT_IBBB(dp, From, To, Setting)		/*As per SSELECT for now */
#define	SSELECTV_IBBB(From,To, Setting) JLibFSSELECT_IBBB(dp, From, To, Setting)		/*As per SSELECT for now */

	/*SELECTATKEY_IBBBBBII(From,To, Setting, Key, ItemId, Vmc, Svmc) - Select from a file to a variable with Index Key */
#define	SELECTATKEY_IBBBBBII(From,To, Setting, Key, ItemId, Vmc, Svmc)	JLibFSELECTATKEY_IBBBBBII(dp, From, To, Setting, Key, ItemId, Vmc, Svmc)

	/*SELECTE_VBB(To, Setting) - Select from default list to a variable */
#define	SELECTE_VBB(To, Setting) JLibFSELECTE_VBB(dp, To, Setting)

	/*SELECTINFO_IBI(SelectList, Code) - Returns status of select list */
#define	SELECTINFO_IBI(SelectList, Code) JLibFSELECTINFO_IBI(dp, SelectList, Code)

	/*READNEXT_IBBBBB(Var, VmVar, SvmVar, From, Setting) Read next record key from select list or index file */
#define READNEXT_IBBBBB(Var, VmVar, SvmVar, From, Setting) JLibFREADNEXT_IBBBBB(dp, Var, VmVar, SvmVar, From, Setting)
#define READNEXT_IIBBBBB(Flags, ToVar, MultiValue, SubValue, List, Setting ) JLibFREADNEXT_IBBBBB(dp, ToVar, MultiValue, SubValue, List, Setting )

	/*READNEXTKEY_IBBBBB(Var, ItemVar, VmVar, From, Setting) Read next index key from index file */
#define READNEXTKEY_IBBBBB(Var,ItemVar,VmVar,From,Setting) JLibFREADINDEX_IIBBBBB(dp, 1, Var, ItemVar, VmVar, From, Setting)
#define READNEXTKEY_IIBBBBB(Flags, Var,ItemVar,VmVar,From,Setting) JLibFREADINDEX_IIBBBBB(dp, 1, Var, ItemVar, VmVar, From, Setting)

	/*READPREV_IBBBBB(Var, VmVar, SvmVar, From, Setting) Read previous record key from select list */
#define READPREV_IBBBBB(Var, VmVar, SvmVar, From, Setting ) JLibFREADINDEX_IIBBBBB(dp, -1, NULL, Var, VmVar, From, Setting )
#define READPREV_IIBBBBB(Flags, Var, VmVar, SvmVar, From, Setting ) JLibFREADINDEX_IIBBBBB(dp, -1, NULL, Var, VmVar, From, Setting )

	/*READPREVKEY_IBBBBB( KeyVar, IdVar, VmVar, From, Setting) Read previous index id from select list or index file */
#define READPREVKEY_IBBBBB(KeyVar, IdVar, VmVar, From, Setting ) JLibFREADINDEX_IIBBBBB(dp, -1, KeyVar, IdVar, VmVar, From, Setting )
#define READPREVKEY_IIBBBBB(Flags, KeyVar, IdVar, VmVar, From, Setting ) JLibFREADINDEX_IIBBBBB(dp, -1, KeyVar, IdVar, VmVar, From, Setting )

	/*RELEASE_VBB(File,Key) - Release a record lock */
#define	RELEASE_VBB(File, Key)		JLibFRELEASE_VBB(dp, File, Key)

	/*RELEASE_IBB(File,Key) - Release a record lock */
#define	RELEASE_IBB(File, Key)		JLibFRELEASE_IBB(dp, File, Key)

	/* RELEASEALL_I() - Release all locks on all opened files */
#define	RELEASEALL_I()		JLibFRELEASEALL_I(dp)

	/*	WRITET_IBII(Buffer, Channel, Flags) - Write a buffer to tape */
#define	WRITET_IBII(Buffer, Channel, Flags) JLibFWRITET_IBII(dp, Buffer, Channel, Flags)

	/*	READT_IBII(Buffer, Channel, Flags) - Read a buffer from tape */
#define READT_IBII(Buffer, Channel, Flags)	JLibFREADT_IBII(dp, Buffer, Channel, Flags)

	/*	REWIND_II(Channel) - Rewind the tape device */
#define	REWIND_II(Channel)	JLibFREWIND_II(dp, Channel)

	/*	WEOF_II(Channel) - Rewind the tape device */
#define	WEOF_II(Channel)	JLibFWEOF_II(dp, Channel)

	/*IOCTL_IBBB(File, Command, Source) - Miscellaneous file control */
#define	IOCTL_IBBB(File, Command, Source)	JLibFIOCTL_IBBB(dp, File, Command, Source)

	/*	WRITELIST, READLIST, GETLIST and DELETELIST - LIST handling */
#define	WRITELIST_IBB(Source, List)	JLibFWRITELIST_IBB(dp, Source, List)
#define WRITELIST_IBBB(Source, List, Setting)	JLibFWRITELIST_IBBB(dp, Source, List, Setting)
#define	READLIST_IBBB(Result, List, Setting)	JLibFREADLIST_IBBB(dp, Result, List, Setting)
#define	GETLIST_IBBB(Result, List, Setting)	JLibFGETLIST_IBBB(dp, Result, List, Setting)
#define	DELETELIST_VB(List)	JLibFDELETELIST_VB(dp, List)
#define	FORMLIST_VBBB(From, To, Setting)	JLibFFORMLIST_VBBB(dp, From, To, Setting)

	/*TRANSACTION oriented functions */
#define	TRANSTART_IIB(Flags,Var) JLibFTRANSACTION_IIB(dp, ((Flags<<16)|1), Var)
#define	TRANSEND_IB(Var) JLibFTRANSACTION_IIB(dp, 2, Var)
#define	TRANSABORT_IB(Var) JLibFTRANSACTION_IIB(dp, 3, Var)
#define	TRANSQUERY_I() JLibFTRANSACTION_IIB(dp, 4, NULL)
#define	TRANSSYNC_IB(Var) (!JLibFTRANSACTION_IIB(dp, 5, Var))

	/*TA_II(port) - Type Ahead count from a line */
#define	TA_II(port)	JLibFTA_II(dp, port)

	/*GET_IBIBBBBB(Retvar,Len,Setting,From,Until,Retn,Wait) - Read characters from a line */
#define	GET_IBIBBBBB(Retvar,Len,Setting,From,Until,Retn,Wait ) JLibFGET_IIBIBBBBB(dp, 0x01,Retvar,Len,Setting,From,Until,Retn,Wait)

	/*GETX_IBIBBBBB(Retvar,Len,Setting,From,Until,Retn,Wait) - Read characters from a line with translation */
#define	GETX_IBIBBBBB(Retvar,Len,Setting,From,Until,Retn,Wait ) JLibFGET_IIBIBBBBB(dp, 0x02,Retvar,Len,Setting,From,Until,Retn,Wait)

	/*SEND_IIBB(flags, passdata, from) - write characters to a line with or without translation */
#define	SEND_IIBB(flags, passdata, from)	JLibFSEND_IIBB(dp, flags, passdata, from)

	/*TTYCTL_IBBB(FileVar, Code, Setting) - Set terminal device characteristics */
#define	TTYCTL_IBBB(FileVar, Code, Setting)	JLibFTTYCTL_IBBB(dp, FileVar, Code, Setting)

	/*TTYGET_IIBBB(Type, FileVar, Variable, Setting) - Get device characteristics */
#define	TTYGET_IIBBB(Type, FileVar, Variable, Setting)	JLibFTTYGET_IIBBB(dp, Type, FileVar, Variable, Setting)

	/*TTYSET_IIBBB(Type, FileVar, Variable, Setting) - Set device characteristics */
#define	TTYSET_IIBBB(Type, FileVar, Variable, Setting)	JLibFTTYSET_IIBBB(dp, Type, FileVar, Variable, Setting)

	/*TIMEOUT_VBB(FileVar, TimeOut) - Set device timeout period */
#define	TIMEOUT_VBB(FileVar, TimeOut)		JLibFTIMEOUT_VBB(dp, FileVar, TimeOut)

/*
	Category (G) - String conversion, time and date.
*/

	/*ICONV_BBBBB(Target, ConvReg, Source, Conversion) - Input conversion */
#define	ICONV_BBBBB(Target, ConvReg, Source, Conversion) JLibGConvRun2(dp, ConvReg, Target, Source, Conversion, 0, 0)

	/*OCONV_BBBBB(Target, ConvReg, Source, Conversion) - Output conversion */
#define	OCONV_BBBBB(Target, ConvReg, Source, Conversion) JLibGConvRun2(dp, ConvReg, Target, Source, Conversion, 1, 0)

	/*Clean up local variable after conversion (ICONV, OCONV, FMT etc)	*/
#define	CONV_CLEAN(Convreg)	JLibGConvCCBFree(dp, Convreg);

	/*ASCII_BBB(Target, Source) - Convert from EBCDIC to ASCII */
#define	ASCII_BBB(Target, Source) JLibGASCII_BBB(dp, Target, Source)

	/*EBCDIC_BBB(Target, Source) - Convert from ASCII to EBCDIC */
#define	EBCDIC_BBB(Target, Source) JLibGEBCDIC_BBB(dp, Target, Source)

	/*UPCASE_BBB(Target, Source) - Convert to uppercase */
#define	UPCASE_BBB(Target, Source) JLibGUPCASE_BBB(dp, Target, Source)

	/*DOWNCASE_BBB(Target, Source) - Convert to lower case */
#define	DOWNCASE_BBB(Target, Source) JLibGDOWNCASE_BBB(dp, Target, Source)

	/*DTX_BBI(Source, universe_mode) - Convert from Decimal to Hex */
#define	DTX_BBBB(Result, Source, universe_mode) JLibGDTX_BBBB(dp, Result, Source, universe_mode)

	/*XTD_IB(Source, universe_mode) - Convert from Hex to Decimal */
#define	XTD_BBBB(Result, Source, universe_mode) JLibGXTD_BBBB(dp, Result, Source, universe_mode)

	/*SEQ_IB(Source) - Return the decimal value of 1st character in string */
#define	SEQ_IB(Source) JLibGSEQ_IB(dp, Source)

	/*FMT_BBBBB(Result,ConvReg, Source,FormatSpec) - Perform format conversions on a string */
#define	FMT_BBBBB(Result, ConvReg, Source, FormatSpec)	JLibGFORMAT_BBBBB(dp, Result, ConvReg, Source, FormatSpec)

	/*TIME_I - Return current time */
#define	TIME_I() JLibGInternalLocalTime(dp)

	/*DATE_I - Return current date */
#define	DATE_I() JLibGInternalLocalDate(dp)

	/*TIMEDATE_BB(Result) - Get a string with current time and date */
#define	TIMEDATE_BB(Result) JLibGTIMEDATE_BB(dp, Result)

	/*TIMESTAMP_BB(Result) - Get a string with current UTC Time and date at high precision */
#define	TIMESTAMP_BB(Result) JLibGTIMESTAMP_BB(dp, Result)

	/*TIMESTAMP_F()) - Get a float with current UTC Time and date at high precision */
#define	TIMESTAMP_F() JLibGTIMESTAMP(dp, CURRENT_CALL_STACK(current_precision))

	/*LOCALTIME_BBBB(Result, TimestampVar, TimeZone) - Convert UTC time to internal local time */
#define	LOCALTIME_BBBB(Result, TimestampVar, TimeZone) JLibGLOCALTIME_BBBB(dp, Result, TimestampVar, TimeZone)

	/*LOCALDATE_BBBB(Result, TimestampVar, TimeZone) - Convert UTC time to internal local date */
#define	LOCALDATE_BBBB(Result, TimestampVar, TimeZone) JLibGLOCALDATE_BBBB(dp, Result, TimestampVar, TimeZone)

	/*MAKETIMESTAMP_BBIIB(Result, intDate, intTime, TimeZone) - Convert local internal time & date to UTC time */
#define	MAKETIMESTAMP_BBIIB(Result, intDate, intTime, TimeZone) JLibGMAKETIMESTAMP_BBIIB(dp, Result, intDate, intTime, TimeZone)

	/*TIMEDIFF_BBBBI(Result, Timestamp1, Timestamp2, Mask) - compare 2 timestamps and return array of differences */
#define	TIMEDIFF_BBBBI(Result, Timestamp1, Timestamp2, Mask) JLibGTIMEDIFF_BBBBI(dp, Result, Timestamp1, Timestamp2, Mask)

	/*CHANGETIMESTAMP_BBBB(Result, Timestamp, ChangeArray) - Add an array of time interval to a timestamp */
#define	CHANGETIMESTAMP_BBBB(Result, Timestamp, ChangeArray) JLibGCHANGETIMESTAMP_BBBB(dp, Result, Timestamp, ChangeArray)

	/*CONVERTF_BBBBB(Result, Source, From, To) - Convert one source to another */
#define	CONVERTF_BBBBB(Result, Source, From, To)	JLibGCONVERTF_BBBBB(dp, Result, Source, From, To)

	/*CONVERTTO_VBIIIBB(SourceAndResult,Attr,Mult,Sub,From,To) - Convert one source to another */
#define	CONVERTTO_VBIIIBB(SourceAndResult,Attr,Mult,Sub,From,To)	JLibGCONVERTTO_VBIIIBB(dp, SourceAndResult,Attr,Mult,Sub,From,To)

	/*CHANGEF_BBBBB(Result, Source, From, To) - Change one string to another */
#define	CHANGEF_BBBBB(Result, Source, From, To)		JLibGCHANGEF_BBBBB(dp, Result, Source, From, To)

	/*CHANGETO_VBIIIBB(SourceAndResult, Attr, Mult, Sub, From, To) - Convert one source to another */
#define	CHANGETO_VBIIIBB(SourceAndResult, Attr, Mult, Sub, From, To)	JLibGCHANGETO_VBIIIBB(dp, SourceAndResult, Attr, Mult, Sub, From, To)

	/*CONVERTFU_BBBBB(Result, From, To, Source) - PRIME/UNIVERSE CONVERT order diff so swap */
#define	CONVERTFU_BBBBB(Result, From, To, Source)	JLibGCONVERTF_BBBBB(dp, Result, Source, From, To)

	/*SWAPF_BBBBBII(Result,Source,From,To,Occurence,Start) - Swap one source to another */
#define	SWAPF_BBBBBII(Result,Source,From,To,Occurence,Start)	JLibGSWAPF_BBBBBII(dp, Result, Source, From, To, Occurence, Start)

/*NEWOPEN_IIBBBB  - Open file */
#define NEWOPEN_IIBBBB(Flags, Dict, File, Desc, Setting)		JLibFNEWOPEN_IIBBBB(dp, Flags, Dict, File, Desc, Setting)

/*NEWOPENSEQ_IIBBIBB  - Open sequential file */
#define	NEWOPENSEQ_IIBBIBB(Flags, Path, File, Action, Desc, Setting)	JLibFNEWOPENSEQ_IIBBIBB(dp, Flags, Path, File, Action, Desc, Setting)

/*NEWOPENSER_IIBBBB  - Open serial file */
#define	NEWOPENSER_IIBBBB(File, Path, Optinos, Desc, Setting)	JLibFNEWOPENSER_IIBBBB(dp, File, Path, Optinos, Desc, Setting)

/*NEWDELETESEQ_IIBBBB  - Delete sequential file */
#define	NEWDELETESEQ_IIBBB(Flags, Path, File, Setting)	JLibFNEWDELETESEQ_IIBBB(dp, Flags, Path, File, Setting)

/*NEWREADSEQ_IIBBB		- Read sequential file */
#define NEWREADSEQ_IIBBB(Flags, Rec, Desc, Setting)	JLibFNEWREADSEQ_IIBBB(dp, Flags, Rec, Desc, Setting)

/*NEWWRITESEQ_IIBIBB	- Write sequential file */
#define NEWWRITESEQ_IIBIBB(Flags, Rec, Action, Desc, Setting)	JLibFNEWWRITESEQ_IIBIBB(dp, Flags, Rec, Action, Desc, Setting)

/*NEWWEOFSEQ_IIBB		- Write end of sequential file */
#define NEWWEOFSEQ_IIBB(Flags, Desc, Setting)		JLibFNEWWEOFSEQ_IIBB(dp, Flags, Desc, Setting)

/*NEWOSDELETE_IIBB  - Delete native file */
#define	NEWOSDELETE_IIBB(Flags, Path, Setting)	JLibFNEWOSDELETE_IIBB(dp, Flags, Path, Setting)

/*NEWOSREAD_IIBBB		- Read native file */
#define NEWOSREAD_IIBBB(Flags, Record, Path, Setting)	JLibFNEWOSREAD_IIBBB(dp, Flags, Record, Path, Setting)

/*NEWOSWRITE_IIBIBB	- Write native file */
#define NEWOSWRITE_IIBIBB(Flags, Record, Action, Path, Setting)	JLibFNEWOSWRITE_IIBIBB(dp, Flags, Record, Action, Path, Setting)

/*NEWFILELOCK_IIBBB	- Lock whole file */
#define NEWFILELOCK_IIBBB(Flags, File, Lock, Setting)	JLibFNEWFILELOCK_IIBBB(dp,Flags, File, Lock, Setting)

/*CLOSESEQ_VB 		- Close sequential file */
#define CLOSESEQ_IB(File)			JLibFCLOSESEQ_IB(dp, File)

/*CREATESEQ_IBB      - Create sequential file */
#define CREATESEQ_IBB(Desc, Setting)		JLibFCREATESEQ_IBB(dp, Desc, Setting)

/*READBLK_IBBIB	- Read sequential in blocks file */
#define READBLK_IBBIB(Rec, Desc, Size, Setting)		JLibFREADBLK_IBBIB(dp, Rec, Desc, Size, Setting)

/*WRITEBLK_IBBB	- Write sequential in blocks file */
#define WRITEBLK_IBBB(Rec, Desc, Setting)		JLibFWRITEBLK_IBBB(dp, Rec, Desc, Setting)

/*NOBUF_IBB		- Force buffer flushing sequential file */
#define NOBUF_IBB(Desc, Setting)			JLibFNOBUF_IBB(dp, Desc, Setting)

/*SEEK_IBIIB		- Seek to offset from relative position sequential file */
#define SEEK_IBIIB(Desc, Offeset, Pos, Setting)			JLibFSEEK_IBIIB(dp, Desc, Offeset, Pos, Setting)

/*OSBREAD_IBBIIB	- Read blocks from sequential file */
#define OSBREAD_IBBIIB(Rec, Desc, Offset, Length, Setting)		JLibFOSBREAD_IBBIIB(dp, Rec, Desc, Offset, Length, Setting)

/*OSBWRITE_IBBIB	- Write blocks on sequential file */
#define OSBWRITE_IBBIB(Rec, Desc, Offset, Setting)		JLibFOSBWRITE_IBBIB(dp, Rec, Desc, Offset, Setting)

/*FILEUNLOCK_IB	- Unlock lock set by FILELOCK */
#define FILEUNLOCK_IB(Desc)		JLibFFILEUNLOCK_IB(dp, Desc)

/*JMODIFYSTTY		- Modify permanently the stty characteristics */
#define	JMODIFYSTTY(NewString)		JLibFModifySttyIB(dp, NewString)

/*STATUS_IBB 		- Get status of open file */
#define STATUS_IBBB(Rec, Desc, Setting)		JLibFSTATUS_IBBB(dp, Rec, Desc, Setting)

/*FILEINFO_BBBB	- Get info from open file */
#define FILEINFO_BBBB(Result, Desc, Code)	JLibFFILEINFO_BBBB(dp, Result, Desc, Code)

/*SET_PRECISION_VI	- Change precision */
#define SET_PRECISION_VI(NewPrec) JLibCSetPrecision_VI(dp, NewPrec)

	/*TRANS_BBBBIB(result, file, key, attr, control_code) - TRANS or XLATE function */
#define	TRANS_BBBBBB(result, file, key, attr, control_code)	JLibGTRANS_BBBBBB(dp, result, file, key, attr, control_code)
/*
 * Dynamic object interfaces
 */
VAR * jObjectCreate(VAR * rtn , struct jBASEDataAreas * dp , int varqty , STRING * classname , ...) ;
INT32	jObjectDestroy(struct jBASEDataAreas * dp , VAR * object) ;
VAR * jObjectGet(VAR * rtn , struct jBASEDataAreas * dp , int numarg , VAR * object , VAR * propertyname,...) ;
INT32	jObjectSet(struct jBASEDataAreas * dp , int numarg, VAR * object , VAR * v1, VAR * v2,...) ;
VAR *	jObjectSet1(VAR * result , struct jBASEDataAreas * dp , VAR * object , VAR * property, VAR * source);
VAR * jObjectGet1(VAR * rtn , struct jBASEDataAreas * dp , VAR * source , VAR * propertyname) ;
VAR	* jObjectMethod(VAR * rtn , struct jBASEDataAreas * dp , int numarg , VAR * object , STRING * methodname,...) ;
VAR	* jObjectToJSON(VAR * rtn , struct jBASEDataAreas * dp , int numarg, VAR * object , ...);
VAR	* jObjectFromJSON(VAR * rtn , struct jBASEDataAreas * dp , int numarg, VAR * source , ...) ;


// jQL<->jBC Interface macros
//
#define	JQL_COMPILE_IBBBB(stmt, cmd, opt, msgs)		jQLCompile(dp, stmt, cmd, opt, msgs)
#define	JQL_GETPROP_IBBBB(propval, stmt, col, prop)	jQLGetProperty(dp, propval, stmt, col, prop)
#define	JQL_PUTPROP_IBBBB(propval, stmt, col, prop)	jQLPutProperty(dp, propval, stmt, col, prop)
#define	JQL_EXECUTE_IBB(stmt, selvar)				jQLExecute(dp, stmt, selvar)
#define JQL_FETCH_IBB(stmt, contv, datav)			jQLFetch(dp, stmt, contv, datav)
#define JQL_CLOSE_STATMENT_IB(stmt)					jQLCloseStatment(dp, stmt)

// XML macros
#define	XML_ADD_TEXT		1
#define	XML_ADD_NODE		2
#define	XML_VALIDATE		3
#define	XML_GET_TAG_VALUE	4
#define	XML_REMOVE_TAG_VALUE	5
#define XMLFUNC_BIBB( result, request, arg1, arg2 )	JLibXMLFUNC_BIBB(dp, result, request, arg1, arg2 )

// XSL translate macros
#define	DYNTOXML_BBBB(result, array, xsldoc, rc)	JLibXSLTRANSLATE_BBBBI(dp, result, array, xsldoc, rc, 1)
#define	XMLTODYN_BBBB(result, xmldoc, xsldoc, rc)	JLibXSLTRANSLATE_BBBBI(dp, result, xmldoc, xsldoc, rc, 2)
#define	XMLTOXML_BBBB(result, xmldoc, xsldoc, rc)	JLibXSLTRANSLATE_BBBBI(dp, result, xmldoc, xsldoc, rc, 3)

// Licensing macros
#define VALIDATE_PRODUCT_LICENSE_BBB( result, product )		JLibLicense_BBBBI( dp, result, product, NULL, 1 )
#define VALIDATE_METRICS_LICENSE_BBB( result, product )		JLibLicense_BBBBI( dp, result, product, NULL, 2 )
#define CONSUME_METRICS_LICENSE_BBBB( result, product, amount )	JLibLicense_BBBBI( dp, result, product, amount, 3 )


/*
		*************************

			I M P O R T A N T

		*************************

	The following defines are present in the two files

		jmakecode.h, jsystem.h.

	It is vital that if they are changed, then both files reflect this.
	the reason that the compile does not just include jsystem.h is that
	there are several typedef/defines that clash.
*/
/*INPUT statements	*/

#define		IN_CURSOR_POS	0x00000001		// @(cc,ll) Present
#define		IN_LENGTH_LIM	0x00000002		// Character limiting present
#define		IN_COLON		0x00000004		// Don't print newline after input
#define		IN_UNDERSCORE	0x00000008		// Underscore (don't echo chars)
#define		IN_MASK			0x00000010		// Input Mask Present
#define		IN_NOECHO		0x00000020		// Never do any echo or this input
#define		IN_RAW			0x00000040		// Raw input - no translation of characters
#define		IN_NOPROMPT		0x00000080		// Ignore any prompting characters
#define		IN_TIMER		0x00000100		// Where in some sort of timer

/*	Optional LOCK or THEN/ELSE statements */

#define		CLAUSE_LOCK_NOWAIT		0x00000100		// If LOCK already taken, exit immediately
#define		CLAUSE_THEN_ELSE		0x00000200		// THEN/ELSE when optional seen
#define		CLAUSE_LOCK				0x00000400		// There is some sort of LOCK clause to take
#define		CLAUSE_FIELD_READ		0x00000800		// Read in a single field from a record
#define		CLAUSE_HEX				0x00010000		// Convert to/from hexadecimal
#define		CLAUSE_LOCK_ONLY		0x00020000		// Only apply lock no readl
#define		CLAUSE_ON_ERROR			0x00040000		// Indicate ON ERROR Clause present

/*
**New function flag assignments
*/

#define		FILE_CLAUSE_NO_ERRMSG	0x00100000		// Do not display an error message on error
#define		FILE_CLAUSE_LOCKED		0x00200000		// Statement has a LOCKED clause
#define		FILE_CLAUSE_ON_ERROR	0x00400000		// Statement has a ON ERROR clause
#define		FILE_CLAUSE_NO_RECORD	0x00800000		// Statement uses NULL for record
#define		FILE_CLAUSE_HEX			0x01000000		// Statement requires hexadecimal
#define		FILE_CLAUSE_FIELD		0x02000000		// Statement requires field value
#define		FILE_CLAUSE_UPDATE_LOCK	0x04000000		// Statement requires update lock
#define		FILE_CLAUSE_READ_LOCK	0x08000000		// Statement requires read lock
#define		FILE_CLAUSE_XML		0x10000000		// Statement requires conversion to/from XML

/*Variable assignment statements */

#define		ASSIGN_MATRIX		0x01		// Assign to Array dimension
#define		ASSIGN_ATTR			0x02		// Assign to Attribute
#define		ASSIGN_VALU			0x06		// Assign to Value
#define		ASSIGN_SUBV			0x0E		// Assign to Sub Value
#define		ASSIGN_SUBS			0x10		// Assign to sub string
#define		ASSIGN_MAT_TO_MAT	0x20		// MAT var = MAT var
#define		ASSIGN_RES_TO_MAT	0x40		// MAT var = Resultant
#define		ASSIGN_FORMAT		0x80		// Assign fmt type e.g. ['.',1,2]
#define		ASSIGN_LENGTH		0x0100		// Assign length type e.g. [4]

#define		JBC_SEQ_READONLY	0x00000001	// Optional Flag for OPENSEQ	
#define		JBC_SEQ_NOLOCK		0x00000002	// Optional Flag for OPENSEQ
#define		JBC_SEQ_APPEND		0x00000004	// Optional Flag OPENSEQ/WRITESEQ
#define		JBC_SEQ_FLUSH		0X00000008	// Optional Flag for WRITESEQ
#define		JBC_SEQ_EXISTING	0X00000010	// Optional Flag for OPENSEQ

/*Define jNULL                                     */
/*This is for OS400 which has problems comparing   */
/*an integer variable to NULL.  OS400 defines NULL */
/*as (void *)0 and not just 0                      */
#if defined __OS400__
#define jNULL 0
#else
#define jNULL NULL
#endif

#ifdef	__cplusplus
}
#endif
/*
 *Define the static in-line functions for performance improvements.
 */
#ifdef WIN32
#define	INIT_DP																			\
	lastThreadId	= GetCurrentThreadId();													\
	dp				= jbase_getdp();
#define	CHECK_DP																			\
	DWORD 	thisThreadId;																\
	if	(lastThreadId != (thisThreadId = GetCurrentThreadId()))									\
	{																						\
		lastThreadId	= thisThreadId;														\
		dp				= jbase_getdp();													\
	}

#else
// The following macro is used by the default constructors in Cvar etc
// to set up the dp and thread id for the first time.
//
#define	INIT_DP																			\
	lastThreadId	= jthread_self();													\
	dp				= jbase_getdp();

//
// The work that was previously done in CHECK_DP is now done in jbase_getdp
//
#define	CHECK_DP 	dp=jbase_getdp();																			\

#endif	
#endif /* JSYSTEM_H */
