/****************************************************************
*
* This source code is used in a 'second pass' effect for a jBC program.
* It's inclusion is optional, but if included it will look for any
* optimisations that can take be made based on earlier definitions.
*
* Copyright (c) 2019 jBASE International. All rights reserved
*
*******************************************************************/

#ifndef JSYSTEM2_H
#define JSYSTEM2_H
#if	defined(JBASE_PRECISION_CONSTANT)
#undef	CONV_IF
#undef	CONV_FI
#define	CONV_IF(Float) ((INT32)((Float)/(FLOAT)JBASE_DEFAULT_PRECMULT))
#define	CONV_FI(Integer) ((FLOAT)(((FLOAT)(Integer))*(FLOAT)JBASE_DEFAULT_PRECMULT))
#endif

#if	defined(JBUILD_OPTION_INLINE_FUNCTIONS)

#undef	STORE_VARINT
#define	STORE_VARINT(vptr, value) I_STORE_VARINT(dp, vptr, value)

static	__inline__	void I_STORE_VARINT(struct jBASEDataAreas * dp, VAR * vptr , VARINT value )
{
	if (vptr->VAR_TYPE != VAR_TYPE_INTEGER)
	{
		VAR_RELEASE2_VB(vptr) ;
		vptr->VAR_TYPE = VAR_TYPE_INTEGER ;
	}
	vptr->VAR_INTEGER = value;
}

#undef	STORE_VBI
#define	STORE_VBI(vptr, value) (I_STORE_VARINT(dp, vptr, (VARINT)(value)))

#undef	CONV_VARINT
#define	CONV_VARINT(vptr) I_CONV_VARINT(dp, vptr)

static	__inline__ VARINT I_CONV_VARINT(struct jBASEDataAreas * dp, VAR * vptr)
{
	return (vptr->VAR_TYPE & VAR_TYPE_INTEGER ? 
							vptr->VAR_INTEGER : JLibBCONV_VARINT(dp, vptr));
}

#undef	CONV_IB
#define	CONV_IB(vptr) ((INT32)(I_CONV_VARINT(dp, vptr)))


	/* CONVCOND_IB(Source) - Return 1 or 0 for conditional evaluations. */
#undef	CONVCOND_VARINT
#define	CONVCOND_VARINT(vptr) I_CONVCOND_VARINT(dp, vptr)

static __inline__ VARINT I_CONVCOND_VARINT(struct jBASEDataAreas * dp, 
																	VAR * vptr)
{
	return (vptr->VAR_TYPE & 
		(VAR_TYPE_FLOAT|VAR_TYPE_FLOAT_CONSTANT|
		VAR_TYPE_FLOAT_CONSTANT_STRING) ? vptr->VAR_FLOAT != (FLOAT)0.0 : 
		(vptr->VAR_TYPE & VAR_TYPE_INTEGER ? vptr->VAR_INTEGER : 
		JLibBCONVCOND_VARINT(dp, vptr)));
}

#undef	CONVCOND_IB
#define	CONVCOND_IB(vptr) ((INT32)(I_CONVCOND_VARINT(dp, vptr)))

#undef	EQ_IBB
#define	EQ_IBB(v1, v2) I_EQ_IBB(dp, v1, v2)

static __inline__ INT32 I_EQ_IBB(struct jBASEDataAreas * dp, VAR * v1 , VAR * v2 )
{
	return (((v1->VAR_TYPE & VAR_TYPE_FLOAT) && (v2->VAR_TYPE & VAR_TYPE_FLOAT) && (v1->VAR_FLOAT_SCALE == v2->VAR_FLOAT_SCALE)) ? (v1->VAR_FLOAT == v2->VAR_FLOAT) : JLibDCOMPARE_IBBI(dp, v1,v2,0)) ;
}

#undef	NE_IBB
#define	NE_IBB(v1, v2) I_NE_IBB(dp, v1, v2)

static __inline__ INT32 I_NE_IBB(struct jBASEDataAreas * dp, VAR * v1 , VAR * v2 )
{
	return (((v1->VAR_TYPE & VAR_TYPE_FLOAT) && (v2->VAR_TYPE & VAR_TYPE_FLOAT) && (v1->VAR_FLOAT_SCALE == v2->VAR_FLOAT_SCALE)) ? (v1->VAR_FLOAT != v2->VAR_FLOAT) : JLibDCOMPARE_IBBI(dp, v1,v2,1)) ;
}

#undef	LT_IBB
#define	LT_IBB(v1, v2) I_LT_IBB(dp, v1, v2)

static __inline__ INT32 I_LT_IBB(struct jBASEDataAreas * dp, VAR * v1 , VAR * v2 )
{
	return (((v1->VAR_TYPE & VAR_TYPE_FLOAT) && (v2->VAR_TYPE & VAR_TYPE_FLOAT) && (v1->VAR_FLOAT_SCALE == v2->VAR_FLOAT_SCALE)) ? (v1->VAR_FLOAT < v2->VAR_FLOAT) : JLibDCOMPARE_IBBI(dp, v1,v2,2)) ;
}

#undef	LE_IBB
#define	LE_IBB(v1, v2) I_LE_IBB(dp, v1, v2)

static __inline__ INT32 I_LE_IBB(struct jBASEDataAreas * dp, VAR * v1 , VAR * v2 )
{
	return (((v1->VAR_TYPE & VAR_TYPE_FLOAT) && (v2->VAR_TYPE & VAR_TYPE_FLOAT) && (v1->VAR_FLOAT_SCALE == v2->VAR_FLOAT_SCALE)) ? (v1->VAR_FLOAT <= v2->VAR_FLOAT) : JLibDCOMPARE_IBBI(dp, v1,v2,3)) ;
}

#undef	GT_IBB
#define	GT_IBB(v1, v2) I_GT_IBB(dp, v1, v2)

static __inline__ INT32 I_GT_IBB(struct jBASEDataAreas * dp, VAR * v1 , VAR * v2 )
{
	return (((v1->VAR_TYPE & VAR_TYPE_FLOAT) && (v2->VAR_TYPE & VAR_TYPE_FLOAT) && (v1->VAR_FLOAT_SCALE == v2->VAR_FLOAT_SCALE)) ? (v1->VAR_FLOAT > v2->VAR_FLOAT) : JLibDCOMPARE_IBBI(dp, v1,v2,4)) ;
}

#undef	GE_IBB
#define	GE_IBB(v1, v2) I_GE_IBB(dp, v1, v2)

static __inline__ INT32 I_GE_IBB(struct jBASEDataAreas * dp, VAR * v1 , VAR * v2 )
{
	return (((v1->VAR_TYPE & VAR_TYPE_FLOAT) && (v2->VAR_TYPE & VAR_TYPE_FLOAT) && (v1->VAR_FLOAT_SCALE == v2->VAR_FLOAT_SCALE)) ? (v1->VAR_FLOAT >= v2->VAR_FLOAT) : JLibDCOMPARE_IBBI(dp, v1,v2,5)) ;
}

#endif
#endif		// #define JSYSTEM2_H
