*
* Example program to demonstrate simple XML functions
* 
* Prerequisites on system for XML functions using stylesheets:
*
* Xalan and Xerces libraries in JBCRELEASEDIR/lib  or Library Path.
*
*
* Obtain StyleSheet for use with functions
*
*****************************************************************************************

    FileName = "XMLCUSTOMERS"
    OPEN 'DICT', FileName TO File ELSE STOP 201,FileName

    OutputStyleSheet = "@READXML"
    READ OutputXsl FROM File,OutputStyleSheet ELSE STOP 202,OutputStyleSheet

    InputStyleSheet = "@WRITEXML"
    READ InputXsl FROM File,InputStyleSheet ELSE STOP 202,InputStyleSheet

*
*
* Convert from Dynamic Array in to XML message using a stylesheet
*
* Xml = DYNTOXML(Dyn, Xsl, Result)
*
* Where:
*  Xml - Target variable for XML string
*     If error then variable contains reason message
*  Dyn - Source variable
*  Xsl - Stylesheet variable containing XSLT
*     If null string then generic XML  is returned
*  Result - 0 If XML conversion successful
*     If conversion failed then return non zero
*
*****************************************************************************************

    Dyn = "a":@AM:"b":@AM:"c" ;* Generate dynamic array
    CRT
    CRT "Original dynamic array"
    CRT CHANGE(Dyn,@AM,"^")

    GenXml = DYNTOXML(Dyn,"",Result)    ;* Convert to generic xml i.e. Without stylesheet
    CRT
    CRT "Data as generic XML"
    CRT "GenXml = DYNTOXML(Dyn,'',Result)"
    CRT GenXml

    Xml = DYNTOXML(Dyn,OutputXsl,Result)          ;* Convert to xml with stylesheet
    CRT
    CRT "Data as external XML using style sheet"
    CRT "Xml = DYNTOXML(Dyn,OutputXsl,Result)"
    CRT Xml

*
*
* Convert from XML string to Dynamic Array using a stylesheet
*
* Dyn = XMLTODYN(Xml, Xsl, Result)
*
* Where:
*  Dyn - Target variable for dynamic array
*     If error then variable contains reason message
*  Xml - Source XML string
*  Xsl - Stylesheet variable containing XSLT
*     If null string then generic XML is expected
*  Result - 0 If XML conversion successful
*     If conversion failed then return non zero
*
******************************************************************************************

    Dyn = XMLTODYN(GenXml, "", Result); ;* Convert generic XML string back to dynamic array
    CRT
    CRT "Dynamic array after conversion from generic XML"
    CRT "Dyn = XMLTODYN(GenXml, '', Result)"
    CRT CHANGE(Dyn,@AM,"^")

    Dyn = XMLTODYN(Xml, InputXsl, Result);        ;* Convert XML string to dynamic array using stylesheet
    CRT
    CRT "Dynamic array after conversion using style sheet from external XML"
    CRT "Dyn = XMLTODYN(Xml, InputXsl, Result)"
    CRT CHANGE(Dyn,@AM,"^")

*
*
* Convert from one XML string to another XML string using a stylesheet
*
* NewXml = XMLTOXML(Xml, Xsl, Result)
*
* Where:
*  NewXml - Target variable for new XML string
*     If error then variable contains reason message
*  Xml - Source XML string
*  Xsl - Stylesheet variable containing XSLT
*     If null string then generic XML is expected
*  Result - 0 If XML conversion successful
*     If conversion failed then return non zero
*
******************************************************************************************

    NewXml = XMLTOXML(GenXml, OutputXsl, Result)  ;* Convert XML string to XML using stylesheet
    CRT
    CRT "Xml string after conversion from generic XML using style sheet"
    CRT "NewXml = XMLTOXML(GenXml, OutputXsl, Result)"
    CRT NewXml

*
*
* READXML Record FROM File, Id ELSE ....
*
*  Where:
*  Record  - Target variable for XML String
*      Using StyleSheet @READXML from Dictionary
*
******************************************************************************************

    OPEN FileName TO File ELSE STOP 201,FileName

    SELECT File

    READNEXT Id ELSE STOP 202,Id

    READ Record FROM File, Id THEN
        CRT
        CRT "Record before READXML "
        CRT CHANGE(CHANGE(Record,@AM,"^"),@VM,"]")
    END

    READXML Record FROM File,Id ELSE STOP 202,Id

    CRT
    CRT "Record containing XML read from file using style sheet @READXML "
    CRT "READXML Record FROM File,Id ..."
    CRT Record

*
*
* WRITEXML Record ON File, Id ON ERROR ...
*
*  Where:
*  Record  - Source variable containing XML String
*      Using StyleSheet @WRITEXML from Dictionary
*
******************************************************************************************

    WRITEXML Record ON File, Id ON ERROR CRT "Problem with XML conversion ":Record

    READ Record FROM File, Id THEN
        CRT
        CRT "Record after WRITEXML using style sheet @WRITEXML "
    	CRT "WRITEXML Record ON File, Id ...."
        CRT CHANGE(CHANGE(Record,@AM,"^"),@VM,"]")
    END

    CRT "Done"
