    SUBROUTINE  DB2_SUB(TableName, Command, RecordKey, Record, DeferCommit, ErrorCode)
#define DB2_DEBUG 1

*
* Generate object files using the following jbc command
*
* jbc -c -Jqd DB2_SUB.b
*
*
* Generate subroutine library in $HOME/lib directory with the following command
*
* jcompile DB2_SUB.o SQLSUBS.o -s $HOME/lib/libDB2.so  (UNIX)
*
* or
*
* jcompile DB2_SUB.obj SQLSUBS.obj -s  %home%\lib\libDB2.dll (NT)
*
*
*
* User named common area for user driver flags and values
*
    COMMON /DB2DriverCommon/ UserArea

*
* Declare C function for error handling
*

    DEFC SQLERROR()
    DEFC SQLSETRESULT()

*
* Equate driver commands min and max
*

    EQUATE  DB2_MIN_CMD TO 0
    EQUATE  DB2_MAX_CMD TO 21
    EQUATE LOGNAME TO "DB2LogFile"
    EQUATE DB2TabName TO DB2TEST

*
* Named common user area to store information
* between driver calls to subroutine
*

    EQUATE UseLock  TO UserArea<1>
    EQUATE LockOpen  TO UserArea<2>
    EQUATE DispErr  TO UserArea<4>
    EQUATE DispWarn  TO UserArea<5>
    EQUATE DispNotFnd TO UserArea<6>
    EQUATE LogOn  TO UserArea<7>
    EQUATE DisplayOn TO UserArea<8>
!!
*
* Driver command values
*
* 0 Initialise
* 1 Open, Declare cursor
* 2 Close
* 3 Select, Open cursor
* 4 SelectEnd, Close cursor
* 5 ReadNext row
* 6 Read Row
* 7 Write Row
* 8 Delete Row
* 9 Lock Row
* 10 UnLock Row
* 11 Ioctl
* 12 Clear table
* 13 Delete table
* 14 Create table
* 15 Sync
* 16 Disconnect
* 17 TransBegin
* 18 TransCommit
* 19 TransRollBack
* 20 ReadV
* 21 WriteV
*

!!
*
* Declare JBC variables to be used in embedded SQL statements
*

    EXEC SQL BEGIN DECLARE SECTION;
    STRING(20) User;
    STRING(20) Password;
    STRING(30) RecordKey;
    STRING(3500) Record;
    STRING(100) LockStatement;
    STRING(100) SelectStatement;
    EXEC SQL END DECLARE SECTION;

*
* Define SQL cursor to use with SQL table
*

    EXEC SQL DECLARE Cursor CURSOR WITH HOLD FOR SELECT DB2Id FROM DB2TabName FOR READ ONLY;

*
* Define SQL cursor to use for locking
*

    EXEC SQL DECLARE LockCursor CURSOR FOR Statement;

*
* Assume no errors
*

    ErrorCode = 0

*
* Test for valid command
*

    IF NUM(Command) AND Command GE DB2_MIN_CMD AND Command LE DB2_MAX_CMD THEN

        Command++   ;* Increment command for ongosub

        ON Command GOSUB ...
        InitCmd, OpenCmd, CloseCmd, ...
        SelectCmd, SelectEndCmd, ReadNextCmd, ...
        ReadRecordCmd, WriteRecordCmd, DeleteRecordCmd, ...
        LockCmd, UnLockCmd, IoctlCmd, ...
        ClearFileCmd, DeleteFileCmd, CreateFileCmd, SyncCmd, ...
        DisconnectCmd, TransBeginCmd, TransCommitCmd, TransRollBackCmd, ...
        ReadVCmd, WriteVCmd

    END ELSE

        CRT SYSTEM(40):":Invalid driver command for SQL driver"
        ErrorCode = 5         ;* EIO

    END


    RETURN
!!
*
* Init routine
*
InitCmd:
    UserArea=""

    IF GETENV("JEDI_DB2_DB2LOCK",DUMMY) THEN UseLock = 1 ELSE UseLock = 0
    IF GETENV("JEDI_DB2_ERROR",DUMMY) THEN DispErr = 1 ELSE DispErr = 0
    IF GETENV("JEDI_DB2_WARN",DUMMY) THEN DispWarn = 1 ELSE DispWarn = 0
    IF GETENV("JEDI_DB2_NOTFOUND",DUMMY) THEN DispNotFnd = 1 ELSE DispNotFnd = 0
    IF GETENV("JEDI_DB2_LOG",DUMMY) THEN LogOn = 1 ELSE LogOn = 0
    IF GETENV("JEDI_DB2_DISPLAY",DUMMY) THEN DisplayOn = 1 ELSE DisplayOn = 0

#ifdef DB2_DEBUG
    ERRMSG ="INIT command" ; GOSUB ERRLOG
#endif

*
* Define what to do on error
*

    EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;
    EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

*
* Connect to database
*

    User = "db2inst1"
    Password = "space1999"

    EXEC SQL CONNECT TO TESTDB USER :User USING :Password;

    RETURN
!!
*
* Open file routine
*
OpenCmd:

    LockOpen=0

#ifdef DB2_DEBUG
    ERRMSG ="OPEN command" ; GOSUB ERRLOG
#endif

*
* Define what to do on error
*

    EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;
    EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

    RETURN
!!
*
* Close file routine
*
CloseCmd:

#ifdef DB2_DEBUG
    ERRMSG = "CLOSE command" ; GOSUB ERRLOG
#endif

    IF LockOpen THEN

        EXEC SQL CLOSE LockCursor ;

    END

    RETURN
!!
*
* SELECT routine
*
SelectCmd:


#ifdef DB2_DEBUG
    ERRMSG = "SELECT command" ; GOSUB ERRLOG
#endif

*
* Open pre declared cursor
*

    EXEC SQL OPEN Cursor ;

    RETURN
!!
*
* SELECTEND routine
*
SelectEndCmd:

#ifdef DB2_DEBUG
    ERRMSG = "SELECTEND command" ; GOSUB ERRLOG
#endif

*
* Close cursor
*

    EXEC SQL CLOSE Cursor ;

    RETURN
!!
*
* ReadNext routine
*
ReadNextCmd:

#ifdef DB2_DEBUG
    ERRMSG = "READNEXT command" ; GOSUB ERRLOG
#endif

*
* Fetch the Record Key from current Cursor position
*

    EXEC SQL FETCH Cursor INTO :RecordKey;

    RecordKey = TRIMB(RecordKey)

    RETURN
!!
*
* Read Record routine
*
ReadRecordCmd:

#ifdef DB2_DEBUG
    ERRMSG = "READRECORD command" ; GOSUB ERRLOG
#endif

*
* Read specific record
*

    EXEC SQL SELECT * INTO :RecordKey, :Record FROM DB2TabName WHERE DB2Id = :RecordKey ;

    RETURN
!!
*
* ReadV Record routine Only invoked with JEDI_DB2_VCMD
* Example of what can be done as this driver only has two columns
* Attribute to read is passed as first attribute in RecordKey
*
ReadVCmd:

#ifdef DB2_DEBUG
    ERRMSG = "READV command" ; GOSUB ERRLOG
#endif

*
* Read specific column into record
*
    ColumnNo = Record<1>

*
* Map attribute number to column
*
    BEGIN CASE
    CASE ColumnNo = 1
        EXEC SQL SELECT DB2Record INTO :Record FROM DB2TabName WHERE DB2Id = :RecordKey ;
    END CASE

    RETURN
!!
*
* Write Record routine
*
WriteRecordCmd:

#ifdef DB2_DEBUG
    ERRMSG = "WRITERECORD command" ; GOSUB ERRLOG
#endif

*
* Update record first
*

    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    EXEC SQL UPDATE DB2TabName SET DB2Record = :Record WHERE DB2Id = :RecordKey;

*
* Check if record did not exist
*

    IF SYSTEM(0) = 100 THEN

#ifdef DB2_DEBUG
        ERRMSG = "WRITERECORD command New Record" ; GOSUB ERRLOG
#endif

*
* Now insert record
*

        EXEC SQL INSERT INTO DB2TabName VALUES( :RecordKey, :Record ) ;
    END

    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

*
* Commit update
*

    IF NOT(DeferCommit) THEN

#ifdef DB2_DEBUG
        ERRMSG = "WRITERECORD Commit" ; GOSUB ERRLOG
#endif

        EXEC SQL COMMIT ;

        LockOpen = 0

    END

    RETURN
!!
*
* WriteV Record routine Only invoked with the JEDI_DB2_VCMD
* Example of what can be done as this driver only has two columns
* Attribute to write is first attribute of Record
*
WriteVCmd:

#ifdef DB2_DEBUG
    ERRMSG = "WRITEV command" ; GOSUB ERRLOG
#endif

*
* Update record first example only
*
    EXEC SQL WHENEVER NOT FOUND CONTINUE;

    ColumnNo = Record<1>      ;* Get required attribute number
    DEL Record<1>   ;* Remove attribute reference

*
* Update only required column
*
    BEGIN CASE
    CASE ColumnNo = 1
        EXEC SQL UPDATE DB2TabName SET DB2Record = :Record WHERE DB2Id = :RecordKey;
    END CASE

    IF SYSTEM(0) = 100 THEN

#ifdef DB2_DEBUG
        ERRMSG = "WRITEV command New Record" ; GOSUB ERRLOG
#endif

*
* Now insert required column
*
        BEGIN CASE
        CASE ColumnNo = 1
            EXEC SQL INSERT INTO DB2TabName VALUES( :RecordKey, :Record ) ;
        END CASE
    END

*
* Commit update
*

    IF NOT(DeferCommit) THEN

#ifdef DB2_DEBUG
        ERRMSG = "WRITEV Commit" ; GOSUB ERRLOG
#endif

        EXEC SQL COMMIT ;

        LockOpen = 0

    END

    RETURN
!!
*
* Delete Record routine
*
DeleteRecordCmd:

#ifdef DB2_DEBUG
    ERRMSG  = "DELETERECORD command" ; GOSUB ERRLOG
#endif

*
* Delete specific record
*

    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    EXEC SQL DELETE FROM DB2TabName WHERE DB2Id = :RecordKey ;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;


    IF NOT(DeferCommit) THEN

#ifdef DB2_DEBUG
        ERRMSG ="DELETERECORD Commit" ; GOSUB ERRLOG
#endif
        EXEC SQL COMMIT ;

        LockOpen = 0

    END

    RETURN
!!
*
* Lock routine
*
LockCmd:

#ifdef DB2_DEBUG
    ERRMSG = "LOCK command" ; GOSUB ERRLOG
#endif

    IF UseLock THEN

#ifdef DB2_DEBUG
        ERRMSG = "LOCK use DB2 lock" ; GOSUB ERRLOG
#endif

        LockStatement = "SELECT DB2Id FROM DB2TEST WHERE DB2Id = '":RecordKey:"' FOR UPDATE"

        EXEC SQL PREPARE Statement FROM :LockStatement ;

        EXEC SQL OPEN LockCursor;

        EXEC SQL WHENEVER NOT FOUND CONTINUE;
        EXEC SQL FETCH LockCursor INTO :RecordKey;
        EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

        LockOpen = 1

    END


    RETURN
!!
*
* UnLock routine
*
UnLockCmd:

#ifdef DB2_DEBUG
    ERRMSG = "UNLOCK command" ; GOSUB ERRLOG
#endif

    IF UseLock THEN

#ifdef DB2_DEBUG
        ERRMSG = "UNLOCK use DB2 lock" ; GOSUB ERRLOG
#endif

        IF LockOpen THEN

            EXEC SQL CLOSE LockCursor;
        END

        LockOpen = 0

    END

    RETURN
!!
*
* Ioctl routine
*
IoctlCmd:

#ifdef DB2_DEBUG
    ERRMSG = "IOCTL command" ; GOSUB ERRLOG
#endif
    BEGIN CASE
    CASE RecordKey = 5000     ;* Check IOCTL SubCommand value
        SelectStatement = Record        ;* Selection statement passed in Record
        EXEC SQL PREPARE S2 FROM :SelectStatement;
        EXEC SQL DECLARE C CURSOR FOR S2;
        EXEC SQL OPEN C ;
        EXEC SQL WHENEVER NOT FOUND GOTO proglab_EXIT5000;
        Record=""
        LOOP
            EXEC SQL FETCH C INTO :RecordKey;
            RecordKey = TRIMB(RecordKey)
            Record := RecordKey:@AM
        REPEAT      ;* NOT FOUND will exit loop
EXIT5000:
        EXEC SQL CLOSE C;
    END CASE

    RETURN
!!
*
* DeleteFile routine
*
DeleteFileCmd:

#ifdef DB2_DEBUG
    ERRMSG = "DELETEFILE command" ; GOSUB ERRLOG
#endif

    IF INDEX(TableName,"]D",1) THEN
        EXEC SQL WHENEVER NOT FOUND CONTINUE;
        EXEC SQL WHENEVER SQLWARNING CONTINUE;
        EXEC SQL WHENEVER SQLERROR CONTINUE;
        EXEC SQL DROP TABLE DB2TabName;
        EXEC SQL COMMIT;
        EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;
        EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;
        EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;
    END

    RETURN
!!
*
* ClearFile routine
*
ClearFileCmd:

#ifdef DB2_DEBUG
    ERRMSG = "CLEARFILE command" ; GOSUB ERRLOG
#endif
    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    EXEC SQL DELETE FROM DB2TabName ;
    EXEC SQL COMMIT;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

    RETURN
!!
*
* CreateFile routine
*
CreateFileCmd:

#ifdef DB2_DEBUG
    ERRMSG = "CREATEFILE command" ; GOSUB ERRLOG
#endif

    IF INDEX(TableName,"]D",1) THEN
        EXEC SQL CREATE TABLE DB2TabName ( DB2Id VARCHAR(50) NOT NULL PRIMARY KEY, DB2Record VARCHAR(3500));
        EXEC SQL COMMIT;
    END

    RETURN
!!
*
* Sync routine
*
SyncCmd:

#ifdef DB2_DEBUG
    ERRMSG = "SYNC command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* Disconnect routine
*
DisconnectCmd:

#ifdef DB2_DEBUG
    ERRMSG = "DISCONNECT command" ; GOSUB ERRLOG
#endif

    EXEC SQL CONNECT RESET;

    RETURN
!!
*
* TransBegin routine
*
TransBeginCmd:

#ifdef DB2_DEBUG
    ERRMSG = "TRANSBEGIN command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* TransCommit routine
*
TransCommitCmd:

#ifdef DB2_DEBUG
    ERRMSG = "TRANSCOMMIT command" ; GOSUB ERRLOG
#endif

    EXEC SQL COMMIT ;

    RETURN
!!
*
* TransRollBack routine
*
TransRollBackCmd:

#ifdef DB2_DEBUG
    ERRMSG = "TRANSROLLBACK command" ; GOSUB ERRLOG
#endif

    EXEC SQL ROLLBACK ;

    RETURN
!!
*
* NotFound routine
*
NOTFOUND:

    IF DispNotFnd THEN SQLERROR()

    ErrorCode = 100

#ifdef DB2_DEBUG
    ERRMSG = "NOTFOUND  :":ErrorCode; GOSUB ERRLOG
    ERRMSG = "UserArea  :":UserArea ; GOSUB ERRLOG
#endif

    RETURN
*
* Warning routine
*
WARNING:

    IF DispWarn THEN SQLERROR()

    ErrorCode = SYSTEM(0)

#ifdef DB2_DEBUG
    ERRMSG = "WARNING  :":ErrorCode ; GOSUB ERRLOG
    ERRMSG = "UserArea :":UserArea ; GOSUB ERRLOG
#endif

    ErrorCode = 5

    RETURN

*
* Error routine
*
ERROR:

    SQLSETRESULT()  ;* Setup result code in SYSTEM(0)

    IF DispErr THEN SQLERROR()

    ErrorCode = SYSTEM(0)

#ifdef DB2_DEBUG
    ERRMSG = "ERROR    :":ErrorCode ; GOSUB ERRLOG
    ERRMSG = "UserArea :":UserArea ; GOSUB ERRLOG
#endif

    ErrorCode = 5

    RETURN
!!
*
* Error Log
*
ERRLOG:

#ifdef DB2_DEBUG

    IF DisplayOn THEN
        CRT SYSTEM(40):":":ERRMSG
    END

    IF LogOn THEN
        LOG=DATE() "D2":":":TIME() "MTS":":":SYSTEM(50):":":SYSTEM(40):":":ERRMSG
        OPENSEQ LOGNAME TO SEQFILE THEN
            WRITESEQ LOG APPEND TO SEQFILE ELSE NULL
            CLOSESEQ SEQFILE
        END
    END
#endif

    RETURN
