    SUBROUTINE  GEN_SUB(FileName, Command, RecordKey, Record, DeferCommit, ErrorCode)

*
* Generate object files using the following jbc command
*
* jbc -c -Jqo GEN_SUB.b
*
* jbc -JDGEN_DEBUG -c -Jqo GEN_SUB.b to include code between #ifdef GEN_DEBUG
*
* Generate subroutine library in $HOME/lib directory with the following command
*
* jcompile GEN_SUB.o {Another.o} -s $HOME/lib/libGEN.so  (UNIX)
*
* or
*
* jcompile GEN_SUB.o {Another.o} -s  %home%\lib\libGEN.dll (NT)
*
*
*
* User named common area for user driver flags and values
*
    COMMON /GENDriverCommon/ UserArea
    COMMON /GENDriverPrivate/ FileVar, FileVarList

*
* Equate driver commands min and max
*

    EQUATE  GEN_MIN_CMD TO 0
    EQUATE  GEN_MAX_CMD TO 19
    EQUATE LOGNAME TO "GENLogFile"

*
* Named common user area to store information
* between driver calls to subroutine
*

    EQUATE UseLock  TO UserArea<1>
    EQUATE LockOpen  TO UserArea<2>
    EQUATE DispErr  TO UserArea<4>
    EQUATE DispWarn  TO UserArea<5>
    EQUATE DispNotFnd TO UserArea<6>
    EQUATE LogOn  TO UserArea<7>
    EQUATE DisplayOn TO UserArea<8>
!!
*
* Driver command values
*
* 0 Initialise
* 1 Open, Declare cursor
* 2 Close
* 3 Select, Open cursor
* 4 SelectEnd, Close cursor
* 5 ReadNext row
* 6 Read FullRecord
* 7 Write FullRecord
* 8 Delete FullRecord
* 9 Lock FullRecord
* 10 UnLock FullRecord
* 11 Ioctl
* 12 Clear table
* 13 Delete table
* 14 Create table
* 15 Sync
* 16 Disconnect
* 17 TransBegin
* 18 TransCommit
* 19 TransRollBack
*


*
* Debug Info
*

#ifdef GEN_DEBUG
    ERRMSG ="FileName :":FileName ; GOSUB ERRLOG
    ERRMSG ="Command :":Command ; GOSUB ERRLOG
    ERRMSG ="RecordKey :":RecordKey ; GOSUB ERRLOG
    ERRMSG ="Record :":Record; GOSUB ERRLOG
    ERRMSG ="ErrorCode :":ErrorCode ; GOSUB ERRLOG
    ERRMSG ="UserArea :":UserArea ; GOSUB ERRLOG
    ERRMSG ="DeferCommit :":DeferCommit ; GOSUB ERRLOG
    ERRMSG ="" ;GOSUB ERRLOG
#endif

!!

*
* Assume no errors
*

    ErrorCode = 0

*
* Test for valid command
*

    IF NUM(Command) AND Command GE GEN_MIN_CMD AND Command LE GEN_MAX_CMD THEN

        Command++   ;* Increment command for ongosub

        ON Command GOSUB ...
        InitCmd, OpenCmd, CloseCmd, ...
        SelectCmd, SelectEndCmd, ReadNextCmd, ...
        ReadRecordCmd, WriteRecordCmd, DeleteRecordCmd, ...
        LockCmd, UnLockCmd, IoctlCmd, ...
        ClearFileCmd, DeleteFileCmd, CreateFileCmd, SyncCmd, ...
        DisconnectCmd, TransBeginCmd, TransCommitCmd, TransRollBackCmd

    END ELSE

        CRT SYSTEM(40):":Invalid driver command for GEN driver"
        ErrorCode = 5         ;* EIO

    END


    RETURN
!!
*
* Init routine
*
InitCmd:
    UserArea=""

    IF GETENV("JEDI_GEN_GENLOCK",DUMMY) THEN UseLock = 1 ELSE UseLock = 0
    IF GETENV("JEDI_GEN_ERROR",DUMMY) THEN DispErr = 1 ELSE DispErr = 0
    IF GETENV("JEDI_GEN_WARN",DUMMY) THEN DispWarn = 1 ELSE DispWarn = 0
    IF GETENV("JEDI_GEN_NOTFOUND",DUMMY) THEN DispNotFnd = 1 ELSE DispNotFnd = 0
    IF GETENV("JEDI_GEN_LOG",DUMMY) THEN LogOn = 1 ELSE LogOn = 0
    IF GETENV("JEDI_GEN_DISPLAY",DUMMY) THEN DisplayOn = 1 ELSE DisplayOn = 0
#ifdef GEN_DEBUG
    ERRMSG ="INIT command" ; GOSUB ERRLOG
#endif

    RETURN

!!
*
* Open file routine
*
OpenCmd:

    LockOpen=0

#ifdef GEN_DEBUG
    ERRMSG ="OPEN command" ; GOSUB ERRLOG
#endif

    OPEN FileName TO FileVar ELSE ERRMSG="Cannot open ":FileName ; GOTO ERROR

    RETURN
!!
*
* Close file routine
*
CloseCmd:

#ifdef GEN_DEBUG
    ERRMSG = "CLOSE command" ; GOSUB ERRLOG
#endif

    IF LockOpen THEN

        RELEASE FileVar

    END

    CLOSE FileVar

    RETURN
!!
*
* SELECT routine
*
SelectCmd:


#ifdef GEN_DEBUG
    ERRMSG = "SELECT command" ; GOSUB ERRLOG
#endif

    SELECT FileVar TO FileVarList

    RETURN
!!
*
* SELECTEND routine
*
SelectEndCmd:

#ifdef GEN_DEBUG
    ERRMSG = "SELECTEND command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* ReadNext routine
*
ReadNextCmd:

#ifdef GEN_DEBUG
    ERRMSG = "READNEXT command" ; GOSUB ERRLOG
#endif

    READNEXT RecordKey FROM FileVarList ELSE GOTO NOTFOUND

    RETURN
!!
*
* Read Record routine
*
ReadRecordCmd:

#ifdef GEN_DEBUG
    ERRMSG = "READRECORD command" ; GOSUB ERRLOG
#endif

*
* Read specific record
*

    READ Record FROM FileVar,RecordKey ELSE GOTO NOTFOUND

    RETURN
!!
*
* Write Record routine
*
WriteRecordCmd:

#ifdef GEN_DEBUG
    ERRMSG = "WRITERECORD command" ; GOSUB ERRLOG
#endif

*
* Update record first
*
    WRITE Record ON FileVar,RecordKey

    LockOpen = 0

    RETURN
!!
*
* Delete Record routine
*
DeleteRecordCmd:

#ifdef GEN_DEBUG
    ERRMSG  = "DELETERECORD command" ; GOSUB ERRLOG
#endif

*
* Delete specific record
*

    DELETE FileVar,RecordKey
    LockOpen = 0

    RETURN
!!
*
* Lock routine
*
LockCmd:

#ifdef GEN_DEBUG
    ERRMSG = "LOCK command" ; GOSUB ERRLOG
#endif

    IF UseLock THEN

#ifdef GEN_DEBUG
        ERRMSG = "LOCK use GEN lock" ; GOSUB ERRLOG
#endif

        READU Record FROM FileVar, RecordKey ELSE NULL

        LockOpen = 1

    END


    RETURN
!!
*
* UnLock routine
*
UnLockCmd:

#ifdef GEN_DEBUG
    ERRMSG = "UNLOCK command" ; GOSUB ERRLOG
#endif

    IF UseLock THEN

#ifdef GEN_DEBUG
        ERRMSG = "UNLOCK use GEN lock" ; GOSUB ERRLOG
#endif

        IF LockOpen THEN

            RELEASE FileVar, RecordKey
        END

        LockOpen = 0

    END

    RETURN
!!
*
* Ioctl routine
*
IoctlCmd:

#ifdef GEN_DEBUG
    ERRMSG = "IOCTL command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* DeleteFile routine
*
DeleteFileCmd:

#ifdef GEN_DEBUG
    ERRMSG = "DELETEFILE command" ; GOSUB ERRLOG
#endif

    IF ASSIGNED(FileVar) THEN CLOSE FileVar

    IF INDEX(FileName,"]D",1) THEN RETURN

    PERFORM "DELETE-FILE ":FileName CAPTURING VIEW

    RETURN
!!
*
* ClearFile routine
*
ClearFileCmd:

#ifdef GEN_DEBUG
    ERRMSG = "CLEARFILE command" ; GOSUB ERRLOG
#endif

    CLEARFILE FileVar


    RETURN
!!
*
* CreateFile routine
*
CreateFileCmd:

#ifdef GEN_DEBUG
    ERRMSG = "CREATEFILE command" ; GOSUB ERRLOG
#endif

    IF INDEX(FileName,"]D",1) THEN RETURN

    PERFORM "CREATE-FILE ":FileName:" 1 11" CAPTURING VIEW

    RETURN
!!
*
* Sync routine
*
SyncCmd:

#ifdef GEN_DEBUG
    ERRMSG = "SYNC command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* Disconnect routine
*
DisconnectCmd:

#ifdef GEN_DEBUG
    ERRMSG = "DISCONNECT command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* TransBegin routine
*
TransBeginCmd:

#ifdef GEN_DEBUG
    ERRMSG = "TRANSBEGIN command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* TransCommit routine
*
TransCommitCmd:

#ifdef GEN_DEBUG
    ERRMSG = "TRANSCOMMIT command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* TransRollBack routine
*
TransRollBackCmd:

#ifdef GEN_DEBUG
    ERRMSG = "TRANSROLLBACK command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* NotFound routine
*
NOTFOUND:

    IF DispNotFnd THEN CRT ERRMSG

    ErrorCode = 100

#ifdef GEN_DEBUG
    ERRMSG = "NOTFOUND  :":ErrorCode; GOSUB ERRLOG
    ERRMSG = "UserArea  :":UserArea ; GOSUB ERRLOG
#endif

    RETURN
*
* Warning routine
*
WARNING:

    IF DispWarn THEN GOSUB ERRLOG

    ErrorCode = SYSTEM(0)

#ifdef GEN_DEBUG
    ERRMSG = "WARNING  :":ErrorCode ; GOSUB ERRLOG
    ERRMSG = "UserArea :":UserArea ; GOSUB ERRLOG
#endif

    ErrorCode = 5

    RETURN

*
* Error routine
*
ERROR:

    IF DispErr THEN GOSUB ERRLOG

    ErrorCode = SYSTEM(0)

#ifdef GEN_DEBUG
    ERRMSG = "ERROR    :":ErrorCode ; GOSUB ERRLOG
    ERRMSG = "UserArea :":UserArea ; GOSUB ERRLOG
#endif

    ErrorCode = 5

    RETURN
!!
*
* Error Log
*
ERRLOG:

#ifdef GEN_DEBUG

    IF DisplayOn THEN
        CRT SYSTEM(40):":":ERRMSG
    END

    IF LogOn THEN
        LOG=DATE() "D2":":":TIME() "MTS":":":SYSTEM(50):":":SYSTEM(40):":":ERRMSG
        OPENSEQ LOGNAME TO SEQFILE THEN
            WRITESEQ LOG APPEND TO SEQFILE ELSE NULL
            CLOSESEQ SEQFILE
        END
    END
#endif

    RETURN
