#
#	all		(Default) Build all libraries and main programs
#	libs		build only the libraries
#	rellib		release libraries only
#	release		release libraries and main programs
#
# Set environment variables for debug 
#
# DEBUG=-JCDDB2_DEBUG
# JEDI_DB2_DISPLAY=1
# JEDI_DB2_LOG=1
# JEDI_DB2_WARN=1
# JEDI_DB2_NOTFOUND=1
# JEDI_DB2_ERROR=1
#
# Set environment variable DB2PREPSCRIPT to precompiler script
# e.g. DB2Prep.ksh
#
#

#DEBUG=-JCDDB2_DEBUG
CC=cc
CFLAGS=
CP_CMD=cp
RM_CMD=rm
CHMOD_CMD=chmod
MODE_R=a+r
MODE_RX=a+rx
REL_DIR_LIB=$(HOME)/lib
JBCMAINFLAGS=-Jqd -Jv
JBCLIBFLAGS=-c -Jqd -Jv
DB2FLAGS=-DDB2SQL
SHARED_LIB_EXTENSION=.so
SHARED_EXPORT_EXTENSION=.el
BUILDSLIB=jcompile
DB2_SHARED_LIB=-ldb2

.SUFFIXES: .b .sqc

EXECS1=DB2_MAIN

#
#-----------------------
# Shared libraries
#-----------------------
#
DB2_SUB_OBJS	= \
		DB2_SUB.o 

SQL_SUB_OBJS	= \
		sqlsubs.o

SQL_SUB_CSOURCE	= \
		sqlsubs.c


#
# Describe the Shared libraries
#

DB2_SUB_LIB_SHARED	= libDB2_SUB$(SHARED_LIB_EXTENSION)
DB2_SUB_LIB_EXPORT	= $(DB2_SUB_LIB_SHARED)$(SHARED_EXPORT_EXTENSION)

.PRECIOUS	: 

#
#----------------------------
#   S T A R T 	H E R E
#----------------------------
#

all:	libs $(EXECS1)

#-------------------------------
# build only the Shared libraries
#-------------------------------
#
libs:  shlib

shlib: $(DB2_SUB_LIB_SHARED)

#
# The shared library is built from the .o files
#

$(DB2_SUB_LIB_SHARED): $(SQL_SUB_OBJS) $(DB2_SUB_OBJS)
	$(BUILDSLIB) -s $(DB2_SUB_LIB_SHARED) $(DB2_SUB_OBJS) $(SQL_SUB_OBJS) $(DB2_SHARED_LIB) 

#
# Rule for .sqc programs
#

.sqc.o:
	$(DB2PREPSCRIPT) $<
	cc -c $(DB2FLAGS) $(*).c -I$(JBCRELEASEDIR)/include

#
# Rule for subroutine programs
#

.b.o: 	
	jbc $(DEBUG) $(JBCLIBFLAGS) -Jo $<

#
# Default rule for Main programs
#
.b:
	jbc $(DEBUG) $(JBCMAINFLAGS) -Jo $(@).b
#
#---------------
# Release Rules
#---------------
# release libraries only
rellib: 
	$(CHMOD_CMD) $(MODE_RX) $(DB2_SUB_LIB_SHARED)
#
# We must delete the shared object first for two reasons.
# (1) AIX believes it to be busy and the 'cp' will fail otherwise.
# (2) If anybody was using it already, then they will continue to use
#     the old version (deleted once file closed), and the new version
#     can SAFELY be copied.
#
	-$(RM_CMD) $(REL_DIR_LIB)/$(DB2_SUB_LIB_SHARED)
	$(CP_CMD) $(DB2_SUB_LIB_SHARED) $(REL_DIR_LIB)
	$(CP_CMD) $(DB2_SUB_LIB_EXPORT) $(REL_DIR_LIB)
	$(CHMOD_CMD) $(MODE_R) $(REL_DIR_LIB)/$(DB2_SUB_LIB_EXPORT)

release: rellib

clean:
	$(RM_CMD) -f $(DB2_SUB_LIB_SHARED) $(DB2_SUB_LIB_EXPORT) $(DB2_SUB_OBJS) $(SQL_SUB_OBJS) $(SQL_SUB_CSOURCE)
