    SUBROUTINE  ORA_SUB(TableName, Command, RecordKey, Record, DeferCommit, ErrorCode)
#define ORA_DEBUG 1

*
* Generate object files using the following jbc command
*
* jbc -c -Jqo ORA_SUB1.b
*
* Generate subroutine library in $HOME/lib directory with the following command
*
* jcompile ORA_SUB1.o SQLSUBS.o -s $HOME/lib/libORA.so  (UNIX)
*
* or
*
* jcompile ORA_SUB1.obj SQLSUBS.obj -s  %home%\lib\libORA.dll (NT)
*
*
*
* User named common area for user driver flags and values
*
    COMMON /ORADriverCommon/ UserArea

*
* Declare C function for error handling
*

    DEFC SQLERROR()
    DEFC SQLSETRESULT()

*
* Equate driver commands min and max
*

    EQUATE  ORA_MIN_CMD TO 0
    EQUATE  ORA_MAX_CMD TO 21
    EQUATE LOGNAME TO "ORALogFile"
    EQUATE ORATabName TO ORATEST1

*
* Named common user area to store information
* between driver calls to subroutine
*

    EQUATE UseLock  TO UserArea<1>
    EQUATE LockOpen  TO UserArea<2>
    EQUATE DispErr  TO UserArea<4>
    EQUATE DispWarn  TO UserArea<5>
    EQUATE DispNotFnd TO UserArea<6>
    EQUATE LogOn  TO UserArea<7>
    EQUATE DisplayOn TO UserArea<8>
!!
*
* Driver command values
*
* 0 Initialise
* 1 Open, Declare cursor
* 2 Close
* 3 Select, Open cursor
* 4 SelectEnd, Close cursor
* 5 ReadNext row
* 6 Read Row
* 7 Write Row
* 8 Delete Row
* 9 Lock Row
* 10 UnLock Row
* 11 Ioctl
* 12 Clear table
* 13 Delete table
* 14 Create table
* 15 Sync
* 16 Disconnect
* 17 TransBegin
* 18 TransCommit
* 19 TransRollBack
* 20 ReadV
* 21 WriteV
*

!!
*
* Declare JBC variables to be used in embedded SQL statements
*

    EXEC SQL BEGIN DECLARE SECTION;
    STRING(20) userid;
    STRING(30) passwd;
    STRING(30) RecordKey;
    VARCHAR(1500) Record;
    VARCHAR(100) RecordX;
    VARCHAR(100) RecordY;
    STRING(100) LockStatement;
    STRING(100) SelectStatement;
    EXEC SQL END DECLARE SECTION;

*
* Define SQL cursor to use with SQL table
*

    EXEC SQL DECLARE OraStatement STATEMENT ;
    EXEC SQL DECLARE OraCursor CURSOR FOR SELECT ORAId FROM ORATabName;

*
* Define SQL cursor to use for locking
*

    EXEC SQL DECLARE LockCursor CURSOR FOR OraStatement;

*
* Assume no errors
*
    ErrorCode = 0

*
* Test for valid command
*

    IF NUM(Command) AND Command GE ORA_MIN_CMD AND Command LE ORA_MAX_CMD THEN

        Command++   ;* Increment command for ongosub

        ON Command GOSUB ...
        InitCmd, OpenCmd, CloseCmd, ...
        SelectCmd, SelectEndCmd, ReadNextCmd, ...
        ReadRecordCmd, WriteRecordCmd, DeleteRecordCmd, ...
        LockCmd, UnLockCmd, IoctlCmd, ...
        ClearFileCmd, DeleteFileCmd, CreateFileCmd, SyncCmd, ...
        DisconnectCmd, TransBeginCmd, TransCommitCmd, TransRollBackCmd, ...
        ReadVCmd, WriteVCmd

    END ELSE

        CRT SYSTEM(40):":Invalid driver command for SQL driver"
        ErrorCode = 5         ;* EIO

    END


    RETURN
!!
*
* Init routine
*
InitCmd:
    UserArea=""

    IF GETENV("JEDI_ORA_ORALOCK",DUMMY) THEN UseLock = 1 ELSE UseLock = 0
    IF GETENV("JEDI_ORA_ERROR",DUMMY) THEN DispErr = 1 ELSE DispErr = 0
    IF GETENV("JEDI_ORA_WARN",DUMMY) THEN DispWarn = 1 ELSE DispWarn = 0
    IF GETENV("JEDI_ORA_NOTFOUND",DUMMY) THEN DispNotFnd = 1 ELSE DispNotFnd = 0
    IF GETENV("JEDI_ORA_LOG",DUMMY) THEN LogOn = 1 ELSE LogOn = 0
    IF GETENV("JEDI_ORA_DISPLAY",DUMMY) THEN DisplayOn = 1 ELSE DisplayOn = 0

#ifdef ORA_DEBUG
    ERRMSG ="INIT command" ; GOSUB ERRLOG
#endif

*
* Define what to do on error
*

    EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;
    EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

*
* Connect to database
*

    userid = "scott"
    passwd = "tiger"

    EXEC SQL CONNECT :userid IDENTIFIED BY :passwd;

    RETURN
!!
*
* Open file routine
*
OpenCmd:

    LockOpen=0

#ifdef ORA_DEBUG
    ERRMSG ="OPEN command" ; GOSUB ERRLOG
#endif

*
* Define what to do on error
*

    EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;
    EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

    RETURN
!!
*
* Close file routine
*
CloseCmd:

#ifdef ORA_DEBUG
    ERRMSG = "CLOSE command" ; GOSUB ERRLOG
#endif

    IF LockOpen THEN

        EXEC SQL CLOSE LockCursor ;

    END

    RETURN
!!
*
* SELECT routine
*
SelectCmd:


#ifdef ORA_DEBUG
    ERRMSG = "SELECT command" ; GOSUB ERRLOG
#endif

*
* Open pre declared cursor
*

    EXEC SQL OPEN OraCursor ;

    RETURN
!!
*
* SELECTEND routine
*
SelectEndCmd:

#ifdef ORA_DEBUG
    ERRMSG = "SELECTEND command" ; GOSUB ERRLOG
#endif

*
* Close cursor
*

    EXEC SQL CLOSE OraCursor ;

    RETURN
!!
*
* ReadNext routine
*
ReadNextCmd:

#ifdef ORA_DEBUG
    ERRMSG = "READNEXT command" ; GOSUB ERRLOG
#endif

*
* Fetch the Record Key from current Cursor position
*

    EXEC SQL FETCH OraCursor INTO :RecordKey;
    RecordKey = TRIMB(RecordKey)

    RETURN
!!
*
* Read Record routine
*
ReadRecordCmd:

#ifdef ORA_DEBUG
    ERRMSG = "READRECORD command" ; GOSUB ERRLOG
#endif

*
* Read specific record
*

    EXEC SQL WHENEVER SQLERROR CONTINUE;
    EXEC SQL SELECT * INTO :RecordKey, :Record, :RecordX, :RecordY FROM ORATabName WHERE ORAId = :RecordKey ;

    IF SYSTEM(0) NE 0 AND SYSTEM(0) NE -1405 THEN GOTO ERROR

    RecordKey = TRIMB(RecordKey)
    INS RecordY BEFORE Record<2>
    INS RecordX BEFORE Record<2>

    EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;

    RETURN
!!
*
* ReadV Record routine Only invoked with JEDI_ORA_VCMD
* Example of what can be done as this driver only has two columns
* Attribute to read is passed as first attribute in RecordKey
* Only called when JEDI_ORA_VCMDS=1 configured
*
ReadVCmd:

#ifdef ORA_DEBUG
    ERRMSG = "READV command" ; GOSUB ERRLOG
#endif

*
* Read specific column into record
*
    ColumnNo = Record<1>

*
* Map attribute number to column
*
    BEGIN CASE
    CASE ColumnNo = 2
        EXEC SQL SELECT ORARecordX INTO :Record FROM ORATabName WHERE ORAId = :RecordKey ;
    CASE ColumnNo = 3
        EXEC SQL SELECT ORARecordY INTO :Record FROM ORATabName WHERE ORAId = :RecordKey ;
    END CASE

    RETURN
!!
*
* Write Record routine
*
WriteRecordCmd:

#ifdef ORA_DEBUG
    ERRMSG = "WRITERECORD command" ; GOSUB ERRLOG
#endif

*
* Update record first
*
    RecordX = Record<2>
    DEL Record<2>
    RecordY = Record<2>
    DEL Record<2>

    EXEC SQL WHENEVER NOT FOUND CONTINUE;

    EXEC SQL UPDATE ORATabName SET ORARecord = :Record, ORARecordX = :RecordX, ORARecordY = :RecordY WHERE ORAId = :RecordKey;

    IF SYSTEM(0) = 1403 OR SYSTEM(0) = 100 THEN

#ifdef ORA_DEBUG
        ERRMSG = "WRITERECORD command New Record" ; GOSUB ERRLOG
#endif

*
* Now insert record
*
        EXEC SQL INSERT INTO ORATabName VALUES( :RecordKey, :Record, :RecordX, :RecordY ) ;
    END

*
* Commit update
*

    IF NOT(DeferCommit) THEN

#ifdef ORA_DEBUG
        ERRMSG = "WRITERECORD Commit" ; GOSUB ERRLOG
#endif

        EXEC SQL COMMIT ;

        LockOpen = 0

    END

    RETURN
!!
*
* WriteV Record routine Only invoked with the JEDI_ORA_VCMD
* Example of what can be done as this driver only has two columns
* Attribute to write is first attribute of Record
* Only called when JEDI_ORA_VCMDS=1 configured
*
WriteVCmd:

#ifdef ORA_DEBUG
    ERRMSG = "WRITEV command" ; GOSUB ERRLOG
#endif

*
* Update record first example only
*
    EXEC SQL WHENEVER NOT FOUND CONTINUE;

    ColumnNo = Record<1>      ;* Get required attribute number
    DEL Record<1>   ;* Remove attribute reference

*
* Update only required column
*
    BEGIN CASE
    CASE ColumnNo = 2
        EXEC SQL UPDATE ORATabName SET ORARecordX = :Record WHERE ORAId = :RecordKey;
    CASE ColumnNo = 3
        EXEC SQL UPDATE ORATabName SET ORARecordY = :Record WHERE ORAId = :RecordKey;
    END CASE

    IF SYSTEM(0) = 1403 OR SYSTEM(0) = 100 THEN

#ifdef ORA_DEBUG
        ERRMSG = "WRITEV command New Record" ; GOSUB ERRLOG
#endif

*
* Now insert required column
*
        BEGIN CASE
        CASE ColumnNo = 2
            EXEC SQL INSERT INTO ORATabName VALUES( :RecordKey, '', :Record, '' ) ;
        CASE ColumnNo = 3
            EXEC SQL INSERT INTO ORATabName VALUES( :RecordKey, '', '', :Record ) ;
        END CASE
    END

*
* Commit update
*

    IF NOT(DeferCommit) THEN

#ifdef ORA_DEBUG
        ERRMSG = "WRITEV Commit" ; GOSUB ERRLOG
#endif

        EXEC SQL COMMIT ;

        LockOpen = 0

    END

    RETURN
!!
*
* Delete Record routine
*
DeleteRecordCmd:

#ifdef ORA_DEBUG
    ERRMSG  = "DELETERECORD command" ; GOSUB ERRLOG
#endif

*
* Delete specific record
*

    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    EXEC SQL DELETE FROM ORATabName WHERE ORAId = :RecordKey ;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;


    IF NOT(DeferCommit) THEN

#ifdef ORA_DEBUG
        ERRMSG ="DELETERECORD Commit" ; GOSUB ERRLOG
#endif
        EXEC SQL COMMIT ;

        LockOpen = 0

    END

    RETURN
!!
*
* Lock routine. Only called when JEDI_ORA_JRLALOCK=1 NOT configured.
*
LockCmd:

#ifdef ORA_DEBUG
    ERRMSG = "LOCK command" ; GOSUB ERRLOG
#endif

    IF UseLock THEN

#ifdef ORA_DEBUG
        ERRMSG = "LOCK use ORA lock" ; GOSUB ERRLOG
#endif

        LockStatement = "SELECT ORAId FROM ORATEST1 WHERE ORAId = '":RecordKey:"' FOR UPDATE"

        EXEC SQL PREPARE OraStatement FROM :LockStatement ;

        EXEC SQL OPEN LockCursor;

        EXEC SQL WHENEVER NOT FOUND CONTINUE;
        EXEC SQL FETCH LockCursor INTO :RecordKey;
        EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;

        LockOpen = 1

    END


    RETURN
!!
*
* UnLock routine. Only called when JEDI_ORA_JRLALOCK=1 NOT configured.
*
UnLockCmd:

#ifdef ORA_DEBUG
    ERRMSG = "UNLOCK command" ; GOSUB ERRLOG
#endif

    IF UseLock THEN

#ifdef ORA_DEBUG
        ERRMSG = "UNLOCK use ORA lock" ; GOSUB ERRLOG
#endif

        IF LockOpen THEN

            EXEC SQL CLOSE LockCursor;
        END

        LockOpen = 0

    END

    RETURN
!!
*
* Ioctl routine
*
IoctlCmd:

#ifdef ORA_DEBUG
    ERRMSG = "IOCTL command" ; GOSUB ERRLOG
#endif
    BEGIN CASE
    CASE RecordKey = 5000     ;* Check IOCTL SubCommand value
        SelectStatement = Record        ;* Selection statement passed in Record
        EXEC SQL PREPARE S2 FROM :SelectStatement;
        EXEC SQL DECLARE C CURSOR FOR S2;
        EXEC SQL OPEN C ;
        EXEC SQL WHENEVER NOT FOUND GOTO proglab_EXIT5000;
        Record=""
        LOOP
            EXEC SQL FETCH C INTO :RecordKey;
            RecordKey = TRIMB(RecordKey)
            Record := RecordKey:@AM
        REPEAT      ;* NOT FOUND will exit loop
EXIT5000:
        EXEC SQL CLOSE C;
    END CASE

    RETURN
!!
*
* DeleteFile routine
*
DeleteFileCmd:

#ifdef ORA_DEBUG
    ERRMSG = "DELETEFILE command" ; GOSUB ERRLOG
#endif

    IF INDEX(TableName,"]D",1) THEN
        EXEC SQL WHENEVER NOT FOUND CONTINUE;
        EXEC SQL WHENEVER SQLWARNING CONTINUE;
        EXEC SQL WHENEVER SQLERROR CONTINUE;
        EXEC SQL DROP TABLE ORATabName;
        EXEC SQL COMMIT;
        EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;
        EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;
        EXEC SQL WHENEVER SQLERROR GOTO proglab_ERROR;
    END

    RETURN
!!
*
* ClearFile routine
*
ClearFileCmd:

#ifdef ORA_DEBUG
    ERRMSG = "CLEARFILE command" ; GOSUB ERRLOG
#endif
    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    EXEC SQL WHENEVER SQLWARNING CONTINUE;
    EXEC SQL DELETE FROM ORATabName ;
    EXEC SQL COMMIT;
    EXEC SQL WHENEVER NOT FOUND GOTO proglab_NOTFOUND;
    EXEC SQL WHENEVER SQLWARNING GOTO proglab_WARNING;

    RETURN
!!
*
* CreateFile routine
*
CreateFileCmd:

#ifdef ORA_DEBUG
    ERRMSG = "CREATEFILE command" ; GOSUB ERRLOG
#endif

    IF INDEX(TableName,"]D",1) THEN
        EXEC SQL CREATE TABLE ORATabName ( ORAId VARCHAR(50) NOT NULL PRIMARY KEY, ORARecord VARCHAR(1500), ORARecordX VARCHAR(100), ORARecordY VARCHAR(100));
        EXEC SQL COMMIT;
    END

    RETURN
!!
*
* Sync routine
*
SyncCmd:

#ifdef ORA_DEBUG
    ERRMSG = "SYNC command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* Disconnect routine
*
DisconnectCmd:

#ifdef ORA_DEBUG
    ERRMSG = "DISCONNECT command" ; GOSUB ERRLOG
#endif

    EXEC SQL COMMIT RELEASE;

    RETURN
!!
*
* TransBegin routine
*
TransBeginCmd:

#ifdef ORA_DEBUG
    ERRMSG = "TRANSBEGIN command" ; GOSUB ERRLOG
#endif

    RETURN
!!
*
* TransCommit routine
*
TransCommitCmd:

#ifdef ORA_DEBUG
    ERRMSG = "TRANSCOMMIT command" ; GOSUB ERRLOG
#endif

    EXEC SQL COMMIT ;

    RETURN
!!
*
* TransRollBack routine
*
TransRollBackCmd:

#ifdef ORA_DEBUG
    ERRMSG = "TRANSROLLBACK command" ; GOSUB ERRLOG
#endif

    EXEC SQL ROLLBACK ;

    RETURN
!!
*
* NotFound routine
*
NOTFOUND:

    IF DispNotFnd THEN SQLERROR()

    ErrorCode = 100

#ifdef ORA_DEBUG
    ERRMSG = "NOTFOUND  :":ErrorCode; GOSUB ERRLOG
    ERRMSG = "UserArea  :":UserArea ; GOSUB ERRLOG
#endif

    RETURN
*
* Warning routine
*
WARNING:

    IF DispWarn THEN SQLERROR()

    ErrorCode = SYSTEM(0)

#ifdef ORA_DEBUG
    ERRMSG = "WARNING  :":ErrorCode ; GOSUB ERRLOG
    ERRMSG = "UserArea :":UserArea ; GOSUB ERRLOG
#endif

    ErrorCode = 5

    RETURN

*
* Error routine
*
ERROR:

    SQLSETRESULT()  ;* Set SYSTEM(0) with result code

    IF DispErr THEN SQLERROR()

    ErrorCode = SYSTEM(0)

#ifdef ORA_DEBUG
    ERRMSG = "ERROR    :":ErrorCode ; GOSUB ERRLOG
    ERRMSG = "UserArea :":UserArea ; GOSUB ERRLOG
#endif

    ErrorCode = 5

    RETURN
!!
*
* Error Log
*
ERRLOG:

#ifdef ORA_DEBUG

    IF DisplayOn THEN
        CRT SYSTEM(40):":":ERRMSG
    END

    IF LogOn THEN
        LOG=DATE() "D2":":":TIME() "MTS":":":SYSTEM(50):":":SYSTEM(40):":":ERRMSG
        OPENSEQ LOGNAME TO SEQFILE THEN
            WRITESEQ LOG APPEND TO SEQFILE ELSE NULL
            CLOSESEQ SEQFILE
        END
    END
#endif

    RETURN
