    PROGRAM WRESTTEST
**********************************************************************
*
* Copyright (C) 2016 Zumasys, Inc., All Rights Reserved
*
* Written by: Patrick Payne, Zumasys
* Date: 01/31/2017
* Description: this is a simple REST test
*
**********************************************************************
    INCLUDE WWW.INFO.h
    RESTVERSION = "1.1"
    JSTATUS = 'ok'
    JSTATUSMSG = ''
*
* Supplied function for setting content type.
*
    CALL WSETCONTENTTYPE("application/json")
*
*
* Set a custom header with supplied function.
*
    CALL WSETHEADER("restversion",RESTVERSION)
*
* Start building our json object, ugly way until objects.
*
    JSON = '{'
*
* Need to know all the variables
*
    CALL WGETINFO(VARS,CGI_VARS)
    CALL WGETINFO(VALS,CGI_VALS)
*
* Need to know all the headers
*
    CALL WGETINFO(HEADERVARS,CGI_HEADERVARS)
    CALL WGETINFO(HEADERVALS,CGI_HEADERVALS)
*
* Set our rest version, this will be replaced in the future with objects
*
    CALL WBUILDJSON(JSON,"RestVersion",RESTVERSION,RERR)
*
* WHO
*
    EXECUTE 'WHO' CAPTURING WRESULT
    CALL WBUILDJSON(JSON,'Who',WRESULT<1,1,1>,RERR)
*
* HOSTNAME
*
*EXECUTE 'hostname' CAPTURING HOSTNAME
*CALL WBUILDJSON(JSON,'hostname',HOSTNAME<1,1,1>,RERR)
*
* IFCONFIG
*
*EXECUTE 'ifconfig' CAPTURING RESULT
*GOSUB exportresult
*JSON:=', "ifconfig": ':R
*
* PWD
*
    OK = GETCWD(PWD)
    CALL WBUILDJSON(JSON,'pwd',PWD,RERR)
*
* PS
*
*EXECUTE 'ps' CAPTURING RESULT
*GOSUB exportresult
*JSON:=', "ps": ':R
*CALL WBUILDJSON(JSON,'ps',PSRESULT,RERR)
*
* JSHOW
*
    EXECUTE 'jshow -v WRESTTEST' CAPTURING RESULT
    GOSUB exportresult
    JSON := ', "wresttest": ':R
*
* Build out vars, we have to build a subobject and manually
* put it into our main json object.
* Right now the primary function of WBUILDJSON is to properly
* encode the variable section.
*
    JSONVARS = '{'
    FOR X = 1 TO DCOUNT(VARS, @AM)
* Lets test our WGETVAR routine
        VARNAME = VARS<X>
*
* WGETVAR is our customer supplied function to get variables
* usually sent by browser as form variables, either in the body
* or in the URL query string.
*
        CALL WGETVAR(THISVAR,VARNAME)
        CALL WBUILDJSON(JSONVARS,VARNAME,THISVAR,RERR)
    NEXT X
    JSONVARS := '}'
*
* Here is our messy add, todo is to adjust WBUILDJSON to be able
* to do this.
*
    JSON := ', "vars": ':JSONVARS
*
* Get all our cookies
*
    CALL WGETINFO(CVARS,CGI_CVARS)
    CALL WGETINFO(CVALS,CGI_CVALS)
    COOKIES = '{'
    NUM_COOKIES = DCOUNT(CVARS,@AM)
    FOR A = 1 TO NUM_COOKIES
        COOKIENAME = CVARS<A>
*
* This is our customer supplied function to get cookies.
*
        CALL WGETCOOKIE(COOKIEVALUE,COOKIENAME)
        CALL WBUILDJSON(COOKIES,COOKIENAME,COOKIEVALUE,RERR)
    NEXT A
    COOKIES := '}'
    JSON := ', "cookies": ':COOKIES
*
* Build out headers
*
    JSONHEADERS = '{'
    numheaders = DCOUNT(HEADERVARS, @AM)
    FOR X = 1 TO numheaders
        HEADERNAME = HEADERVARS<X>
*
* This is our customer supplied function to get headers.
*
        CALL WGETHEADER(THISVAR,HEADERNAME)
        CALL WBUILDJSON(JSONHEADERS,HEADERNAME,THISVAR,RERR)
    NEXT X
    JSONHEADERS := '}'
    JSON := ', "headers": ':JSONHEADERS
*
* This is where the raw body is put, this is used primarily for
* AJAX functions where the body should be a json string.
*
    CALL WGETBODY(BODY)
    CALL WBUILDJSON(JSON,'body',BODY,RERR)
*
* Set our test cookie
*
    COOKIE.NAME = 'wresttestcookie'
    COOKIE.VALUE = TIMEDATE()
    CONVERT ' :' TO '' IN COOKIE.VALUE
*
* Our function for setting a cookie.  Hides all the formatting requirements
* and correctly sets expiration date/time using PICK dates and times.
*
* CALL WSETCOOKIE(NAME,VALUE,EXPDATE,EXPTIME,DOMAIN,PATH,SECURE)
*
    CALL WSETCOOKIE(COOKIE.NAME,COOKIE.VALUE,DATE()+1,TIME()+300,'','','')
*
* Now lets process our command requests
*
* This section allows this REST request to perform some functions
*
* Format is
* ...api/wresttest/<key>/<command>/<opt1>/<opt2>/<opt3>/etc
*
* key = basically a password and stored in the config file
* command = a command to execute, leaving blank does the same as help
* opt# = options that the commands would use
*
* Remember that for us, key is the first REAL param, command is 2, opt# is 3 on
*
    CALL WGETPARAM(REQKEY,1)
    IF REQKEY = '' THEN GOTO end.of.program       ;* no key then ignore
*
* Function to get our options already broken out.
*
    CALL WGETCONFIG(APIKEY,'APIKEY')
    IF APIKEY = '' THEN
*
* The way I like to handle errors, see end of program right before stop
*
        JSTATUS = 'error'
        JSTATUSMSG = 'No api key setup in WDB.RESOURCE'
        GOTO end.of.program
    END
    IF OCONV(APIKEY,'MX') # OCONV(REQKEY,'MX') THEN
        JSTATUS = 'error'
        JSTATUSMSG = 'Invalid api key'
        GOTO end.of.program
    END
    GOSUB processrest
*
end.of.program:     *
*
* Build our default status/result
*
    CALL WBUILDJSON(JSON,'status',JSTATUS,RERR)
    CALL WBUILDJSON(JSON,'statusmsg',JSTATUSMSG,ERR)
*
* Close out our object
*
    JSON := '}'
*
* Send string json to the client. This can be done anytime and multiple times.
*
    CALL WSEND(JSON)
*
    STOP
*
processrest:        *
*
* Build new object to store rest responses
*
    RJSON = '{'
*
* Get our command
*
    CALL WGETPARAM(COMMAND,2)
    COMMAND = OCONV(COMMAND,'MCU')
*
* Respond in REST response our requested command, nice for debugging
*
    CALL WBUILDJSON(RJSON,'requestedcommand',COMMAND,RERR)
*
* Now process our commands
*
    BEGIN CASE
    CASE COMMAND = 'DEBUGON'
        CALL WGETPARAM(DEBUGLINE,3)
        IF DEBUGLINE = '' OR NOT(NUM(DEBUGLINE)) THEN
            JSTATUS = 'error'
            JSTATUSMSG = 'You must put in a debug option as next paramater'
        END ELSE
            JSTATUSMSG = 'Setting debug to on for ':DEBUGLINE
            CALL WSETCOOKIE('wrestapidebug',DEBUGLINE,DATE()+1,TIME()+300,'','',' ')
        END
    CASE COMMAND = 'DEBUGOFF'
        CALL WSETCOOKIE('wrestapidebug','',DATE()-1,TIME(),'','','')
    CASE COMMAND = 'ENV'
* Get environment variables
#ifdef WIN32
        EXECUTE @IM:'kset' CAPTURING ENVIRONMENT
#else
        EXECUTE 'env' CAPTURING ENVIRONMENT
#endif
*
* New object to store response
*
        J = '{'
        NUMENV = DCOUNT(ENVIRONMENT,@AM)
        FOR A = 1 TO NUMENV
            ENV = ENVIRONMENT<A>
            ENV.NAME = FIELD(ENV,'=',1)
            ENV.VALUE = FIELD(ENV,'=',2,9999)
            IF TRIM(ENV.NAME) # '' THEN
                CALL WBUILDJSON(J,ENV.NAME,ENV.VALUE,RERR)
            END
        NEXT A
        J := '}'
        RJSON := ', "environment": ':J
    CASE COMMAND = 'USERS'
        EXECUTE 'LISTU.JSON' CAPTURING USERS
        USERS = USERS<1,1,1>
        RJSON := ', "users": ':USERS
    CASE COMMAND = 'JSHOW'
        CALL WGETPARAM(OPTN,3)
        CMND = 'jshow -v ':OPTN
        EXECUTE CMND CAPTURING RESULT
        GOSUB exportresult
        RJSON := ', "jshow": ':R
    CASE COMMAND = 'JLICENSINGINFO'
        EXECUTE 'JLICENSINGINFO' CAPTURING RESULT
        GOSUB exportresult
        RJSON := ', "jlicensinginfo": ':R
    CASE 1          ;* DO HELP
        C = '{'
        CALL WBUILDJSON(C,'help','List all commands',RERR)
*CALL WBUILDJSON(C,'debugon','Turn debug on, next param is line no',RERR)
*CALL WBUILDJSON(C,'debugoff','Turn off debugging',RERR)
*CALL WBUILDJSON(C,'listweberrors','Lists web errors',RERR)
*CALL WBUILDJSON(C,'env','List Environment Variables',RERR)
*CALL WBUILDJSON(C,'users','List users',RERR)
        CALL WBUILDJSON(C,'jshow','Issue jshow, next param is program/file',RERR )
        CALL WBUILDJSON(C,'jlicensinginfo','Licensing information',RERR)
        C := '}'
        RJSON := ', "commands": ':C
    END CASE
    RJSON := '}'
    JSON := ', "rest": ':RJSON
    RETURN
*
exportresult:       *
    NUM.LINES = DCOUNT(RESULT,@AM)
    R= '[ '
    FOR XX = 1 TO NUM.LINES
        LINE = RESULT<XX>
        LINE = SWAP(LINE,CHAR(9),'     ')
        CALL WENCODEJSON('',LINE,OUTLINE)
        IF XX > 1 THEN R := ','
        R := '"':OUTLINE:'"'
    NEXT XX
    R := ']'
    RETURN
*
END

