#!/bin/ksh
set +x
#
#	jRFS: jBASE Remote File Server startup script
#
#	Copyright (c) 2013 T-jBASE SA. All rights reserved
#
#	All Rights Reserved.
#
# To start the jRFS server automatically on powerup then perform
# the following:
#  [ The references to '/etc' below should be replaced with '/sbin' for certain 
#    Unix operating systems. Notably HPUX. ]
# -	 Copy this script into "/etc/init.d/jRFSstart"
# -  Make a link to the script from the directory /etc/rc2.d, with a name of
#	 the form S<xx>jRFS.  The <xx> field should be replaced by two digits,
#	 for example 99, (signifies the order in which scripts are to be run).
#
#	 eg		ln /etc/init.d/jRFSstart /etc/rc2.d/S99jRFS
#
# WARNING: This startup script should only be executed as root and
# the following jBASE variables will assume default settings
# unless otherwise specified. Therefore ensure variables unset
# if default settings required.
#
#     JBCRELEASEDIR       - set to required release directory
#     JBCGLOBALDIR        - set to required global directory
#     JEDIFILENAME_SYSTEM - set to required SYSTEM file
#
if [ -z "$JBCRELEASEDIR" ]
then
	JBCRELEASEDIR="/opt/jbase5/5.2"
	if [ ! -d "$JBCRELEASEDIR" ]
	then
		JBCRELEASEDIR=/usr/jbc
	fi
	export JBCRELEASEDIR
fi

if [ -z "$JBCGLOBALDIR" ]
then
	JBCGLOBALDIR="$JBCRELEASEDIR"
	export JBCGLOBALDIR
fi

if [ -z "$JEDIFILENAME_SYSTEM" ]
then
	JEDIFILENAME_SYSTEM="$JBCGLOBALDIR/src/SYSTEM"
	export JEDIFILENAME_SYSTEM
fi

PATH=/bin:/usr/bin:$JBCRELEASEDIR/bin; export PATH
LD_LIBRARY_PATH=$JBCRELEASEDIR/lib; export LD_LIBRARY_PATH
SHLIB_PATH=$JBCRELEASEDIR/lib; export SHLIB_PATH
LIBPATH=$JBCRELEASEDIR/lib; export LIBPATH

USAGE="Usage: $0 {start | stop }"

ExecRFS=jRFS
StartOptionsRFS="-b"

PV_AWK=awk

case "$1" in
'start')
	echo "\nStarting jBASE Remote File Server $ExecRFS"
	$ExecRFS $StartOptionsRFS
	sleep 1
	;;

'stop')
	echo "Stopping $ExecRFS..."
	PV_PROCESSES=`ps -ef | grep -v grep | grep "jRFS -b" | $PV_AWK '{ print $2 }'`
	kill -HUP  $PV_PROCESSES > /dev/null 2>&1
	;;
*)
	echo $USAGE
	exit 1
	;;
esac
echo
