# -------------------- Profile.Ksh -----------------------------
#
# .profile template for korn shell to execute j shell
# after configuring the environment
# After creating a user and his home directory you
# should append this file to the .profile in ${HOME}
# with the command:  cat $JBCRELEASEDIR/src/profile.ksh >>.profile
# issued from their home directory, unless the IJU command has
# been run, in which case this is done for you.
#
#	        ######     #     #####  #######
#	      # #     #   # #   #     # #
#	      # #     #  #   #  #       #
#	      # ######  #     #  #####  #####
#	      # #     # #######       # #
#	 #    # #     # #     # #     # #
#	  ####  ######  #     #  #####  #######

# Set up the port, if it ever goes wild, then typing a single 'X'
# should hopefully put it back, using the sequence Ctrl-J (^J), X, Ctrl-J (^J)
# whereby Ctrl-J means new line.
#
# Certain control characters used in some system stty settings may clash with 
# control characters used within the JED/JSH programs, thus making the function
# appear not to work. In this case the clashing stty settings can be safely set
# to undefined thus allowing the control character to be processed by JED/JSH.
# e.g. Undefine stty 'flush' character. stty flush ^-, where ^ is uparrow and 
# - is the minus character. Alternatively JED can be configured to use alternative
# control characters via the .jedsrc file. See knowledge base.
#
stty intr ^C kill  icanon opost echo echoe echok onlcr -lcase tab3
#
# undefine common stty settings to avoid clash with some terminfo and jed controls
if [ `uname` != "Linux" ]
then
	stty quit ^- dsusp ^- susp ^-
else
	stty quit ^- susp ^-
fi

alias X="stty $(stty -g)"

#
# Set the default umask to 002. This allows the CREATE-FILE to create files with
# default permissions of 664. i.e. rw-rw-r--. 
# Read/Write User, Read/Write Group, ReadOnly Others
# This enables file access permissions to be more easily controlled via user groups.
# Also in the case of network file access, unix locks required read/write access.
# 
umask 002

#
# Configure default prompt
#
PS1="$(uname)-\$PWD: "

#
# Configure ENV for environment
ENV=${HOME}/.env
#
# default to vi editor and vi options and cd to home
VISUAL=/usr/bin/vi
set -o vi
cd

#
# Set up the directory of where jBASE is being run from.
# The JBCRELEASEDIR shows where the executables, libraries, scripts etc.
# can be found for the particular release you want to run. The 
# JBCGLOBALDIR shows where a few of the global constants can be
# found, such as the configuration file for the record locking
# mechanism -- these should be common to all releases running on
# the same system
#
# If undefined, they will resort to /opt/jbase4/4.1 as a default. However, they
# are initialised in this script to the default, so that if you move
# to another test release at a later stage, it makes it very easy to
# update the JBCRELEASEDIR variable to say /opt/jbase4/testjbc and all the
# other variables fall in to place.
#
LEAVE_FOR_IJU_TO_UPDATE
export JBCRELEASEDIR JBCGLOBALDIR

#
#    Set up the shared object file name where we will resolve
#    all the calls to subroutines made via. the CALL @Var() statements.
#    The default is $HOME/lib. If you want to use shared objects other
#    than in the default, set up a : delimited path of directory names
#    and/or object name to search in, by setting the JBCOBJECTLIST variable.
#
#    NOTE: When executing programs the PATH environment variable is used.
#    When locating subroutines the JBCOBJECTLIST environment variable is used
#    else by default the $HOME/lib directory. Therefore when cataloging ensure
# 	 that either the default $HOME/bin or $HOME/lib directories areused to store
#    the executables and shared libraries or else the environment variables
#	 JBCDEV_BIN and JBCDEV_LIB are configured to match PATH and JBCOBJECTLIST.
#
#	 e.g. If JBCOBJECTLIST were configured below then the directory /apps/myapp/lib
#	 would be used to locate any subroutines called by my main program. In this case
#	 the subroutines should of been cataloged with JBCDEV_LIB=/app1/myapp/lib also.
#
#JBCOBJECTLIST=/apps/myapp/lib 
#export JBCOBJECTLIST
#
#    During the search for a shared object, if an object is specified
#    without a path component, e.g. "libsubroutines.so" , then AIX
#    will use LIBPATH to find out what directory the object is in,
#    and SVR4 systems will use LD_LIBRARY_PATH and HPUX the SHLIB_PATH.
#
LD_LIBRARY_PATH=$JBCRELEASEDIR/lib:${LD_LIBRARY_PATH:-/usr/ccs/lib:/usr/lib}
LIBPATH=$JBCRELEASEDIR/lib:${LIBPATH:-/usr/ccs/lib:/usr/lib}
SHLIB_PATH=$JBCRELEASEDIR/lib:${SHLIB_PATH:-/usr/lib:/lib}
export LD_LIBRARY_PATH LIBPATH SHLIB_PATH

#
# Set up path of where to find data files. By default if the JEDIFILEPATH
# is not set then files will be looked for first in the $HOME directory and
# if not found then in the '.' current directory.
#
#JEDIFILEPATH=.
#export JEDIFILEPATH 

#
# Set up base directory where we keep the spooler (default == /usr/jspooler)
#
# JBCSPOOLERDIR=/usr/jspooler
# export JBCSPOOLERDIR

#
# Set up where the MD and SYSTEM files can be found.
# By default, the MD and SYSTEM files are undefined, so Q pointers
# and other considerations (such as PQ procs from jsh) will not work.
JEDIFILENAME_MD=${HOME}/MD
JEDIFILENAME_SYSTEM=$JBCRELEASEDIR/src/SYSTEM
export JEDIFILENAME_MD JEDIFILENAME_SYSTEM

#
# Setup the Unix PATH environment variable. This variable specifies the location and 
# order in which directories will be search to find the command line executable.
#
PATH=$HOME/bin:$JBCRELEASEDIR/bin:$PATH:.
export PATH

# Default pager

PAGER=more
export PAGER

# You can now start to add any jBASE programs in your .profile at this stage.
# Some examples you may want to insert are shown below, but commented out.
#
# TERM 80,10,,,,132,60
# SP-ASSIGN =PRINTRONIX
#
#--------------- End of Profile.Ksh -----------------
