
#include <jsystem.h>

EXEC SQL INCLUDE SQLCA;

/*****************************************************************************
 *
 * Copyright (c) 2019 jBASE International. All rights reserved
 * 
 * Include file for jBASE embedded SQL.
 *
 * This header contains some small support functions, for jBASE embedded
 * SQL.  This file is automatically included when required.
 *
 * SQL_SETRESULT()
 *	Called automatically after each embedded SQL statement to Assign the
 *	result to SYSTEM(0).  This can then be tested from within jBASE 
 *	to get the status of each statement, eg
 *
 *		EXEC SQL connect :DataBase;
 *		IF SYSTEM(0) THEN CRT "Connect failed"; STOP
 *
 * SQL_ERROR()
 *	Prints the SQL error, this is only called if set to do so (see below)
 *
 * SQL_ABORT()
 *	Prints the SQL error and Exits (Result of 1).
 *
 * There is no default action on error, but this can be configured by
 * including the following embedded SQL statements in your source code.
 *
 * ingres *	exec sql whenever sqlerror call SQL_ERROR;
 * oracle *	exec sql whenever sqlerror do SQL_ERROR();
 *****************************************************************************/
/*****************************************************************************
*
* Function:             SQL_SETRESULT()
*
* Description:  Assign SYSTEM(0) to the result of the last SQL statement.
*
* Parameters:   None
*
* Return Value: Always 0.
*
* Notes:                None.
*
*******************************************************************************/
int   SQL_SETRESULT()
{
        VAR             scratchvar ;

        STRING_INITIALISE_USER_VB(&scratchvar);
        STORE_VBI(&scratchvar , sqlca.sqlcode);
        ASSIGNSYS_VBI(&scratchvar, 0);
        return(0);
}

/*****************************************************************************
*
* Function:             SQL_ERROR()
*
* Description:  Display The SQL Error
*
* Parameters:   None
*
* Return Value: None
*
* Notes:                None.
*
*******************************************************************************/
#define SQL_ERROR SQLERROR
int   SQLERROR()
{
	fprintf (stderr,"\nError: sqlcode : %d\n",sqlca.sqlcode);

	fprintf (stderr,"Error: sqlerrmc: %s\n", sqlca.sqlerrm.sqlerrmc);

	return(0);
}

/*****************************************************************************
*
* Function:             SQL_ABORT()
*
* Description:  Terminate with the SQL error message
*
* Parameters:   None
*
* Return Value: None
*
* Notes:                None.
*
*******************************************************************************/
#define SQL_ABORT SQLABORT
int   SQLABORT()
{
        SQL_ERROR();
        EXIT_VI(1);
}

/*****************************************************************************
*
* Function:             SQLSETRESULT()
*
* Description:  Update SYSTEM(0) with SQLCODE call from BASIC statement.
*
* Parameters:   None
*
* Return Value: None
*
* Notes:                None.
*
*******************************************************************************/
int   SQLSETRESULT()
{
	return(SQL_SETRESULT());
}
